(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.1'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     48820,       1319]*)
(*NotebookOutlinePosition[     49450,       1341]*)
(*  CellTagsIndexPosition[     49406,       1337]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[BoxData[
    \(<< MB.m\)], "Input"],

Cell[BoxData[
    \("MB 1.1"\)], "Print"],

Cell[BoxData[
    \("by Michal Czakon"\)], "Print"],

Cell[BoxData[
    \("more info in hep-ph/0511200"\)], "Print"],

Cell[BoxData[
    \("last modified 12 Dec 06"\)], "Print"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(<< AMBRE.m\)], "Input"],

Cell[BoxData[
    \("by K.Kajda   ver: 1.0"\)], "Print"],

Cell[BoxData[
    \("last modified 19 Apr 2007"\)], "Print"]
}, Open  ]],

Cell[BoxData[
    \( (*\ \[IndentingNewLine]Pentabox, \ 
      Fig  .7\ \[IndentingNewLine]we\ use\ kinematics, \ Eq  .38, 
      and\ perform\ calculations\ for\ two\ different\ orders\ of\ \
\(integrations : \[IndentingNewLine]\t\(example : \ 
            ONE\  - \ {k2, k1}\[IndentingNewLine]
          \ 
          example : \ TWO\  - \ {k1, k2}\)\)\[IndentingNewLine]*) \)], "Input"],

Cell[BoxData[
    \( (**************************************************************) \)], \
"Input"],

Cell[BoxData[
    \( (*\ example : \ ONE\ *) \)], "Input"],

Cell[BoxData[
    \(\(Fullintegral[{1}, {PR[k1, 0, n5] PR[k1 - k2, 0, n2] 
            PR[k1 + p1 + p2, 0, n8] PR[k1 - p4 - p5, 0, n7] 
            PR[k1 - p5, 0, n6] PR[k2, 0, n1] PR[k2 + p1, 0, n4] 
            PR[k2 + p1 + p2, 0, n3]}, {k2, k1}];\)\)], "Input"],

Cell[BoxData[
    \(\(invariants = {p1^2 \[Rule] 0, p2^2 \[Rule] 0, p3^2 \[Rule] 0, 
          p4^2 \[Rule] 0, p5^2 \[Rule] 0, p1*p2 \[Rule] s12/2, 
          p2*p3 \[Rule] s23/2, p3*p4 \[Rule] s34/2, p4*p5 \[Rule] s45/2, 
          p1*p5 \[Rule] s15/2, p1*p3 \[Rule] \((\(-s12\) - s23 + s45)\)/2, 
          p1*p4 \[Rule] \((\(-s15\) + s23 - s45)\)/2, 
          p2*p4 \[Rule] \((s15 - s23 - s34)\)/2, 
          p2*p5 \[Rule] \((\(-s12\) - s15 + s34)\)/2, 
          p3*p5 \[Rule] \((s12 - s34 - s45)\)/2};\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(IntPart[1]\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("numerator="\[InvisibleSpace]1\),
      SequenceForm[ "numerator=", 1],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("integral="\[InvisibleSpace]\(PR[k1 - k2, 0, n2]\ PR[
            k2, 0, n1]\ PR[k2 + p1, 0, n4]\ PR[k2 + p1 + p2, 0, n3]\)\),
      SequenceForm[ "integral=", 
        Times[ 
          PR[ 
            Plus[ k1, 
              Times[ -1, k2]], 0, n2], 
          PR[ k2, 0, n1], 
          PR[ 
            Plus[ k2, p1], 0, n4], 
          PR[ 
            Plus[ k2, p1, p2], 0, n3]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("momentum="\[InvisibleSpace]k2\),
      SequenceForm[ "momentum=", k2],
      Editable->False]], "Print"],

Cell[BoxData[
    \(Fauto::"mode" \(\(:\)\(\ \)\) 
      "U and F polynomials will be calculated in AUTO mode. In order to use \
MANUAL mode execute Fauto[0]."\)], "Message"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(SubLoop[integral]\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("Iteration nr"\[InvisibleSpace]1\[InvisibleSpace]": \
>>Integrating over "\[InvisibleSpace]k2\[InvisibleSpace]"<<"\),
      SequenceForm[ "Iteration nr", 1, ": >>Integrating over ", k2, "<<"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Computing U & F polynomial in AUTO mode >>Fauto["\
\[InvisibleSpace]1\[InvisibleSpace]"]<<"\),
      SequenceForm[ 
      "Computing U & F polynomial in AUTO mode >>Fauto[", 1, "]<<"],
      Editable->False]], "Print"],

Cell[BoxData[
    \("U polynomial..."\)], "Print"],

Cell[BoxData[
    \(X[1] + X[2] + X[3] + X[4]\)], "Print"],

Cell[BoxData[
    \("F polynomial..."\)], "Print"],

Cell[BoxData[
    \(\(-PR[k1, 0]\)\ X[1]\ X[2] - PR[k1 + p1, 0]\ X[1]\ X[3] - 
      PR[k1 + p1 + p2, 0]\ X[1]\ X[4] - s12\ X[2]\ X[4]\)], "Print"],

Cell[BoxData[
    InterpretationBox[\("Representation after integrating over: "\
\[InvisibleSpace]"k2..."\),
      SequenceForm[ "Representation after integrating over: ", "k2..."],
      Editable->False]], "Print"],

Cell[BoxData[
    RowBox[{"SubLoop1", "[", 
      RowBox[{
        TagBox[
          
          StyleBox[\(\((\((\(-1\))\)\^\(n1 + n2 + n3 + n4 + z1 + z2 + z3\)\ \
\((\(-s12\))\)\^\(2 - eps - n1 - n2 - n3 - n4 - z1 - z2 - z3\)\ \
Gamma[\(-z1\)]\ Gamma[
                    2 - eps - n1 - n2 - n4 - z1 - z2]\ Gamma[\(-z2\)]\ Gamma[
                    n4 + z2]\ Gamma[
                    2 - eps - n2 - n3 - n4 - z2 - z3]\ Gamma[\(-z3\)]\ Gamma[
                    n2 + z1 + z2 + z3]\ Gamma[\(-2\) + eps + n1 + n2 + n3 + 
                      n4 + z1 + z2 + z3])\)/\((Gamma[n1]\ Gamma[n2]\ Gamma[
                    n3]\ Gamma[4 - 2\ eps - n1 - n2 - n3 - n4]\ Gamma[
                    n4])\)\),
            FontColor->RGBColor[0, 0, 1]],
          (StyleForm[ #, FontColor -> RGBColor[ 0, 0, 1]]&)], ",", 
        TagBox[
          
          StyleBox[\(PR[k1, 0, z1]\ PR[k1 + p1, 0, z2]\ PR[k1 + p1 + p2, 0, 
                z3]\),
            FontColor->RGBColor[1, 0, 0]],
          (StyleForm[ #, FontColor -> RGBColor[ 1, 0, 0]]&)]}], 
      "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(IntPart[2]\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("numerator="\[InvisibleSpace]1\),
      SequenceForm[ "numerator=", 1],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("integral="\[InvisibleSpace]\(PR[k1, 0, n5 - z1]\ PR[
            k1 + p1, 0, \(-z2\)]\ PR[k1 + p1 + p2, 0, n8 - z3]\ PR[k1 - p5, 
            0, n6]\ PR[k1 - p4 - p5, 0, n7]\)\),
      SequenceForm[ "integral=", 
        Times[ 
          PR[ k1, 0, 
            Plus[ n5, 
              Times[ -1, z1]]], 
          PR[ 
            Plus[ k1, p1], 0, 
            Times[ -1, z2]], 
          PR[ 
            Plus[ k1, p1, p2], 0, 
            Plus[ n8, 
              Times[ -1, z3]]], 
          PR[ 
            Plus[ k1, 
              Times[ -1, p5]], 0, n6], 
          PR[ 
            Plus[ k1, 
              Times[ -1, p4], 
              Times[ -1, p5]], 0, n7]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("momentum="\[InvisibleSpace]k1\),
      SequenceForm[ "momentum=", k1],
      Editable->False]], "Print"],

Cell[BoxData[
    \(Fauto::"mode" \(\(:\)\(\ \)\) 
      "U and F polynomials will be calculated in AUTO mode. In order to use \
MANUAL mode execute Fauto[0]."\)], "Message"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(repr = SubLoop[integral];\)\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("Iteration nr"\[InvisibleSpace]2\[InvisibleSpace]": \
>>Integrating over "\[InvisibleSpace]k1\[InvisibleSpace]"<<"\),
      SequenceForm[ "Iteration nr", 2, ": >>Integrating over ", k1, "<<"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Computing U & F polynomial in AUTO mode >>Fauto["\
\[InvisibleSpace]1\[InvisibleSpace]"]<<"\),
      SequenceForm[ 
      "Computing U & F polynomial in AUTO mode >>Fauto[", 1, "]<<"],
      Editable->False]], "Print"],

Cell[BoxData[
    \("U polynomial..."\)], "Print"],

Cell[BoxData[
    \(X[1] + X[2] + X[3] + X[4] + X[5]\)], "Print"],

Cell[BoxData[
    \("F polynomial..."\)], "Print"],

Cell[BoxData[
    \(\(-s12\)\ X[1]\ X[3] - s15\ X[2]\ X[4] - s34\ X[3]\ X[4] - 
      s45\ X[1]\ X[5] - s23\ X[2]\ X[5]\)], "Print"],

Cell[BoxData[
    \("Final representation:"\)], "Print"],

Cell[BoxData[
    TagBox[
      StyleBox[\(\((\((\(-1\))\)\^\(n1 + n2 + n3 + n4 + n5 + n6 + n7 + n8\)\ \
\((\(-s12\))\)\^\(2 - eps - n1 - n2 - n3 - n4 - z1 - z2 - z3 + z4\)\ \
\((\(-s15\))\)\^z5\ \((\(-s23\))\)\^\(2 - eps - n5 - n6 - n7 - n8 + z1 + z2 + \
z3 - z4 - z5 - z6 - z7\)\ \((\(-s34\))\)\^z6\ \((\(-s45\))\)\^z7\ Gamma[\(-z1\
\)]\ Gamma[2 - eps - n1 - n2 - n4 - z1 - z2]\ Gamma[n4 + z2]\ Gamma[
                2 - eps - n2 - n3 - n4 - z2 - z3]\ Gamma[\(-z3\)]\ Gamma[
                n2 + z1 + z2 + z3]\ Gamma[\(-2\) + eps + n1 + n2 + n3 + n4 + 
                  z1 + z2 + z3]\ Gamma[\(-z4\)]\ Gamma[\(-z5\)]\ Gamma[
                2 - eps - n5 - n6 - n8 + z1 + z2 + z3 - z4 - z5 - 
                  z6]\ Gamma[\(-z6\)]\ Gamma[n8 - z3 + z4 + z6]\ Gamma[
                n6 + z5 + z6]\ Gamma[
                2 - eps - n5 - n6 - n7 - n8 + z1 + z3 - z4 - z6 - 
                  z7]\ Gamma[\(-z7\)]\ Gamma[
                n5 - z1 + z4 + z7]\ Gamma[\(-2\) + eps + n5 + n6 + n7 + n8 - 
                  z1 - z2 - z3 + z4 + z5 + z6 + z7])\)/\((Gamma[n1]\ Gamma[
                n2]\ Gamma[n3]\ Gamma[4 - 2\ eps - n1 - n2 - n3 - n4]\ Gamma[
                n4]\ Gamma[n6]\ Gamma[n7]\ Gamma[n5 - z1]\ Gamma[
                n8 - z3]\ Gamma[
                4 - 2\ eps - n5 - n6 - n7 - n8 + z1 + z2 + z3])\)\),
        FontColor->RGBColor[0, 0, 1]],
      (StyleForm[ #, FontColor -> RGBColor[ 0, 0, 1]]&)]], "Print"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(BarnesLemma[repr, 1]\)], "Input"],

Cell[BoxData[
    InterpretationBox[\(">> Barnes 1st Lemma will be checked for: "\
\[InvisibleSpace]{}\[InvisibleSpace]" <<"\),
      SequenceForm[ ">> Barnes 1st Lemma will be checked for: ", {}, " <<"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("   Starting with dim="\[InvisibleSpace]7\
\[InvisibleSpace]" representation..."\),
      SequenceForm[ "   Starting with dim=", 7, " representation..."],
      Editable->False]], "Print"],

Cell[BoxData[
    \(">> Representation after 1st Barnes Lemma:  <<"\)], "Print"],

Cell[BoxData[
    TagBox[
      StyleBox["\<\"   Could not apply Barnes-Lemma\"\>",
        FontColor->RGBColor[1, 0, 0]],
      (StyleForm[ #, FontColor -> RGBColor[ 1, 0, 0]]&)]], "Print"],

Cell[BoxData[
    \(\((\((\(-1\))\)\^\(n1 + n2 + n3 + n4 + n5 + n6 + n7 + n8\)\ \
\((\(-s12\))\)\^\(2 - eps - n1 - n2 - n3 - n4 - z1 - z2 - z3 + z4\)\ \
\((\(-s15\))\)\^z5\ \((\(-s23\))\)\^\(2 - eps - n5 - n6 - n7 - n8 + z1 + z2 + \
z3 - z4 - z5 - z6 - z7\)\ \((\(-s34\))\)\^z6\ \((\(-s45\))\)\^z7\ Gamma[\(-z1\
\)]\ Gamma[2 - eps - n1 - n2 - n4 - z1 - z2]\ Gamma[n4 + z2]\ Gamma[
            2 - eps - n2 - n3 - n4 - z2 - z3]\ Gamma[\(-z3\)]\ Gamma[
            n2 + z1 + z2 + z3]\ Gamma[\(-2\) + eps + n1 + n2 + n3 + n4 + z1 + 
              z2 + z3]\ Gamma[\(-z4\)]\ Gamma[\(-z5\)]\ Gamma[
            2 - eps - n5 - n6 - n8 + z1 + z2 + z3 - z4 - z5 - 
              z6]\ Gamma[\(-z6\)]\ Gamma[n8 - z3 + z4 + z6]\ Gamma[
            n6 + z5 + z6]\ Gamma[
            2 - eps - n5 - n6 - n7 - n8 + z1 + z3 - z4 - z6 - 
              z7]\ Gamma[\(-z7\)]\ Gamma[
            n5 - z1 + z4 + z7]\ Gamma[\(-2\) + eps + n5 + n6 + n7 + n8 - z1 - 
              z2 - z3 + z4 + z5 + z6 + z7])\)/\((Gamma[n1]\ Gamma[n2]\ Gamma[
            n3]\ Gamma[4 - 2\ eps - n1 - n2 - n3 - n4]\ Gamma[n4]\ Gamma[
            n6]\ Gamma[n7]\ Gamma[n5 - z1]\ Gamma[n8 - z3]\ Gamma[
            4 - 2\ eps - n5 - n6 - n7 - n8 + z1 + z2 + z3])\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(repr = %\  /. \ {n1 \[Rule] 1, n2 \[Rule] 1, n3 \[Rule] 1, 
          n4 \[Rule] 1, n5 \[Rule] 1, n6 \[Rule] 1, n7 \[Rule] 1, 
          n8 \[Rule] 1}\)], "Input"],

Cell[BoxData[
    \(\((\((\(-s12\))\)\^\(\(-2\) - eps - z1 - z2 - z3 + z4\)\ \((\(-s15\))\)\
\^z5\ \((\(-s23\))\)\^\(\(-2\) - eps + z1 + z2 + z3 - z4 - z5 - z6 - z7\)\ \
\((\(-s34\))\)\^z6\ \((\(-s45\))\)\^z7\ Gamma[\(-z1\)]\ Gamma[\(-1\) - eps - 
              z1 - z2]\ Gamma[
            1 + z2]\ Gamma[\(-1\) - eps - z2 - z3]\ Gamma[\(-z3\)]\ Gamma[
            1 + z1 + z2 + z3]\ Gamma[
            2 + eps + z1 + z2 + 
              z3]\ Gamma[\(-z4\)]\ Gamma[\(-z5\)]\ Gamma[\(-1\) - eps + z1 + 
              z2 + z3 - z4 - z5 - z6]\ Gamma[\(-z6\)]\ Gamma[
            1 - z3 + z4 + z6]\ Gamma[
            1 + z5 + z6]\ Gamma[\(-2\) - eps + z1 + z3 - z4 - z6 - 
              z7]\ Gamma[\(-z7\)]\ Gamma[1 - z1 + z4 + z7]\ Gamma[
            2 + eps - z1 - z2 - z3 + z4 + z5 + z6 + 
              z7])\)/\((Gamma[\(-2\)\ eps]\ Gamma[1 - z1]\ Gamma[
            1 - z3]\ Gamma[\(-2\)\ eps + z1 + z2 + z3])\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(rules = MBoptimizedRules[repr, eps \[Rule] 0, {}, {eps}]\)], "Input"],

Cell[BoxData[
    \(MBresidues::"contour" \(\(:\)\(\ \)\) 
      "contour starts and/or ends on a pole of Gamma[\!\(\(\(-2\)\) - \(\(2\\ \
eps\)\) - z2 - z6\)]"\)], "Message"],

Cell[BoxData[
    \(MBrules::"norules" \(\(:\)\(\ \)\) 
      "no rules could be found to regulate this integral"\)], "Message"],

Cell[BoxData[
    \(MBrules::"norules" \(\(:\)\(\ \)\) 
      "no rules could be found to regulate this integral"\)], "Message"],

Cell[BoxData[
    \(MBrules::"norules" \(\(:\)\(\ \)\) 
      "no rules could be found to regulate this integral"\)], "Message"],

Cell[BoxData[
    RowBox[{\(General::"stop"\), \(\(:\)\(\ \)\), "\<\"Further output of \
\\!\\(MBrules :: \\\"norules\\\"\\) will be suppressed during this \
calculation. \\!\\(\\*ButtonBox[\\\"More\[Ellipsis]\\\", \
ButtonStyle->\\\"RefGuideLinkText\\\", ButtonFrame->None, \
ButtonData:>\\\"General::stop\\\"]\\)\"\>"}]], "Message"],

Cell[BoxData[
    \({{eps \[Rule] \(-\(45\/64\)\)}, {z1 \[Rule] \(-\(1\/4\)\), 
        z2 \[Rule] \(-\(17\/32\)\), z3 \[Rule] \(-\(1\/8\)\), 
        z4 \[Rule] \(-\(153\/256\)\), z5 \[Rule] \(-\(25\/64\)\), 
        z6 \[Rule] \(-\(33\/64\)\), 
        z7 \[Rule] \(-\(161\/256\)\)}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(integrals = 
        MBcontinue[repr, eps \[Rule] 0, rules, 
          Verbose \[Rule] False];\)\)], "Input"],

Cell[BoxData[
    RowBox[{\(General::"spell1"\), \(\(:\)\(\ \)\), "\<\"Possible spelling \
error: new symbol name \\\"\\!\\(integrals\\)\\\" is similar to existing \
symbol \\\"\\!\\(integral\\)\\\". \\!\\(\\*ButtonBox[\\\"More\[Ellipsis]\\\", \
ButtonStyle->\\\"RefGuideLinkText\\\", ButtonFrame->None, \
ButtonData:>\\\"General::spell1\\\"]\\)\"\>"}]], "Message"]
}, Open  ]],

Cell[BoxData[
    \(\(ser = 
        MBexpand[{integrals}, 
          Exp[2\ eps\ EulerGamma], {eps, 0, \(-1\)}];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(AbsoluteTiming[
      MBintegrate[
        ser, {s12 \[Rule] \(-2\), s15 \[Rule] \(-1\)/2, s23 \[Rule] \(-4\), 
          s34 \[Rule] \(-5\), s45 \[Rule] \(-6\)}, 
        Verbose \[Rule] False]]\)], "Input"],

Cell[BoxData[
    \({30.792086`7.9399841045363075\ Second, {0.159375`\/eps\^4 + 
          0.3193080330282121`\/eps\^3 + 0.42274543880473414`\/eps\^2 + 
          0.264480823446921`\/eps, {0.00006874560279338051`\/eps, 
          0}}}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \( (*\ example : \ TWO\ *) \)], "Input"],

Cell[BoxData[
    \(\(Fullintegral[{1}, {PR[k1, 0, n5] PR[k1 - k2, 0, n2] 
            PR[k1 + p1 + p2, 0, n8] PR[k1 - p4 - p5, 0, n7] 
            PR[k1 - p5, 0, n6] PR[k2, 0, n1] PR[k2 + p1, 0, n4] 
            PR[k2 + p1 + p2, 0, n3]}, {k1, k2}];\)\)], "Input"],

Cell[BoxData[
    \(\(invariants = {p1^2 \[Rule] 0, p2^2 \[Rule] 0, p3^2 \[Rule] 0, 
          p4^2 \[Rule] 0, p5^2 \[Rule] 0, p1*p2 \[Rule] s12/2, 
          p2*p3 \[Rule] s23/2, p3*p4 \[Rule] s34/2, p4*p5 \[Rule] s45/2, 
          p1*p5 \[Rule] s15/2, p1*p3 \[Rule] \((\(-s12\) - s23 + s45)\)/2, 
          p1*p4 \[Rule] \((\(-s15\) + s23 - s45)\)/2, 
          p2*p4 \[Rule] \((s15 - s23 - s34)\)/2, 
          p2*p5 \[Rule] \((\(-s12\) - s15 + s34)\)/2, 
          p3*p5 \[Rule] \((s12 - s34 - s45)\)/2};\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(IntPart[1]\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("numerator="\[InvisibleSpace]1\),
      SequenceForm[ "numerator=", 1],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("integral="\[InvisibleSpace]\(PR[k1, 0, n5]\ PR[
            k1 - k2, 0, n2]\ PR[k1 + p1 + p2, 0, n8]\ PR[k1 - p5, 0, n6]\ PR[
            k1 - p4 - p5, 0, n7]\)\),
      SequenceForm[ "integral=", 
        Times[ 
          PR[ k1, 0, n5], 
          PR[ 
            Plus[ k1, 
              Times[ -1, k2]], 0, n2], 
          PR[ 
            Plus[ k1, p1, p2], 0, n8], 
          PR[ 
            Plus[ k1, 
              Times[ -1, p5]], 0, n6], 
          PR[ 
            Plus[ k1, 
              Times[ -1, p4], 
              Times[ -1, p5]], 0, n7]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("momentum="\[InvisibleSpace]k1\),
      SequenceForm[ "momentum=", k1],
      Editable->False]], "Print"],

Cell[BoxData[
    \(Fauto::"mode" \(\(:\)\(\ \)\) 
      "U and F polynomials will be calculated in AUTO mode. In order to use \
MANUAL mode execute Fauto[0]."\)], "Message"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(SubLoop[integral]\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("Iteration nr"\[InvisibleSpace]1\[InvisibleSpace]": \
>>Integrating over "\[InvisibleSpace]k1\[InvisibleSpace]"<<"\),
      SequenceForm[ "Iteration nr", 1, ": >>Integrating over ", k1, "<<"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Computing U & F polynomial in AUTO mode >>Fauto["\
\[InvisibleSpace]1\[InvisibleSpace]"]<<"\),
      SequenceForm[ 
      "Computing U & F polynomial in AUTO mode >>Fauto[", 1, "]<<"],
      Editable->False]], "Print"],

Cell[BoxData[
    \("U polynomial..."\)], "Print"],

Cell[BoxData[
    \(X[1] + X[2] + X[3] + X[4] + X[5]\)], "Print"],

Cell[BoxData[
    \("F polynomial..."\)], "Print"],

Cell[BoxData[
    \(\(-PR[k2, 0]\)\ X[1]\ X[2] - s12\ X[1]\ X[3] - 
      PR[k2 + p1 + p2, 0]\ X[2]\ X[3] - PR[k2 - p5, 0]\ X[2]\ X[4] - 
      s34\ X[3]\ X[4] - s45\ X[1]\ X[5] - 
      PR[\(-k2\) + p4 + p5, 0]\ X[2]\ X[5]\)], "Print"],

Cell[BoxData[
    InterpretationBox[\("Representation after integrating over: "\
\[InvisibleSpace]"k1..."\),
      SequenceForm[ "Representation after integrating over: ", "k1..."],
      Editable->False]], "Print"],

Cell[BoxData[
    RowBox[{"SubLoop1", "[", 
      RowBox[{
        TagBox[
          
          StyleBox[\(\((\((\(-1\))\)\^\(2 - eps - z2 - z5 - z6\)\ \
\((\(-s12\))\)\^z2\ \((\(-s34\))\)\^z5\ \((\(-s45\))\)\^z6\ Gamma[\(-z1\)]\ \
Gamma[\(-z2\)]\ Gamma[\(-z3\)]\ Gamma[\(-z4\)]\ Gamma[
                    2 - eps - n2 - n5 - n6 - n8 - z1 - z2 - z3 - z4 - 
                      z5]\ Gamma[\(-z5\)]\ Gamma[n8 + z2 + z3 + z5]\ Gamma[
                    n6 + z4 + z5]\ Gamma[
                    2 - eps - n5 - n6 - n7 - n8 - z2 - z5 - 
                      z6]\ Gamma[\(-z6\)]\ Gamma[
                    n5 + z1 + z2 + z6]\ Gamma[\(-2\) + eps + n2 + n5 + n6 + 
                      n7 + n8 + z1 + z2 + z3 + z4 + z5 + z6])\)/\((Gamma[
                    n2]\ Gamma[n5]\ Gamma[n6]\ Gamma[n7]\ Gamma[
                    4 - 2\ eps - n2 - n5 - n6 - n7 - n8]\ Gamma[n8])\)\),
            FontColor->RGBColor[0, 0, 1]],
          (StyleForm[ #, FontColor -> RGBColor[ 0, 0, 1]]&)], ",", 
        TagBox[
          
          StyleBox[\(PR[k2, 0, z1]\ PR[k2 + p1 + p2, 0, z3]\ PR[k2 - p5, 0, 
                z4]\ PR[\(-k2\) + p4 + p5, 0, 
                2 - eps - n2 - n5 - n6 - n7 - n8 - z1 - z2 - z3 - z4 - z5 - 
                  z6]\),
            FontColor->RGBColor[1, 0, 0]],
          (StyleForm[ #, FontColor -> RGBColor[ 1, 0, 0]]&)]}], 
      "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(IntPart[2]\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("numerator="\[InvisibleSpace]1\),
      SequenceForm[ "numerator=", 1],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("integral="\[InvisibleSpace]\(PR[k2, 0, n1 - z1]\ PR[
            k2 + p1, 0, n4]\ PR[k2 + p1 + p2, 0, n3 - z3]\ PR[k2 - p5, 
            0, \(-z4\)]\ PR[\(-k2\) + p4 + p5, 
            0, \(-2\) + eps + n2 + n5 + n6 + n7 + n8 + z1 + z2 + z3 + z4 + 
              z5 + z6]\)\),
      SequenceForm[ "integral=", 
        Times[ 
          PR[ k2, 0, 
            Plus[ n1, 
              Times[ -1, z1]]], 
          PR[ 
            Plus[ k2, p1], 0, n4], 
          PR[ 
            Plus[ k2, p1, p2], 0, 
            Plus[ n3, 
              Times[ -1, z3]]], 
          PR[ 
            Plus[ k2, 
              Times[ -1, p5]], 0, 
            Times[ -1, z4]], 
          PR[ 
            Plus[ 
              Times[ -1, k2], p4, p5], 0, 
            Plus[ -2, eps, n2, n5, n6, n7, n8, z1, z2, z3, z4, z5, z6]]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("momentum="\[InvisibleSpace]k2\),
      SequenceForm[ "momentum=", k2],
      Editable->False]], "Print"],

Cell[BoxData[
    \(Fauto::"mode" \(\(:\)\(\ \)\) 
      "U and F polynomials will be calculated in AUTO mode. In order to use \
MANUAL mode execute Fauto[0]."\)], "Message"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(repr = SubLoop[integral];\)\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("Iteration nr"\[InvisibleSpace]2\[InvisibleSpace]": \
>>Integrating over "\[InvisibleSpace]k2\[InvisibleSpace]"<<"\),
      SequenceForm[ "Iteration nr", 2, ": >>Integrating over ", k2, "<<"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Computing U & F polynomial in AUTO mode >>Fauto["\
\[InvisibleSpace]1\[InvisibleSpace]"]<<"\),
      SequenceForm[ 
      "Computing U & F polynomial in AUTO mode >>Fauto[", 1, "]<<"],
      Editable->False]], "Print"],

Cell[BoxData[
    \("U polynomial..."\)], "Print"],

Cell[BoxData[
    \(X[1] + X[2] + X[3] + X[4] + X[5]\)], "Print"],

Cell[BoxData[
    \("F polynomial..."\)], "Print"],

Cell[BoxData[
    \(\(-s12\)\ X[1]\ X[3] - s15\ X[2]\ X[4] - s34\ X[3]\ X[4] - 
      s45\ X[1]\ X[5] - s23\ X[2]\ X[5]\)], "Print"],

Cell[BoxData[
    \("Final representation:"\)], "Print"],

Cell[BoxData[
    TagBox[
      StyleBox[\(\((\((\(-1\))\)\^\(n1 + n2 + n3 + n4 + n5 + n6 + n7 + n8\)\ \
\((\(-s12\))\)\^\(z2 + z7\)\ \((\(-s15\))\)\^z8\ \((\(-s23\))\)\^\(4 - 2\ eps \
- n1 - n2 - n3 - n4 - n5 - n6 - n7 - n8 - z10 - z2 - z5 - z6 - z7 - z8 - z9\)\
\ \((\(-s34\))\)\^\(z5 + z9\)\ \((\(-s45\))\)\^\(z10 + z6\)\ Gamma[\(-z1\)]\ \
Gamma[\(-z10\)]\ Gamma[\(-z2\)]\ Gamma[\(-z3\)]\ Gamma[
                2 - eps - n2 - n5 - n6 - n8 - z1 - z2 - z3 - z4 - 
                  z5]\ Gamma[\(-z5\)]\ Gamma[n8 + z2 + z3 + z5]\ Gamma[
                n6 + z4 + z5]\ Gamma[
                2 - eps - n5 - n6 - n7 - n8 - z2 - z5 - 
                  z6]\ Gamma[\(-z6\)]\ Gamma[
                n5 + z1 + z2 + z6]\ Gamma[\(-z7\)]\ Gamma[
                n1 - z1 + z10 + z7]\ Gamma[\(-z8\)]\ Gamma[
                4 - 2\ eps - n1 - n2 - n3 - n5 - n6 - n7 - n8 - z10 - z2 - 
                  z5 - z6 - z7 - z9]\ Gamma[
                2 - eps - n1 - n3 - n4 + z1 + z3 + z4 - z7 - z8 - 
                  z9]\ Gamma[\(-z9\)]\ Gamma[
                n3 - z3 + z7 + z9]\ Gamma[\(-z4\) + z8 + z9]\ Gamma[\(-4\) + 
                  2\ eps + n1 + n2 + n3 + n4 + n5 + n6 + n7 + n8 + z10 + z2 + 
                  z5 + z6 + z7 + z8 + z9])\)/\((Gamma[n2]\ Gamma[n4]\ Gamma[
                n5]\ Gamma[n6]\ Gamma[n7]\ Gamma[
                4 - 2\ eps - n2 - n5 - n6 - n7 - n8]\ Gamma[n8]\ Gamma[
                n1 - z1]\ Gamma[n3 - z3]\ Gamma[
                6 - 3\ eps - n1 - n2 - n3 - n4 - n5 - n6 - n7 - n8 - z2 - 
                  z5 - z6])\)\),
        FontColor->RGBColor[0, 0, 1]],
      (StyleForm[ #, FontColor -> RGBColor[ 0, 0, 1]]&)]], "Print"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(BarnesLemma[repr, 1]\)], "Input"],

Cell[BoxData[
    InterpretationBox[\(">> Barnes 1st Lemma will be checked for: "\
\[InvisibleSpace]{z4, z3, z1}\[InvisibleSpace]" <<"\),
      SequenceForm[ 
      ">> Barnes 1st Lemma will be checked for: ", {z4, z3, z1}, " <<"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("   Starting with dim="\[InvisibleSpace]10\
\[InvisibleSpace]" representation..."\),
      SequenceForm[ "   Starting with dim=", 10, " representation..."],
      Editable->False]], "Print"],

Cell["\<\

1. Checking z4...Barnes Lemma was applied.
2. Checking z3
3. Checking z1\
\>", "Print"],

Cell[BoxData[
    \(">> Representation after 1st Barnes Lemma:  <<"\)], "Print"],

Cell[BoxData[
    InterpretationBox[
      RowBox[{
        TagBox[
          StyleBox["\<\"   1st Barnes Lemma was applied for: \"\>",
            FontColor->RGBColor[1, 0, 0]],
          (StyleForm[ #, FontColor -> RGBColor[ 1, 0, 0]]&)], 
        "\[InvisibleSpace]", \({z4}\)}],
      SequenceForm[ 
        StyleForm[ 
        "   1st Barnes Lemma was applied for: ", FontColor -> RGBColor[ 
          1, 0, 0]], {z4}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("   Obtained representation has: dim="\
\[InvisibleSpace]9\),
      SequenceForm[ "   Obtained representation has: dim=", 9],
      Editable->False]], "Print"],

Cell[BoxData[
    \(\((\((\(-1\))\)\^\(n1 + n2 + n3 + n4 + n5 + n6 + n7 + n8\)\ \
\((\(-s12\))\)\^\(z2 + z7\)\ \((\(-s15\))\)\^z8\ \((\(-s23\))\)\^\(4 - 2\ eps \
- n1 - n2 - n3 - n4 - n5 - n6 - n7 - n8 - z10 - z2 - z5 - z6 - z7 - z8 - z9\)\
\ \((\(-s34\))\)\^\(z5 + z9\)\ \((\(-s45\))\)\^\(z10 + z6\)\ Gamma[\(-z1\)]\ \
Gamma[\(-z10\)]\ Gamma[\(-z2\)]\ Gamma[
            2 - eps - n2 - n5 - n8 - z1 - z2 - 
              z3]\ Gamma[\(-z3\)]\ Gamma[\(-z5\)]\ Gamma[
            n8 + z2 + z3 + z5]\ Gamma[
            2 - eps - n5 - n6 - n7 - n8 - z2 - z5 - 
              z6]\ Gamma[\(-z6\)]\ Gamma[n5 + z1 + z2 + z6]\ Gamma[
            2 - eps - n1 - n3 - n4 + z1 + z3 - z7]\ Gamma[\(-z7\)]\ Gamma[
            n1 - z1 + z10 + z7]\ Gamma[\(-z8\)]\ Gamma[
            4 - 2\ eps - n1 - n2 - n3 - n5 - n6 - n7 - n8 - z10 - z2 - z5 - 
              z6 - z7 - z9]\ Gamma[
            4 - 2\ eps - n1 - n2 - n3 - n4 - n5 - n6 - n8 - z2 - z5 - z7 - 
              z8 - z9]\ Gamma[\(-z9\)]\ Gamma[n3 - z3 + z7 + z9]\ Gamma[
            n6 + z5 + z8 + z9]\ Gamma[\(-4\) + 2\ eps + n1 + n2 + n3 + n4 + 
              n5 + n6 + n7 + n8 + z10 + z2 + z5 + z6 + z7 + z8 + 
              z9])\)/\((Gamma[n2]\ Gamma[n4]\ Gamma[n5]\ Gamma[n6]\ Gamma[
            n7]\ Gamma[4 - 2\ eps - n2 - n5 - n6 - n7 - n8]\ Gamma[n8]\ Gamma[
            n1 - z1]\ Gamma[n3 - z3]\ Gamma[
            6 - 3\ eps - n1 - n2 - n3 - n4 - n5 - n6 - n7 - n8 - z2 - z5 - 
              z6]\ Gamma[
            4 - 2\ eps - n1 - n2 - n3 - n4 - n5 - n8 - z2 - 
              z7])\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(repr = %\  /. \ {n1 \[Rule] 1, n2 \[Rule] 1, n3 \[Rule] 1, 
          n4 \[Rule] 1, n5 \[Rule] 1, n6 \[Rule] 1, n7 \[Rule] 1, 
          n8 \[Rule] 1}\)], "Input"],

Cell[BoxData[
    \(\((\((\(-s12\))\)\^\(z2 + z7\)\ \((\(-s15\))\)\^z8\ \
\((\(-s23\))\)\^\(\(-4\) - 2\ eps - z10 - z2 - z5 - z6 - z7 - z8 - z9\)\ \
\((\(-s34\))\)\^\(z5 + z9\)\ \((\(-s45\))\)\^\(z10 + z6\)\ Gamma[\(-z1\)]\ \
Gamma[\(-z10\)]\ Gamma[\(-z2\)]\ Gamma[\(-1\) - eps - z1 - z2 - 
              z3]\ Gamma[\(-z3\)]\ Gamma[\(-z5\)]\ Gamma[
            1 + z2 + z3 + z5]\ Gamma[\(-2\) - eps - z2 - z5 - 
              z6]\ Gamma[\(-z6\)]\ Gamma[
            1 + z1 + z2 + z6]\ Gamma[\(-1\) - eps + z1 + z3 - 
              z7]\ Gamma[\(-z7\)]\ Gamma[
            1 - z1 + z10 + z7]\ Gamma[\(-z8\)]\ Gamma[\(-3\) - 2\ eps - z10 - 
              z2 - z5 - z6 - z7 - z9]\ Gamma[\(-3\) - 2\ eps - z2 - z5 - z7 - 
              z8 - z9]\ Gamma[\(-z9\)]\ Gamma[1 - z3 + z7 + z9]\ Gamma[
            1 + z5 + z8 + z9]\ Gamma[
            4 + 2\ eps + z10 + z2 + z5 + z6 + z7 + z8 + 
              z9])\)/\((Gamma[\(-1\) - 2\ eps]\ Gamma[1 - z1]\ Gamma[
            1 - z3]\ Gamma[\(-2\) - 3\ eps - z2 - z5 - z6]\ Gamma[\(-2\) - 
              2\ eps - z2 - z7])\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(rules = MBoptimizedRules[repr, eps \[Rule] 0, {}, {eps}]\)], "Input"],

Cell[BoxData[
    \(MBresidues::"contour" \(\(:\)\(\ \)\) 
      "contour starts and/or ends on a pole of Gamma[\!\(\(\(-1\)\) - eps - \
z1 - z2 - z3\)]"\)], "Message"],

Cell[BoxData[
    \(MBresidues::"contour" \(\(:\)\(\ \)\) 
      "contour starts and/or ends on a pole of Gamma[\!\(\(\(-1\)\) + eps + \
z1 - z10 - z5 - z6\)]"\)], "Message"],

Cell[BoxData[
    \(MBresidues::"contour" \(\(:\)\(\ \)\) 
      "contour starts and/or ends on a pole of Gamma[\!\(\(\(-eps\)\) + z10 - \
z2\)]"\)], "Message"],

Cell[BoxData[
    RowBox[{\(General::"stop"\), \(\(:\)\(\ \)\), "\<\"Further output of \
\\!\\(MBresidues :: \\\"contour\\\"\\) will be suppressed during this \
calculation. \\!\\(\\*ButtonBox[\\\"More\[Ellipsis]\\\", \
ButtonStyle->\\\"RefGuideLinkText\\\", ButtonFrame->None, \
ButtonData:>\\\"General::stop\\\"]\\)\"\>"}]], "Message"],

Cell[BoxData[
    \(MBrules::"norules" \(\(:\)\(\ \)\) 
      "no rules could be found to regulate this integral"\)], "Message"],

Cell[BoxData[
    \(MBrules::"norules" \(\(:\)\(\ \)\) 
      "no rules could be found to regulate this integral"\)], "Message"],

Cell[BoxData[
    \(MBrules::"norules" \(\(:\)\(\ \)\) 
      "no rules could be found to regulate this integral"\)], "Message"],

Cell[BoxData[
    RowBox[{\(General::"stop"\), \(\(:\)\(\ \)\), "\<\"Further output of \
\\!\\(MBrules :: \\\"norules\\\"\\) will be suppressed during this \
calculation. \\!\\(\\*ButtonBox[\\\"More\[Ellipsis]\\\", \
ButtonStyle->\\\"RefGuideLinkText\\\", ButtonFrame->None, \
ButtonData:>\\\"General::stop\\\"]\\)\"\>"}]], "Message"],

Cell[BoxData[
    \({{eps \[Rule] \(-\(65843\/57344\)\)}, {z1 \[Rule] \
\(-\(84443\/516096\)\), z10 \[Rule] \(-\(29\/384\)\), 
        z2 \[Rule] \(-\(133799\/172032\)\), 
        z3 \[Rule] \(-\(59131\/516096\)\), z5 \[Rule] \(-\(157\/2688\)\), 
        z6 \[Rule] \(-\(1213\/73728\)\), z7 \[Rule] \(-\(234571\/516096\)\), 
        z8 \[Rule] \(-\(160507\/516096\)\), 
        z9 \[Rule] \(-\(61\/9216\)\)}}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(integrals = 
        MBcontinue[repr, eps \[Rule] 0, rules, 
          Verbose \[Rule] False];\)\)], "Input"],

Cell[BoxData[
    \(\(ser = 
        MBexpand[{integrals}, 
          Exp[2\ eps\ EulerGamma], {eps, 0, \(-1\)}];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(MBintegrate[
        ser, {s12 \[Rule] \(-2\), s15 \[Rule] \(-1\)/2, s23 \[Rule] \(-4\), 
          s34 \[Rule] \(-5\), s45 \[Rule] \(-6\)}, Verbose \[Rule] False] // 
      AbsoluteTiming\)], "Input"],

Cell[BoxData[
    \({52.614521`8.172650614452866\ Second, {0.159375`\/eps\^4 + 
          0.3193080326901067`\/eps\^3 + 0.4227445612862789`\/eps\^2 + 
          0.2644803629530377`\/eps, {6.865529226745309`*^-6\/eps\^2 + 
            0.0002049692073145298`\/eps, 0}}}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \( (*\ previous\ result\ *) \)], "Input"],

Cell[BoxData[
    \({30.792086`7.9399841045363075\ Second, {0.159375`\/eps\^4 + 
          0.3193080330282121`\/eps\^3 + 0.42274543880473414`\/eps\^2 + 
          0.264480823446921`\/eps, {0.00006874560279338051`\/eps, 
          0}}}\)], "Input"],

Cell[BoxData[
    \( (*\ \[IndentingNewLine]Now\ let\ us\ see\ how\ important\ is\ a\ \
choice\ of\ momenta\ flow\ for\ the\ same\ invariants, \ 
      below\ is\ an\ example\ where\ external\ momenta\ are\ chosen\ in\ a\ \
different\ way\ \[IndentingNewLine]*) \)], "Input"],

Cell[BoxData[
    \(\(Fullintegral[{1}, {PR[k1, 0, n1] PR[k1 + p1, 0, n2] 
            PR[k1 + p1 + p2, 0, n3] PR[k2, 0, n4] PR[k1 - k2, 0, n5] 
            PR[k2 + p1 + p2, 0, n6] PR[k2 - p3 - p4, 0, n7] 
            PR[k2 - p3, 0, n8]}, {k2, k1}];\)\)], "Input"],

Cell[BoxData[
    \(\(invariants = {p1^2 \[Rule] 0, p2^2 \[Rule] 0, p3^2 \[Rule] 0, 
          p4^2 \[Rule] 0, p5^2 \[Rule] 0, p1*p2 \[Rule] s12/2, 
          p2*p3 \[Rule] s23/2, p3*p4 \[Rule] s34/2, p4*p5 \[Rule] s45/2, 
          p1*p5 \[Rule] s15/2, p1*p3 \[Rule] \((\(-s12\) - s23 + s45)\)/2, 
          p1*p4 \[Rule] \((\(-s15\) + s23 - s45)\)/2, 
          p2*p4 \[Rule] \((s15 - s23 - s34)\)/2, 
          p2*p5 \[Rule] \((\(-s12\) - s15 + s34)\)/2, 
          p3*p5 \[Rule] \((s12 - s34 - s45)\)/2};\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(IntPart[1]\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("numerator="\[InvisibleSpace]1\),
      SequenceForm[ "numerator=", 1],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("integral="\[InvisibleSpace]\(PR[k1 - k2, 0, n5]\ PR[
            k2, 0, n4]\ PR[k2 + p1 + p2, 0, n6]\ PR[k2 - p3, 0, n8]\ PR[
            k2 - p3 - p4, 0, n7]\)\),
      SequenceForm[ "integral=", 
        Times[ 
          PR[ 
            Plus[ k1, 
              Times[ -1, k2]], 0, n5], 
          PR[ k2, 0, n4], 
          PR[ 
            Plus[ k2, p1, p2], 0, n6], 
          PR[ 
            Plus[ k2, 
              Times[ -1, p3]], 0, n8], 
          PR[ 
            Plus[ k2, 
              Times[ -1, p3], 
              Times[ -1, p4]], 0, n7]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("momentum="\[InvisibleSpace]k2\),
      SequenceForm[ "momentum=", k2],
      Editable->False]], "Print"],

Cell[BoxData[
    \(Fauto::"mode" \(\(:\)\(\ \)\) 
      "U and F polynomials will be calculated in AUTO mode. In order to use \
MANUAL mode execute Fauto[0]."\)], "Message"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(SubLoop[integral]\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("Iteration nr"\[InvisibleSpace]1\[InvisibleSpace]": \
>>Integrating over "\[InvisibleSpace]k2\[InvisibleSpace]"<<"\),
      SequenceForm[ "Iteration nr", 1, ": >>Integrating over ", k2, "<<"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Computing U & F polynomial in AUTO mode >>Fauto["\
\[InvisibleSpace]1\[InvisibleSpace]"]<<"\),
      SequenceForm[ 
      "Computing U & F polynomial in AUTO mode >>Fauto[", 1, "]<<"],
      Editable->False]], "Print"],

Cell[BoxData[
    \("U polynomial..."\)], "Print"],

Cell[BoxData[
    \(X[1] + X[2] + X[3] + X[4] + X[5]\)], "Print"],

Cell[BoxData[
    \("F polynomial..."\)], "Print"],

Cell[BoxData[
    \(\(-PR[k1, 0]\)\ X[1]\ X[2] - PR[k1 + p1 + p2, 0]\ X[1]\ X[3] - 
      s12\ X[2]\ X[3] - PR[k1 - p3, 0]\ X[1]\ X[4] - s45\ X[3]\ X[4] - 
      PR[\(-k1\) + p3 + p4, 0]\ X[1]\ X[5] - s34\ X[2]\ X[5]\)], "Print"],

Cell[BoxData[
    InterpretationBox[\("Representation after integrating over: "\
\[InvisibleSpace]"k2..."\),
      SequenceForm[ "Representation after integrating over: ", "k2..."],
      Editable->False]], "Print"],

Cell[BoxData[
    RowBox[{"SubLoop1", "[", 
      RowBox[{
        TagBox[
          
          StyleBox[\(\((\((\(-1\))\)\^\(n4 + n5 + n6 + n7 + n8 + z1 + z2 + z4 \
+ z6\)\ \((\(-s12\))\)\^z3\ \((\(-s34\))\)\^\(2 - eps - n4 - n5 - n6 - n7 - \
n8 - z1 - z2 - z3 - z4 - z5 - z6\)\ \((\(-s45\))\)\^z5\ Gamma[\(-z1\)]\ \
Gamma[\(-z2\)]\ Gamma[\(-z3\)]\ Gamma[\(-z4\)]\ Gamma[
                    2 - eps - n4 - n5 - n6 - n8 - z1 - z2 - z3 - z4 - 
                      z5]\ Gamma[\(-z5\)]\ Gamma[n6 + z2 + z3 + z5]\ Gamma[
                    n8 + z4 + z5]\ Gamma[
                    2 - eps - n5 - n6 - n7 - n8 - z2 - z4 - z5 - 
                      z6]\ Gamma[\(-z6\)]\ Gamma[
                    n5 + z1 + z2 + z4 + z6]\ Gamma[\(-2\) + eps + n4 + n5 + 
                      n6 + n7 + n8 + z1 + z2 + z3 + z4 + z5 + z6])\)/\((Gamma[
                    n4]\ Gamma[n5]\ Gamma[n6]\ Gamma[n7]\ Gamma[
                    4 - 2\ eps - n4 - n5 - n6 - n7 - n8]\ Gamma[n8])\)\),
            FontColor->RGBColor[0, 0, 1]],
          (StyleForm[ #, FontColor -> RGBColor[ 0, 0, 1]]&)], ",", 
        TagBox[
          
          StyleBox[\(PR[k1, 0, z1]\ PR[k1 + p1 + p2, 0, z2]\ PR[k1 - p3, 0, 
                z4]\ PR[\(-k1\) + p3 + p4, 0, z6]\),
            FontColor->RGBColor[1, 0, 0]],
          (StyleForm[ #, FontColor -> RGBColor[ 1, 0, 0]]&)]}], 
      "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(IntPart[2]\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("numerator="\[InvisibleSpace]1\),
      SequenceForm[ "numerator=", 1],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("integral="\[InvisibleSpace]\(PR[k1, 0, n1 - z1]\ PR[
            k1 + p1, 0, n2]\ PR[k1 + p1 + p2, 0, n3 - z2]\ PR[k1 - p3, 
            0, \(-z4\)]\ PR[\(-k1\) + p3 + p4, 0, \(-z6\)]\)\),
      SequenceForm[ "integral=", 
        Times[ 
          PR[ k1, 0, 
            Plus[ n1, 
              Times[ -1, z1]]], 
          PR[ 
            Plus[ k1, p1], 0, n2], 
          PR[ 
            Plus[ k1, p1, p2], 0, 
            Plus[ n3, 
              Times[ -1, z2]]], 
          PR[ 
            Plus[ k1, 
              Times[ -1, p3]], 0, 
            Times[ -1, z4]], 
          PR[ 
            Plus[ 
              Times[ -1, k1], p3, p4], 0, 
            Times[ -1, z6]]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("momentum="\[InvisibleSpace]k1\),
      SequenceForm[ "momentum=", k1],
      Editable->False]], "Print"],

Cell[BoxData[
    \(Fauto::"mode" \(\(:\)\(\ \)\) 
      "U and F polynomials will be calculated in AUTO mode. In order to use \
MANUAL mode execute Fauto[0]."\)], "Message"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(repr = SubLoop[integral];\)\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("Iteration nr"\[InvisibleSpace]2\[InvisibleSpace]": \
>>Integrating over "\[InvisibleSpace]k1\[InvisibleSpace]"<<"\),
      SequenceForm[ "Iteration nr", 2, ": >>Integrating over ", k1, "<<"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Computing U & F polynomial in AUTO mode >>Fauto["\
\[InvisibleSpace]1\[InvisibleSpace]"]<<"\),
      SequenceForm[ 
      "Computing U & F polynomial in AUTO mode >>Fauto[", 1, "]<<"],
      Editable->False]], "Print"],

Cell[BoxData[
    \("U polynomial..."\)], "Print"],

Cell[BoxData[
    \(X[1] + X[2] + X[3] + X[4] + X[5]\)], "Print"],

Cell[BoxData[
    \("F polynomial..."\)], "Print"],

Cell[BoxData[
    \(\(-s12\)\ X[1]\ X[3] + s12\ X[2]\ X[4] + s23\ X[2]\ X[4] - 
      s45\ X[2]\ X[4] - s45\ X[3]\ X[4] - s34\ X[1]\ X[5] + s12\ X[2]\ X[5] + 
      s15\ X[2]\ X[5] - s34\ X[2]\ X[5]\)], "Print"],

Cell[BoxData[
    \("Final representation:"\)], "Print"],

Cell[BoxData[
    TagBox[
      StyleBox[\(\((\((\(-1\))\)\^\(n1 + n2 + n3 + n4 + n5 + n6 + n7 + n8\)\ \
\((\(-s12\))\)\^\(z3 + z7\)\ s12\^\(z13 + z8\)\ s15\^z14\ s23\^z9\ \
\((\(-s34\))\)\^\(4 - 2\ eps - n1 - n2 - n3 - n4 - n5 - n6 - n7 - n8 - z10 - \
z11 - z13 - z14 - z3 - z5 - z7 - z8 - z9\)\ \((\(-s45\))\)\^\(z10 + z11 + \
z5\)\ Gamma[\(-z1\)]\ Gamma[\(-z10\)]\ Gamma[\(-z11\)]\ Gamma[\(-z12\)]\ \
Gamma[\(-z13\)]\ Gamma[\(-z14\)]\ Gamma[\(-z2\)]\ Gamma[\(-z3\)]\ Gamma[
                2 - eps - n4 - n5 - n6 - n8 - z1 - z2 - z3 - z4 - 
                  z5]\ Gamma[\(-z5\)]\ Gamma[n6 + z2 + z3 + z5]\ Gamma[
                n8 + z4 + z5]\ Gamma[
                2 - eps - n5 - n6 - n7 - n8 - z2 - z4 - z5 - z6]\ Gamma[
                n5 + z1 + z2 + z4 + z6]\ Gamma[\(-2\) + eps + n4 + n5 + n6 + 
                  n7 + n8 + z1 + z2 + z3 + z4 + z5 + z6]\ Gamma[
                2 - eps - n1 - n3 + z1 - z11 - z12 + z2 + z4 + z6 - 
                  z7]\ Gamma[\(-z7\)]\ Gamma[n1 - z1 + z12 + z7]\ Gamma[
                n3 + z11 - z2 + z7]\ Gamma[\(-z8\)]\ Gamma[
                2 - eps - n1 - n2 - n3 + z1 - z10 - z11 + z2 + z4 - z7 - z8 - 
                  z9]\ Gamma[\(-z9\)]\ Gamma[
                z10 + z11 - z4 + z8 + z9]\ Gamma[\(-2\) + eps + n1 + n2 + 
                  n3 - z1 + z10 + z11 + z12 + z13 + z14 - z2 - z4 - z6 + z7 + 
                  z8 + z9])\)/\((Gamma[n2]\ Gamma[n4]\ Gamma[n5]\ Gamma[
                n6]\ Gamma[n7]\ Gamma[
                4 - 2\ eps - n4 - n5 - n6 - n7 - n8]\ Gamma[n8]\ Gamma[
                n1 - z1]\ Gamma[n3 - z2]\ Gamma[
                4 - 2\ eps - n1 - n2 - n3 + z1 + z2 + z4 + z6])\)\),
        FontColor->RGBColor[0, 0, 1]],
      (StyleForm[ #, FontColor -> RGBColor[ 0, 0, 1]]&)]], "Print"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(BarnesLemma[repr, 1]\)], "Input"],

Cell[BoxData[
    InterpretationBox[\(">> Barnes 1st Lemma will be checked for: "\
\[InvisibleSpace]{z12, z6, z4, z2, z1}\[InvisibleSpace]" <<"\),
      SequenceForm[ 
      ">> Barnes 1st Lemma will be checked for: ", {z12, z6, z4, z2, z1}, 
        " <<"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("   Starting with dim="\[InvisibleSpace]14\
\[InvisibleSpace]" representation..."\),
      SequenceForm[ "   Starting with dim=", 14, " representation..."],
      Editable->False]], "Print"],

Cell["\<\

1. Checking z12...Barnes Lemma was applied.
2. Checking z6
3. Checking z4
4. Checking z2
5. Checking z1\
\>", "Print"],

Cell[BoxData[
    \(">> Representation after 1st Barnes Lemma:  <<"\)], "Print"],

Cell[BoxData[
    InterpretationBox[
      RowBox[{
        TagBox[
          StyleBox["\<\"   1st Barnes Lemma was applied for: \"\>",
            FontColor->RGBColor[1, 0, 0]],
          (StyleForm[ #, FontColor -> RGBColor[ 1, 0, 0]]&)], 
        "\[InvisibleSpace]", \({z12}\)}],
      SequenceForm[ 
        StyleForm[ 
        "   1st Barnes Lemma was applied for: ", FontColor -> RGBColor[ 
          1, 0, 0]], {z12}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("   Obtained representation has: dim="\
\[InvisibleSpace]13\),
      SequenceForm[ "   Obtained representation has: dim=", 13],
      Editable->False]], "Print"],

Cell[BoxData[
    \(\((\((\(-1\))\)\^\(n1 + n2 + n3 + n4 + n5 + n6 + n7 + n8\)\ \
\((\(-s12\))\)\^\(z3 + z7\)\ s12\^\(z13 + z8\)\ s15\^z14\ s23\^z9\ \
\((\(-s34\))\)\^\(4 - 2\ eps - n1 - n2 - n3 - n4 - n5 - n6 - n7 - n8 - z10 - \
z11 - z13 - z14 - z3 - z5 - z7 - z8 - z9\)\ \((\(-s45\))\)\^\(z10 + z11 + \
z5\)\ Gamma[\(-z1\)]\ Gamma[\(-z10\)]\ Gamma[\(-z11\)]\ Gamma[\(-z13\)]\ \
Gamma[\(-z14\)]\ Gamma[\(-z2\)]\ Gamma[\(-z3\)]\ Gamma[
            2 - eps - n4 - n5 - n6 - n8 - z1 - z2 - z3 - z4 - 
              z5]\ Gamma[\(-z5\)]\ Gamma[n6 + z2 + z3 + z5]\ Gamma[
            n8 + z4 + z5]\ Gamma[
            2 - eps - n5 - n6 - n7 - n8 - z2 - z4 - z5 - z6]\ Gamma[
            n5 + z1 + z2 + z4 + z6]\ Gamma[
            2 - eps - n3 - z11 + z2 + z4 + z6]\ Gamma[\(-2\) + eps + n4 + 
              n5 + n6 + n7 + n8 + z1 + z2 + z3 + z4 + z5 + 
              z6]\ Gamma[\(-z7\)]\ Gamma[n1 - z1 + z7]\ Gamma[
            n3 + z11 - z2 + z7]\ Gamma[\(-z8\)]\ Gamma[
            2 - eps - n1 - n2 - n3 + z1 - z10 - z11 + z2 + z4 - z7 - z8 - 
              z9]\ Gamma[\(-z9\)]\ Gamma[
            n2 + z10 + z13 + z14 + z8 + z9]\ Gamma[
            z10 + z11 - z4 + z8 + z9]\ Gamma[\(-2\) + eps + n1 + n2 + n3 - 
              z1 + z10 + z11 + z13 + z14 - z2 - z4 - z6 + z7 + z8 + 
              z9])\)/\((Gamma[n2]\ Gamma[n4]\ Gamma[n5]\ Gamma[n6]\ Gamma[
            n7]\ Gamma[4 - 2\ eps - n4 - n5 - n6 - n7 - n8]\ Gamma[n8]\ Gamma[
            n1 - z1]\ Gamma[n3 - z2]\ Gamma[
            4 - 2\ eps - n1 - n2 - n3 + z1 + z2 + z4 + z6]\ Gamma[
            n1 + n2 - z1 + z10 + z13 + z14 + z7 + z8 + z9])\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(repr = % /. \ {n1 \[Rule] 1, n2 \[Rule] 1, n3 \[Rule] 1, n4 \[Rule] 1, 
          n5 \[Rule] 1, n6 \[Rule] 1, n7 \[Rule] 1, n8 \[Rule] 1}\)], "Input"],

Cell[BoxData[
    \(\((\((\(-s12\))\)\^\(z3 + z7\)\ s12\^\(z13 + z8\)\ s15\^z14\ s23\^z9\ \
\((\(-s34\))\)\^\(\(-4\) - 2\ eps - z10 - z11 - z13 - z14 - z3 - z5 - z7 - z8 \
- z9\)\ \((\(-s45\))\)\^\(z10 + z11 + z5\)\ Gamma[\(-z1\)]\ Gamma[\(-z10\)]\ \
Gamma[\(-z11\)]\ Gamma[\(-z13\)]\ Gamma[\(-z14\)]\ Gamma[\(-z2\)]\ \
Gamma[\(-z3\)]\ Gamma[\(-2\) - eps - z1 - z2 - z3 - z4 - 
              z5]\ Gamma[\(-z5\)]\ Gamma[1 + z2 + z3 + z5]\ Gamma[
            1 + z4 + z5]\ Gamma[\(-2\) - eps - z2 - z4 - z5 - z6]\ Gamma[
            1 + z1 + z2 + z4 + z6]\ Gamma[
            1 - eps - z11 + z2 + z4 + z6]\ Gamma[
            3 + eps + z1 + z2 + z3 + z4 + z5 + z6]\ Gamma[\(-z7\)]\ Gamma[
            1 - z1 + z7]\ Gamma[
            1 + z11 - z2 + z7]\ Gamma[\(-z8\)]\ Gamma[\(-1\) - eps + z1 - 
              z10 - z11 + z2 + z4 - z7 - z8 - z9]\ Gamma[\(-z9\)]\ Gamma[
            1 + z10 + z13 + z14 + z8 + z9]\ Gamma[
            z10 + z11 - z4 + z8 + z9]\ Gamma[
            1 + eps - z1 + z10 + z11 + z13 + z14 - z2 - z4 - z6 + z7 + z8 + 
              z9])\)/\((Gamma[\(-1\) - 2\ eps]\ Gamma[1 - z1]\ Gamma[
            1 - z2]\ Gamma[1 - 2\ eps + z1 + z2 + z4 + z6]\ Gamma[
            2 - z1 + z10 + z13 + z14 + z7 + z8 + z9])\)\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \( (*\ so\ we\ got\ 13  dimensional\ integral\ *) \)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(rules = MBoptimizedRules[repr, eps \[Rule] 0, {}, {eps}]\)], "Input"],

Cell[BoxData[
    \(MBresidues::"contour" \(\(:\)\(\ \)\) 
      "contour starts and/or ends on a pole of Gamma[\!\(\(\(-eps\)\) - z11 + \
z3\)]"\)], "Message"],

Cell[BoxData[
    \(MBrules::"norules" \(\(:\)\(\ \)\) 
      "no rules could be found to regulate this integral"\)], "Message"],

Cell[BoxData[
    \(MBrules::"norules" \(\(:\)\(\ \)\) 
      "no rules could be found to regulate this integral"\)], "Message"],

Cell[BoxData[
    \(MBrules::"norules" \(\(:\)\(\ \)\) 
      "no rules could be found to regulate this integral"\)], "Message"],

Cell[BoxData[
    RowBox[{\(General::"stop"\), \(\(:\)\(\ \)\), "\<\"Further output of \
\\!\\(MBrules :: \\\"norules\\\"\\) will be suppressed during this \
calculation. \\!\\(\\*ButtonBox[\\\"More\[Ellipsis]\\\", \
ButtonStyle->\\\"RefGuideLinkText\\\", ButtonFrame->None, \
ButtonData:>\\\"General::stop\\\"]\\)\"\>"}]], "Message"],

Cell[BoxData[
    \(MBresidues::"contour" \(\(:\)\(\ \)\) 
      "contour starts and/or ends on a pole of Gamma[\!\(\(\(-1\)\) - \(\(2\\ \
eps\)\) - z3\)]"\)], "Message"],

Cell[BoxData[
    \({{eps \[Rule] \(-\(8705\/8192\)\)}, {z1 \[Rule] \(-\(3\/32\)\), 
        z10 \[Rule] \(-\(13\/128\)\), z11 \[Rule] \(-\(13\/32\)\), 
        z13 \[Rule] \(-\(19\/1024\)\), z14 \[Rule] \(-\(19\/256\)\), 
        z2 \[Rule] \(-\(5491\/131072\)\), z3 \[Rule] \(-\(118157\/131072\)\), 
        z4 \[Rule] \(-\(8339\/16384\)\), z5 \[Rule] \(-\(39\/1024\)\), 
        z6 \[Rule] \(-\(46277\/131072\)\), z7 \[Rule] \(-\(43707\/131072\)\), 
        z8 \[Rule] \(-\(19\/32768\)\), 
        z9 \[Rule] \(-\(19\/65536\)\)}}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(integrals = 
        MBcontinue[repr, eps \[Rule] 0, rules, 
          Verbose \[Rule] False];\)\)], "Input"],

Cell[BoxData[
    \(\(ser = 
        MBexpand[{integrals}, 
          Exp[2\ eps\ EulerGamma], {eps, 0, \(-3\)}];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(MBintegrate[
        ser, {s12 \[Rule] \(-2\), s15 \[Rule] \(-1\)/2, s23 \[Rule] \(-4\), 
          s34 \[Rule] \(-5\), s45 \[Rule] \(-6\)}, Verbose \[Rule] False] // 
      AbsoluteTiming\)], "Input"],

Cell[BoxData[
    \({115.889031`8.515587325045304\ Second, \
{\(\(0.`\)\(\[InvisibleSpace]\)\) - 0.0015972691177776619`\/eps\^4 - 
          0.0009770602285296345`\/eps\^3, {1.340357842813021`*^-6\/eps\^4 + 
            6.336253594792198`*^-6\/eps\^3, 0}}}\)], "Output"]
}, Open  ]]
},
FrontEndVersion->"5.1 for X",
ScreenRectangle->{{0, 1024}, {0, 768}},
WindowSize->{615, 689},
WindowMargins->{{Automatic, 131}, {Automatic, 25}}
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 40, 1, 27, "Input"],
Cell[1819, 56, 41, 1, 23, "Print"],
Cell[1863, 59, 51, 1, 23, "Print"],
Cell[1917, 62, 62, 1, 23, "Print"],
Cell[1982, 65, 58, 1, 23, "Print"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2077, 71, 43, 1, 27, "Input"],
Cell[2123, 74, 56, 1, 23, "Print"],
Cell[2182, 77, 60, 1, 23, "Print"]
}, Open  ]],
Cell[2257, 81, 388, 7, 139, "Input"],
Cell[2648, 90, 101, 2, 27, "Input"],
Cell[2752, 94, 58, 1, 27, "Input"],
Cell[2813, 97, 264, 4, 75, "Input"],
Cell[3080, 103, 523, 8, 91, "Input"],

Cell[CellGroupData[{
Cell[3628, 115, 43, 1, 27, "Input"],
Cell[3674, 118, 143, 3, 23, "Print"],
Cell[3820, 123, 460, 13, 39, "Print"],
Cell[4283, 138, 143, 3, 23, "Print"],
Cell[4429, 143, 174, 3, 33, "Message"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4640, 151, 50, 1, 27, "Input"],
Cell[4693, 154, 264, 4, 23, "Print"],
Cell[4960, 160, 257, 5, 23, "Print"],
Cell[5220, 167, 50, 1, 23, "Print"],
Cell[5273, 170, 58, 1, 23, "Print"],
Cell[5334, 173, 50, 1, 23, "Print"],
Cell[5387, 176, 147, 2, 39, "Print"],
Cell[5537, 180, 215, 4, 23, "Print"],
Cell[5755, 186, 1069, 23, 113, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6861, 214, 43, 1, 27, "Input"],
Cell[6907, 217, 143, 3, 23, "Print"],
Cell[7053, 222, 749, 23, 39, "Print"],
Cell[7805, 247, 143, 3, 23, "Print"],
Cell[7951, 252, 174, 3, 33, "Message"]
}, Open  ]],

Cell[CellGroupData[{
Cell[8162, 260, 62, 1, 27, "Input"],
Cell[8227, 263, 264, 4, 23, "Print"],
Cell[8494, 269, 257, 5, 23, "Print"],
Cell[8754, 276, 50, 1, 23, "Print"],
Cell[8807, 279, 65, 1, 23, "Print"],
Cell[8875, 282, 50, 1, 23, "Print"],
Cell[8928, 285, 132, 2, 23, "Print"],
Cell[9063, 289, 56, 1, 23, "Print"],
Cell[9122, 292, 1424, 22, 223, "Print"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10583, 319, 53, 1, 27, "Input"],
Cell[10639, 322, 238, 4, 23, "Print"],
Cell[10880, 328, 226, 4, 23, "Print"],
Cell[11109, 334, 80, 1, 23, "Print"],
Cell[11192, 337, 190, 4, 23, "Print"],
Cell[11385, 343, 1236, 18, 227, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[12658, 366, 183, 3, 27, "Input"],
Cell[12844, 371, 928, 15, 147, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[13809, 391, 89, 1, 27, "Input"],
Cell[13901, 394, 175, 3, 33, "Message"],
Cell[14079, 399, 128, 2, 20, "Message"],
Cell[14210, 403, 128, 2, 20, "Message"],
Cell[14341, 407, 128, 2, 20, "Message"],
Cell[14472, 411, 334, 5, 33, "Message"],
Cell[14809, 418, 299, 5, 80, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[15145, 428, 131, 3, 27, "Input"],
Cell[15279, 433, 365, 5, 33, "Message"]
}, Open  ]],
Cell[15659, 441, 128, 3, 27, "Input"],

Cell[CellGroupData[{
Cell[15812, 448, 228, 5, 59, "Input"],
Cell[16043, 455, 248, 4, 73, "Output"]
}, Open  ]],
Cell[16306, 462, 58, 1, 27, "Input"],
Cell[16367, 465, 264, 4, 75, "Input"],
Cell[16634, 471, 523, 8, 91, "Input"],

Cell[CellGroupData[{
Cell[17182, 483, 43, 1, 27, "Input"],
Cell[17228, 486, 143, 3, 23, "Print"],
Cell[17374, 491, 634, 19, 39, "Print"],
Cell[18011, 512, 143, 3, 23, "Print"],
Cell[18157, 517, 174, 3, 33, "Message"]
}, Open  ]],

Cell[CellGroupData[{
Cell[18368, 525, 50, 1, 27, "Input"],
Cell[18421, 528, 264, 4, 23, "Print"],
Cell[18688, 534, 257, 5, 23, "Print"],
Cell[18948, 541, 50, 1, 23, "Print"],
Cell[19001, 544, 65, 1, 23, "Print"],
Cell[19069, 547, 50, 1, 23, "Print"],
Cell[19122, 550, 236, 4, 55, "Print"],
Cell[19361, 556, 215, 4, 23, "Print"],
Cell[19579, 562, 1372, 27, 177, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[20988, 594, 43, 1, 27, "Input"],
Cell[21034, 597, 143, 3, 23, "Print"],
Cell[21180, 602, 889, 25, 55, "Print"],
Cell[22072, 629, 143, 3, 23, "Print"],
Cell[22218, 634, 174, 3, 33, "Message"]
}, Open  ]],

Cell[CellGroupData[{
Cell[22429, 642, 62, 1, 27, "Input"],
Cell[22494, 645, 264, 4, 23, "Print"],
Cell[22761, 651, 257, 5, 23, "Print"],
Cell[23021, 658, 50, 1, 23, "Print"],
Cell[23074, 661, 65, 1, 23, "Print"],
Cell[23142, 664, 50, 1, 23, "Print"],
Cell[23195, 667, 132, 2, 23, "Print"],
Cell[23330, 671, 56, 1, 23, "Print"],
Cell[23389, 674, 1651, 27, 241, "Print"]
}, Open  ]],

Cell[CellGroupData[{
Cell[25077, 706, 53, 1, 27, "Input"],
Cell[25133, 709, 265, 5, 23, "Print"],
Cell[25401, 716, 228, 4, 23, "Print"],
Cell[25632, 722, 98, 5, 68, "Print"],
Cell[25733, 729, 80, 1, 23, "Print"],
Cell[25816, 732, 458, 12, 23, "Print"],
Cell[26277, 746, 197, 4, 23, "Print"],
Cell[26477, 752, 1558, 25, 277, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[28072, 782, 183, 3, 27, "Input"],
Cell[28258, 787, 1078, 17, 179, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[29373, 809, 89, 1, 27, "Input"],
Cell[29465, 812, 168, 3, 33, "Message"],
Cell[29636, 817, 174, 3, 33, "Message"],
Cell[29813, 822, 160, 3, 20, "Message"],
Cell[29976, 827, 337, 5, 33, "Message"],
Cell[30316, 834, 128, 2, 20, "Message"],
Cell[30447, 838, 128, 2, 20, "Message"],
Cell[30578, 842, 128, 2, 20, "Message"],
Cell[30709, 846, 334, 5, 33, "Message"],
Cell[31046, 853, 422, 7, 116, "Output"]
}, Open  ]],
Cell[31483, 863, 131, 3, 27, "Input"],
Cell[31617, 868, 128, 3, 27, "Input"],

Cell[CellGroupData[{
Cell[31770, 875, 221, 4, 43, "Input"],
Cell[31994, 881, 281, 4, 89, "Output"]
}, Open  ]],
Cell[32290, 888, 59, 1, 27, "Input"],
Cell[32352, 891, 247, 4, 107, "Input"],
Cell[32602, 897, 275, 4, 107, "Input"],
Cell[32880, 903, 264, 4, 75, "Input"],
Cell[33147, 909, 523, 8, 91, "Input"],

Cell[CellGroupData[{
Cell[33695, 921, 43, 1, 27, "Input"],
Cell[33741, 924, 143, 3, 23, "Print"],
Cell[33887, 929, 634, 19, 39, "Print"],
Cell[34524, 950, 143, 3, 23, "Print"],
Cell[34670, 955, 174, 3, 33, "Message"]
}, Open  ]],

Cell[CellGroupData[{
Cell[34881, 963, 50, 1, 27, "Input"],
Cell[34934, 966, 264, 4, 23, "Print"],
Cell[35201, 972, 257, 5, 23, "Print"],
Cell[35461, 979, 50, 1, 23, "Print"],
Cell[35514, 982, 65, 1, 23, "Print"],
Cell[35582, 985, 50, 1, 23, "Print"],
Cell[35635, 988, 229, 3, 55, "Print"],
Cell[35867, 993, 215, 4, 23, "Print"],
Cell[36085, 999, 1372, 26, 213, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[37494, 1030, 43, 1, 27, "Input"],
Cell[37540, 1033, 143, 3, 23, "Print"],
Cell[37686, 1038, 757, 23, 39, "Print"],
Cell[38446, 1063, 143, 3, 23, "Print"],
Cell[38592, 1068, 174, 3, 33, "Message"]
}, Open  ]],

Cell[CellGroupData[{
Cell[38803, 1076, 62, 1, 27, "Input"],
Cell[38868, 1079, 264, 4, 23, "Print"],
Cell[39135, 1085, 257, 5, 23, "Print"],
Cell[39395, 1092, 50, 1, 23, "Print"],
Cell[39448, 1095, 65, 1, 23, "Print"],
Cell[39516, 1098, 50, 1, 23, "Print"],
Cell[39569, 1101, 211, 3, 39, "Print"],
Cell[39783, 1106, 56, 1, 23, "Print"],
Cell[39842, 1109, 1759, 27, 273, "Print"]
}, Open  ]],

Cell[CellGroupData[{
Cell[41638, 1141, 53, 1, 27, "Input"],
Cell[41694, 1144, 292, 6, 23, "Print"],
Cell[41989, 1152, 228, 4, 23, "Print"],
Cell[42220, 1158, 129, 7, 98, "Print"],
Cell[42352, 1167, 80, 1, 23, "Print"],
Cell[42435, 1170, 460, 12, 23, "Print"],
Cell[42898, 1184, 199, 4, 23, "Print"],
Cell[43100, 1190, 1624, 25, 293, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[44761, 1220, 170, 2, 27, "Input"],
Cell[44934, 1224, 1249, 19, 211, "Output"]
}, Open  ]],
Cell[46198, 1246, 81, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[46304, 1251, 89, 1, 27, "Input"],
Cell[46396, 1254, 160, 3, 20, "Message"],
Cell[46559, 1259, 128, 2, 20, "Message"],
Cell[46690, 1263, 128, 2, 20, "Message"],
Cell[46821, 1267, 128, 2, 20, "Message"],
Cell[46952, 1271, 334, 5, 33, "Message"],
Cell[47289, 1278, 170, 3, 20, "Message"],
Cell[47462, 1283, 546, 8, 116, "Output"]
}, Open  ]],
Cell[48023, 1294, 131, 3, 27, "Input"],
Cell[48157, 1299, 128, 3, 27, "Input"],

Cell[CellGroupData[{
Cell[48310, 1306, 221, 4, 43, "Input"],
Cell[48534, 1312, 270, 4, 75, "Output"]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

