(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.2'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     61919,       1611]*)
(*NotebookOutlinePosition[     62549,       1633]*)
(*  CellTagsIndexPosition[     62505,       1629]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[BoxData[
    \(<< MB.m\)], "Input"],

Cell[BoxData[
    \("MB 1.1"\)], "Print"],

Cell[BoxData[
    \("by Michal Czakon"\)], "Print"],

Cell[BoxData[
    \("more info in hep-ph/0511200"\)], "Print"],

Cell[BoxData[
    \("last modified 12 Dec 06"\)], "Print"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(<< AMBRE.m\)], "Input"],

Cell[BoxData[
    \("by K.Kajda   ver: 1.0"\)], "Print"],

Cell[BoxData[
    \("last modified 19 Apr 2007"\)], "Print"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(<< KinematicsGen.m\)], "Input"],

Cell[BoxData[
    \("by K.Kajda ver:1.2"\)], "Print"],

Cell[BoxData[
    \("last modification 01 Feb 2007"\)], "Print"]
}, Open  ]],

Cell[BoxData[
    \(\(int = 
        Fullintegral[{1}, {PR[k1, 0, n1]\ PR[k1 + p1, 0, n2]\ PR[
                k1 + p1 + p2, 0, n3]\ PR[k1 + p1 + p2 + p3, 0, n4]\ PR[
                k1 + p1 + p2 + p3 + p4, 0, n5] 
              PR[k1 + p1 + p2 + p3 + p4 + p5, 0, n6]}, {k1}];\)\)], "Input"],

Cell[BoxData[
    \(\(kin = {p1^2 \[Rule] 0, p2^2 \[Rule] 0, p3^2 \[Rule] 0, 
          p4^2 \[Rule] 0, p5^2 \[Rule] 0, p6^2 \[Rule] 0};\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(choice = {s12, s23, s34, s45, s56, s16, s123, s234, 
          s345};\)\)], "Input"],

Cell[BoxData[
    RowBox[{\(General::"spell1"\), \(\(:\)\(\ \)\), "\<\"Possible spelling \
error: new symbol name \\\"\\!\\(s123\\)\\\" is similar to existing symbol \\\
\"\\!\\(s23\\)\\\". \\!\\(\\*ButtonBox[\\\"More\[Ellipsis]\\\", ButtonStyle->\
\\\"RefGuideLinkText\\\", ButtonFrame->None, \
ButtonData:>\\\"General::spell1\\\"]\\)\"\>"}]], "Message"],

Cell[BoxData[
    RowBox[{\(General::"spell1"\), \(\(:\)\(\ \)\), "\<\"Possible spelling \
error: new symbol name \\\"\\!\\(s234\\)\\\" is similar to existing symbol \\\
\"\\!\\(s34\\)\\\". \\!\\(\\*ButtonBox[\\\"More\[Ellipsis]\\\", ButtonStyle->\
\\\"RefGuideLinkText\\\", ButtonFrame->None, \
ButtonData:>\\\"General::spell1\\\"]\\)\"\>"}]], "Message"],

Cell[BoxData[
    RowBox[{\(General::"spell1"\), \(\(:\)\(\ \)\), "\<\"Possible spelling \
error: new symbol name \\\"\\!\\(s345\\)\\\" is similar to existing symbol \\\
\"\\!\\(s45\\)\\\". \\!\\(\\*ButtonBox[\\\"More\[Ellipsis]\\\", ButtonStyle->\
\\\"RefGuideLinkText\\\", ButtonFrame->None, \
ButtonData:>\\\"General::spell1\\\"]\\)\"\>"}]], "Message"],

Cell[BoxData[
    RowBox[{\(General::"stop"\), \(\(:\)\(\ \)\), "\<\"Further output of \
\\!\\(General :: \\\"spell1\\\"\\) will be suppressed during this \
calculation. \\!\\(\\*ButtonBox[\\\"More\[Ellipsis]\\\", \
ButtonStyle->\\\"RefGuideLinkText\\\", ButtonFrame->None, \
ButtonData:>\\\"General::stop\\\"]\\)\"\>"}]], "Message"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(invariants = Kinematics[kin, choice]\)], "Input"],

Cell[BoxData[
    \({p1\^2 \[Rule] 0, p2\^2 \[Rule] 0, p3\^2 \[Rule] 0, p4\^2 \[Rule] 0, 
      p5\^2 \[Rule] 0, p6\^2 \[Rule] 0, p1\ p2 \[Rule] s12\/2, 
      p2\ p3 \[Rule] s23\/2, p3\ p4 \[Rule] s34\/2, p4\ p5 \[Rule] s45\/2, 
      p5\ p6 \[Rule] s56\/2, p1\ p6 \[Rule] s16\/2, 
      p1\ p3 \[Rule] 1\/2\ \((\(-s12\) + s123 - s23)\), 
      p2\ p4 \[Rule] 1\/2\ \((\(-s23\) + s234 - s34)\), 
      p3\ p5 \[Rule] 1\/2\ \((\(-s34\) + s345 - s45)\), 
      p1\ p4 \[Rule] 1\/2\ \((\(-s123\) + s23 - s234 + s56)\), 
      p1\ p5 \[Rule] 1\/2\ \((\(-s16\) + s234 - s56)\), 
      p2\ p5 \[Rule] 1\/2\ \((s16 - s234 + s34 - s345)\), 
      p2\ p6 \[Rule] 1\/2\ \((\(-s12\) - s16 + s345)\), 
      p3\ p6 \[Rule] 1\/2\ \((s12 - s123 - s345 + s45)\), 
      p4\ p6 \[Rule] 1\/2\ \((s123 - s45 - s56)\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(IntPart[1]\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("numerator="\[InvisibleSpace]1\),
      SequenceForm[ "numerator=", 1],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("integral="\[InvisibleSpace]\(PR[k1, 0, n1]\ PR[
            k1 + p1, 0, n2]\ PR[k1 + p1 + p2, 0, n3]\ PR[k1 + p1 + p2 + p3, 
            0, n4]\ PR[k1 + p1 + p2 + p3 + p4, 0, n5]\ PR[
            k1 + p1 + p2 + p3 + p4 + p5, 0, n6]\)\),
      SequenceForm[ "integral=", 
        Times[ 
          PR[ k1, 0, n1], 
          PR[ 
            Plus[ k1, p1], 0, n2], 
          PR[ 
            Plus[ k1, p1, p2], 0, n3], 
          PR[ 
            Plus[ k1, p1, p2, p3], 0, n4], 
          PR[ 
            Plus[ k1, p1, p2, p3, p4], 0, n5], 
          PR[ 
            Plus[ k1, p1, p2, p3, p4, p5], 0, n6]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("momentum="\[InvisibleSpace]k1\),
      SequenceForm[ "momentum=", k1],
      Editable->False]], "Print"],

Cell[BoxData[
    \(Fauto::"mode" \(\(:\)\(\ \)\) 
      "U and F polynomials will be calculated in AUTO mode. In order to use \
MANUAL mode execute Fauto[0]."\)], "Message"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(rep = SubLoop[integral];\)\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("Iteration nr"\[InvisibleSpace]1\[InvisibleSpace]": \
>>Integrating over "\[InvisibleSpace]k1\[InvisibleSpace]"<<"\),
      SequenceForm[ "Iteration nr", 1, ": >>Integrating over ", k1, "<<"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Computing U & F polynomial in AUTO mode >>Fauto["\
\[InvisibleSpace]1\[InvisibleSpace]"]<<"\),
      SequenceForm[ 
      "Computing U & F polynomial in AUTO mode >>Fauto[", 1, "]<<"],
      Editable->False]], "Print"],

Cell[BoxData[
    \("U polynomial..."\)], "Print"],

Cell[BoxData[
    \(X[1] + X[2] + X[3] + X[4] + X[5] + X[6]\)], "Print"],

Cell[BoxData[
    \("F polynomial..."\)], "Print"],

Cell[BoxData[
    \(\(-s12\)\ X[1]\ X[3] - s123\ X[1]\ X[4] - s23\ X[2]\ X[4] - 
      s56\ X[1]\ X[5] - s234\ X[2]\ X[5] - s34\ X[3]\ X[5] - 
      s16\ X[2]\ X[6] - s345\ X[3]\ X[6] - s45\ X[4]\ X[6]\)], "Print"],

Cell[BoxData[
    \("Final representation:"\)], "Print"],

Cell[BoxData[
    TagBox[
      StyleBox[\(\((\((\(-1\))\)\^\(n1 + n2 + n3 + n4 + n5 + n6\)\ \
\((\(-s12\))\)\^z1\ \((\(-s123\))\)\^z2\ \((\(-s16\))\)\^z7\ \
\((\(-s23\))\)\^z3\ \((\(-s234\))\)\^z5\ \((\(-s34\))\)\^z6\ \
\((\(-s345\))\)\^z8\ \((\(-s45\))\)\^\(2 - eps - n1 - n2 - n3 - n4 - n5 - n6 \
- z1 - z2 - z3 - z4 - z5 - z6 - z7 - z8\)\ \((\(-s56\))\)\^z4\ Gamma[\(-z1\)]\
\ Gamma[\(-z2\)]\ Gamma[\(-z3\)]\ Gamma[\(-z4\)]\ Gamma[
                n1 + z1 + z2 + z4]\ Gamma[\(-z5\)]\ Gamma[
                2 - eps - n1 - n2 - n3 - n4 - n5 - z1 - z2 - z3 - z4 - z5 - 
                  z6]\ Gamma[\(-z6\)]\ Gamma[
                n5 + z4 + z5 + z6]\ Gamma[\(-z7\)]\ Gamma[
                n2 + z3 + z5 + z7]\ Gamma[
                2 - eps - n1 - n2 - n3 - n5 - n6 - z1 - z4 - z5 - z6 - z7 - 
                  z8]\ Gamma[\(-z8\)]\ Gamma[
                n3 + z1 + z6 + z8]\ Gamma[\(-2\) + eps + n1 + n2 + n3 + n4 + 
                  n5 + n6 + z1 + z2 + z3 + z4 + z5 + z6 + z7 + 
                  z8])\)/\((Gamma[n1]\ Gamma[n2]\ Gamma[n3]\ Gamma[n4]\ Gamma[
                n5]\ Gamma[4 - 2\ eps - n1 - n2 - n3 - n4 - n5 - n6]\ Gamma[
                n6])\)\),
        FontColor->RGBColor[0, 0, 1]],
      (StyleForm[ #, FontColor -> RGBColor[ 0, 0, 1]]&)]], "Print"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(fin = BarnesLemma[rep, 1]\)], "Input"],

Cell[BoxData[
    InterpretationBox[\(">> Barnes 1st Lemma will be checked for: "\
\[InvisibleSpace]{}\[InvisibleSpace]" <<"\),
      SequenceForm[ ">> Barnes 1st Lemma will be checked for: ", {}, " <<"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("   Starting with dim="\[InvisibleSpace]8\
\[InvisibleSpace]" representation..."\),
      SequenceForm[ "   Starting with dim=", 8, " representation..."],
      Editable->False]], "Print"],

Cell[BoxData[
    \(">> Representation after 1st Barnes Lemma:  <<"\)], "Print"],

Cell[BoxData[
    TagBox[
      StyleBox["\<\"   Could not apply Barnes-Lemma\"\>",
        FontColor->RGBColor[1, 0, 0]],
      (StyleForm[ #, FontColor -> RGBColor[ 1, 0, 0]]&)]], "Print"],

Cell[BoxData[
    \(\((\((\(-1\))\)\^\(n1 + n2 + n3 + n4 + n5 + n6\)\ \((\(-s12\))\)\^z1\ \
\((\(-s123\))\)\^z2\ \((\(-s16\))\)\^z7\ \((\(-s23\))\)\^z3\ \
\((\(-s234\))\)\^z5\ \((\(-s34\))\)\^z6\ \((\(-s345\))\)\^z8\ \
\((\(-s45\))\)\^\(2 - eps - n1 - n2 - n3 - n4 - n5 - n6 - z1 - z2 - z3 - z4 - \
z5 - z6 - z7 - z8\)\ \((\(-s56\))\)\^z4\ Gamma[\(-z1\)]\ Gamma[\(-z2\)]\ \
Gamma[\(-z3\)]\ Gamma[\(-z4\)]\ Gamma[
            n1 + z1 + z2 + z4]\ Gamma[\(-z5\)]\ Gamma[
            2 - eps - n1 - n2 - n3 - n4 - n5 - z1 - z2 - z3 - z4 - z5 - 
              z6]\ Gamma[\(-z6\)]\ Gamma[
            n5 + z4 + z5 + z6]\ Gamma[\(-z7\)]\ Gamma[
            n2 + z3 + z5 + z7]\ Gamma[
            2 - eps - n1 - n2 - n3 - n5 - n6 - z1 - z4 - z5 - z6 - z7 - 
              z8]\ Gamma[\(-z8\)]\ Gamma[
            n3 + z1 + z6 + z8]\ Gamma[\(-2\) + eps + n1 + n2 + n3 + n4 + n5 + 
              n6 + z1 + z2 + z3 + z4 + z5 + z6 + z7 + z8])\)/\((Gamma[
            n1]\ Gamma[n2]\ Gamma[n3]\ Gamma[n4]\ Gamma[n5]\ Gamma[
            4 - 2\ eps - n1 - n2 - n3 - n4 - n5 - n6]\ Gamma[
            n6])\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(fin = \(\(\(\(\(fin /. n1 \[Rule] 1\) /. n2 \[Rule] 1\) /. 
              n3 \[Rule] 1\) /. n4 \[Rule] 1\) /. n5 \[Rule] 1\) /. 
        n6 \[Rule] 1\)], "Input"],

Cell[BoxData[
    \(\(\(1\/Gamma[\(-2\) - 
            2\ eps]\)\((\((\(-s12\))\)\^z1\ \((\(-s123\))\)\^z2\ \((\(-s16\))\
\)\^z7\ \((\(-s23\))\)\^z3\ \((\(-s234\))\)\^z5\ \((\(-s34\))\)\^z6\ \
\((\(-s345\))\)\^z8\ \((\(-s45\))\)\^\(\(-4\) - eps - z1 - z2 - z3 - z4 - z5 \
- z6 - z7 - z8\)\ \((\(-s56\))\)\^z4\ Gamma[\(-z1\)]\ Gamma[\(-z2\)]\ \
Gamma[\(-z3\)]\ Gamma[\(-z4\)]\ Gamma[
          1 + z1 + z2 + z4]\ Gamma[\(-z5\)]\ Gamma[\(-3\) - eps - z1 - z2 - 
            z3 - z4 - z5 - z6]\ Gamma[\(-z6\)]\ Gamma[
          1 + z4 + z5 + z6]\ Gamma[\(-z7\)]\ Gamma[
          1 + z3 + z5 + z7]\ Gamma[\(-3\) - eps - z1 - z4 - z5 - z6 - z7 - 
            z8]\ Gamma[\(-z8\)]\ Gamma[1 + z1 + z6 + z8]\ Gamma[
          4 + eps + z1 + z2 + z3 + z4 + z5 + z6 + z7 + z8])\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(rules = MBoptimizedRules[fin, eps \[Rule] 0, {}, {eps}]\)], "Input"],

Cell[BoxData[
    \(MBresidues::"contour" \(\(:\)\(\ \)\) 
      "contour starts and/or ends on a pole of Gamma[\!\(\(\(-2\)\) - eps - \
z4 - z5 - z7\)]"\)], "Message"],

Cell[BoxData[
    \(MBresidues::"contour" \(\(:\)\(\ \)\) 
      "contour starts and/or ends on a pole of Gamma[\!\(\(\(-2\)\) - eps - \
z2 - z3 - z4 - z5\)]"\)], "Message"],

Cell[BoxData[
    \(MBrules::"norules" \(\(:\)\(\ \)\) 
      "no rules could be found to regulate this integral"\)], "Message"],

Cell[BoxData[
    \(MBrules::"norules" \(\(:\)\(\ \)\) 
      "no rules could be found to regulate this integral"\)], "Message"],

Cell[BoxData[
    \(MBrules::"norules" \(\(:\)\(\ \)\) 
      "no rules could be found to regulate this integral"\)], "Message"],

Cell[BoxData[
    RowBox[{\(General::"stop"\), \(\(:\)\(\ \)\), "\<\"Further output of \
\\!\\(MBrules :: \\\"norules\\\"\\) will be suppressed during this \
calculation. \\!\\(\\*ButtonBox[\\\"More\[Ellipsis]\\\", \
ButtonStyle->\\\"RefGuideLinkText\\\", ButtonFrame->None, \
ButtonData:>\\\"General::stop\\\"]\\)\"\>"}]], "Message"],

Cell[BoxData[
    \(MBresidues::"contour" \(\(:\)\(\ \)\) 
      "contour starts and/or ends on a pole of Gamma[\!\(\(\(-eps\)\) + z1 - \
z5\)]"\)], "Message"],

Cell[BoxData[
    RowBox[{\(General::"stop"\), \(\(:\)\(\ \)\), "\<\"Further output of \
\\!\\(MBresidues :: \\\"contour\\\"\\) will be suppressed during this \
calculation. \\!\\(\\*ButtonBox[\\\"More\[Ellipsis]\\\", \
ButtonStyle->\\\"RefGuideLinkText\\\", ButtonFrame->None, \
ButtonData:>\\\"General::stop\\\"]\\)\"\>"}]], "Message"],

Cell[BoxData[
    \({{eps \[Rule] \(-\(445\/256\)\)}, {z1 \[Rule] \(-\(37\/64\)\), 
        z2 \[Rule] \(-\(1\/8\)\), z3 \[Rule] \(-\(51\/64\)\), 
        z4 \[Rule] \(-\(1\/4\)\), z5 \[Rule] \(-\(1\/16\)\), 
        z6 \[Rule] \(-\(63\/256\)\), z7 \[Rule] \(-\(3\/64\)\), 
        z8 \[Rule] \(-\(15\/128\)\)}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(integrals = 
        MBcontinue[fin, eps \[Rule] 0, rules, 
          Verbose \[Rule] False];\)\)], "Input"],

Cell[BoxData[
    RowBox[{\(General::"spell1"\), \(\(:\)\(\ \)\), "\<\"Possible spelling \
error: new symbol name \\\"\\!\\(integrals\\)\\\" is similar to existing \
symbol \\\"\\!\\(integral\\)\\\". \\!\\(\\*ButtonBox[\\\"More\[Ellipsis]\\\", \
ButtonStyle->\\\"RefGuideLinkText\\\", ButtonFrame->None, \
ButtonData:>\\\"General::spell1\\\"]\\)\"\>"}]], "Message"]
}, Open  ]],

Cell[BoxData[
    \(\(ser0 = 
        MBexpand[{integrals}, 
          Exp[eps\ EulerGamma], {eps, 0, 0}];\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(AbsoluteTiming[
      MBintegrate[
        ser0, {s12 \[Rule] \(-1\), s23 \[Rule] \(-2\), s34 \[Rule] \(-3\), 
          s45 \[Rule] \(-4\), s56 \[Rule] \(-5\), s16 \[Rule] \(-6\), 
          s123 \[Rule] \(-7\), s234 \[Rule] \(-8\), 
          s345 \[Rule] \(-9\)}]]\)], "Input"],

Cell[BoxData[
    \("Shifting contours..."\)], "Print"],

Cell["\<\
Performing 63 1-dimensional \
integrations...1...2...3...4...5...6...7...8...9...10...11...12...13...14...\
15...16...17...18...19...20...21...22...23...24...25...26...27...28...29...30.\
..31...32...33...34...35...36...37...38...39...40...41...42...43...44...45...\
46...47...48...49...50...51...52...53...54...55...56...57...58...59...60...61.\
..62...63\
\>", "Print"],

Cell[BoxData[
    \("Higher-dimensional integrals"\)], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart1eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart1eps0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart2eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart2eps0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart3eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart3eps0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart4eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart4eps0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart5eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart5eps0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart6eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart6eps0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart7eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart7eps0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart8eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart8eps0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart9eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart9eps0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart10eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart10eps0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart11eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart11eps0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart12eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart12eps0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart13eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart13eps0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart14eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart14eps0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart15eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart15eps0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart16eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart16eps0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart17eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart17eps0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart18eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart18eps0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart19eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart19eps0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart20eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart20eps0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart21eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart21eps0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart22eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart22eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart23eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart23eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart24eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart24eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart25eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart25eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart26eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart26eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart27eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart27eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart28eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart28eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart29eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart29eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart30eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart30eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart31eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart31eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart32eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart32eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart33eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart33eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart34eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart34eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart35eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart35eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart36eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart36eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart37eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart37eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart38eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart38eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart39eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart39eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart40eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart40eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart41eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart41eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart42eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart42eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart43eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart43eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart44eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart44eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart45eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart45eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart46eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart46eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart47eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart47eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart48eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart48eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart49eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart49eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart50eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart50eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart51eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart51eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart52eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart52eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart53eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart53eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart54eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart54eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart55eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart55eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart56eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart56eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart57eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart57eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart58eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart58eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart59eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart59eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart60eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart60eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart61eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart61eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart62eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart62eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart63eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart63eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart64eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart64eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart65eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart65eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart66eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart66eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart67eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart67eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart68eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart68eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart69eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart69eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart70eps-1"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart70eps-1", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart71eps-1"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart71eps-1", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart72eps-1"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart72eps-1", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart73eps-1"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart73eps-1", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart74eps-1"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart74eps-1", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart75eps-1"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart75eps-1", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart1eps0"\),
      SequenceForm[ "Running ", "MBpart1eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart2eps0"\),
      SequenceForm[ "Running ", "MBpart2eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart3eps0"\),
      SequenceForm[ "Running ", "MBpart3eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart4eps0"\),
      SequenceForm[ "Running ", "MBpart4eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart5eps0"\),
      SequenceForm[ "Running ", "MBpart5eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart6eps0"\),
      SequenceForm[ "Running ", "MBpart6eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart7eps0"\),
      SequenceForm[ "Running ", "MBpart7eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart8eps0"\),
      SequenceForm[ "Running ", "MBpart8eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart9eps0"\),
      SequenceForm[ "Running ", "MBpart9eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart10eps0"\),
      SequenceForm[ "Running ", "MBpart10eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart11eps0"\),
      SequenceForm[ "Running ", "MBpart11eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart12eps0"\),
      SequenceForm[ "Running ", "MBpart12eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart13eps0"\),
      SequenceForm[ "Running ", "MBpart13eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart14eps0"\),
      SequenceForm[ "Running ", "MBpart14eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart15eps0"\),
      SequenceForm[ "Running ", "MBpart15eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart16eps0"\),
      SequenceForm[ "Running ", "MBpart16eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart17eps0"\),
      SequenceForm[ "Running ", "MBpart17eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart18eps0"\),
      SequenceForm[ "Running ", "MBpart18eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart19eps0"\),
      SequenceForm[ "Running ", "MBpart19eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart20eps0"\),
      SequenceForm[ "Running ", "MBpart20eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart21eps0"\),
      SequenceForm[ "Running ", "MBpart21eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart22eps0"\),
      SequenceForm[ "Running ", "MBpart22eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart23eps0"\),
      SequenceForm[ "Running ", "MBpart23eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart24eps0"\),
      SequenceForm[ "Running ", "MBpart24eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart25eps0"\),
      SequenceForm[ "Running ", "MBpart25eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart26eps0"\),
      SequenceForm[ "Running ", "MBpart26eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart27eps0"\),
      SequenceForm[ "Running ", "MBpart27eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart28eps0"\),
      SequenceForm[ "Running ", "MBpart28eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart29eps0"\),
      SequenceForm[ "Running ", "MBpart29eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart30eps0"\),
      SequenceForm[ "Running ", "MBpart30eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart31eps0"\),
      SequenceForm[ "Running ", "MBpart31eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart32eps0"\),
      SequenceForm[ "Running ", "MBpart32eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart33eps0"\),
      SequenceForm[ "Running ", "MBpart33eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart34eps0"\),
      SequenceForm[ "Running ", "MBpart34eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart35eps0"\),
      SequenceForm[ "Running ", "MBpart35eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart36eps0"\),
      SequenceForm[ "Running ", "MBpart36eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart37eps0"\),
      SequenceForm[ "Running ", "MBpart37eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart38eps0"\),
      SequenceForm[ "Running ", "MBpart38eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart39eps0"\),
      SequenceForm[ "Running ", "MBpart39eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart40eps0"\),
      SequenceForm[ "Running ", "MBpart40eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart41eps0"\),
      SequenceForm[ "Running ", "MBpart41eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart42eps0"\),
      SequenceForm[ "Running ", "MBpart42eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart43eps0"\),
      SequenceForm[ "Running ", "MBpart43eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart44eps0"\),
      SequenceForm[ "Running ", "MBpart44eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart45eps0"\),
      SequenceForm[ "Running ", "MBpart45eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart46eps0"\),
      SequenceForm[ "Running ", "MBpart46eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart47eps0"\),
      SequenceForm[ "Running ", "MBpart47eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart48eps0"\),
      SequenceForm[ "Running ", "MBpart48eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart49eps0"\),
      SequenceForm[ "Running ", "MBpart49eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart50eps0"\),
      SequenceForm[ "Running ", "MBpart50eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart51eps0"\),
      SequenceForm[ "Running ", "MBpart51eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart52eps0"\),
      SequenceForm[ "Running ", "MBpart52eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart53eps0"\),
      SequenceForm[ "Running ", "MBpart53eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart54eps0"\),
      SequenceForm[ "Running ", "MBpart54eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart55eps0"\),
      SequenceForm[ "Running ", "MBpart55eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart56eps0"\),
      SequenceForm[ "Running ", "MBpart56eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart57eps0"\),
      SequenceForm[ "Running ", "MBpart57eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart58eps0"\),
      SequenceForm[ "Running ", "MBpart58eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart59eps0"\),
      SequenceForm[ "Running ", "MBpart59eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart60eps0"\),
      SequenceForm[ "Running ", "MBpart60eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart61eps0"\),
      SequenceForm[ "Running ", "MBpart61eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart62eps0"\),
      SequenceForm[ "Running ", "MBpart62eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart63eps0"\),
      SequenceForm[ "Running ", "MBpart63eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart64eps0"\),
      SequenceForm[ "Running ", "MBpart64eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart65eps0"\),
      SequenceForm[ "Running ", "MBpart65eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart66eps0"\),
      SequenceForm[ "Running ", "MBpart66eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart67eps0"\),
      SequenceForm[ "Running ", "MBpart67eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart68eps0"\),
      SequenceForm[ "Running ", "MBpart68eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart69eps0"\),
      SequenceForm[ "Running ", "MBpart69eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart70eps-1"\),
      SequenceForm[ "Running ", "MBpart70eps-1"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart71eps-1"\),
      SequenceForm[ "Running ", "MBpart71eps-1"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart72eps-1"\),
      SequenceForm[ "Running ", "MBpart72eps-1"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart73eps-1"\),
      SequenceForm[ "Running ", "MBpart73eps-1"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart74eps-1"\),
      SequenceForm[ "Running ", "MBpart74eps-1"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart75eps-1"\),
      SequenceForm[ "Running ", "MBpart75eps-1"],
      Editable->False]], "Print"],

Cell[BoxData[
    \({191.039231`8.732667554760944\ Second, {\(\(0.03528279479214159`\)\(\
\[InvisibleSpace]\)\) + 0.04265873015873016`\/eps\^2 + 
          0.032177104127239245`\/eps, {\(\(0.000018885478407551604`\)\(\
\[InvisibleSpace]\)\) + 1.1393871241562284`*^-6\/eps, 0}}}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \( (*\[IndentingNewLine]sectors\[IndentingNewLine]{50.959184`\ Second, {{\
\(\(0.03516979557127395`\)\(\[InvisibleSpace]\)\) + 
                0.042661451700768016`\/ep\^2 + 
                0.032145003870503716`\/ep, \(\(0.0007859234224555659`\)\(\
\[InvisibleSpace]\)\) + 0.000056059196442435445`\/ep\^2 + 
                0.0002445850279203611`\/ep, \(\(1.263567130584154`\)\(\
\[InvisibleSpace]\)\) + 0.008038549801100152`\/ep\^2 + 
                0.33710515536754804`\/ep}}}\[IndentingNewLine]*) \)], "Input"],

Cell[BoxData[
    \( (*\(\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\
\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(\ \)\(massive\)\)\ internal\ lines\ \ *******************************) \
\)], "Input"],

Cell[BoxData[
    \(\(int = 
        Fullintegral[{1}, {PR[k1, m, n1]\ PR[k1 + p1, m, n2]\ PR[
                k1 + p1 + p2, m, n3]\ PR[k1 + p1 + p2 + p3, m, n4]\ PR[
                k1 + p1 + p2 + p3 + p4, m, n5] 
              PR[k1 + p1 + p2 + p3 + p4 + p5, m, n6]}, {k1}];\)\)], "Input"],

Cell[BoxData[
    \(\(invariants = {p1^2 \[Rule] 0, p2^2 \[Rule] 0, p3^2 \[Rule] 0, 
          p4^2 \[Rule] 0, p5^2 \[Rule] 0, p6^2 \[Rule] 0, 
          p1*p4 \[Rule] \(-\((s123 - s23 + s234 - s56)\)\)/2, 
          p1*p5 \[Rule] \(-\((s16 - s234 + s56)\)\)/2, 
          p2*p5 \[Rule] \(-\((\(-s16\) + s234 - s34 + s345)\)\)/2, 
          p2*p6 \[Rule] \(-\((s12 + s16 - s345)\)\)/2, 
          p4*p6 \[Rule] \(-\((\(-s123\) + s45 + s56)\)\)/2, 
          p1*p2 \[Rule] s12/2, p1*p3 \[Rule] \(-\((s12 - s123 + s23)\)\)/2, 
          p2*p3 \[Rule] s23/2, p2*p4 \[Rule] \(-\((s23 - s234 + s34)\)\)/2, 
          p3*p4 \[Rule] s34/2, p3*p5 \[Rule] \(-\((s34 - s345 + s45)\)\)/2, 
          p4*p5 \[Rule] s45/2, p1*p6 \[Rule] s16/2, p5*p6 \[Rule] s56/2, 
          p3*p6 \[Rule] s12/2 - s123/2 - s345/2 + s45/2};\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(IntPart[1]\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("numerator="\[InvisibleSpace]1\),
      SequenceForm[ "numerator=", 1],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("integral="\[InvisibleSpace]\(PR[k1, m, n1]\ PR[
            k1 + p1, m, n2]\ PR[k1 + p1 + p2, m, n3]\ PR[k1 + p1 + p2 + p3, 
            m, n4]\ PR[k1 + p1 + p2 + p3 + p4, m, n5]\ PR[
            k1 + p1 + p2 + p3 + p4 + p5, m, n6]\)\),
      SequenceForm[ "integral=", 
        Times[ 
          PR[ k1, m, n1], 
          PR[ 
            Plus[ k1, p1], m, n2], 
          PR[ 
            Plus[ k1, p1, p2], m, n3], 
          PR[ 
            Plus[ k1, p1, p2, p3], m, n4], 
          PR[ 
            Plus[ k1, p1, p2, p3, p4], m, n5], 
          PR[ 
            Plus[ k1, p1, p2, p3, p4, p5], m, n6]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("momentum="\[InvisibleSpace]k1\),
      SequenceForm[ "momentum=", k1],
      Editable->False]], "Print"],

Cell[BoxData[
    \(Fauto::"mode" \(\(:\)\(\ \)\) 
      "U and F polynomials will be calculated in AUTO mode. In order to use \
MANUAL mode execute Fauto[0]."\)], "Message"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(rep = SubLoop[integral];\)\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("Iteration nr"\[InvisibleSpace]1\[InvisibleSpace]": \
>>Integrating over "\[InvisibleSpace]k1\[InvisibleSpace]"<<"\),
      SequenceForm[ "Iteration nr", 1, ": >>Integrating over ", k1, "<<"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Computing U & F polynomial in AUTO mode >>Fauto["\
\[InvisibleSpace]1\[InvisibleSpace]"]<<"\),
      SequenceForm[ 
      "Computing U & F polynomial in AUTO mode >>Fauto[", 1, "]<<"],
      Editable->False]], "Print"],

Cell[BoxData[
    \("U polynomial..."\)], "Print"],

Cell[BoxData[
    \(X[1] + X[2] + X[3] + X[4] + X[5] + X[6]\)], "Print"],

Cell[BoxData[
    \("F polynomial..."\)], "Print"],

Cell[BoxData[
    \(m\^2\ FX[X[1] + X[2] + X[3] + X[4] + X[5] + X[6]]\^2 - 
      s12\ X[1]\ X[3] - s123\ X[1]\ X[4] - s23\ X[2]\ X[4] - 
      s56\ X[1]\ X[5] - s234\ X[2]\ X[5] - s34\ X[3]\ X[5] - 
      s16\ X[2]\ X[6] - s345\ X[3]\ X[6] - s45\ X[4]\ X[6]\)], "Print"],

Cell[BoxData[
    \("Final representation:"\)], "Print"],

Cell[BoxData[
    TagBox[
      StyleBox[\(\((\((\(-1\))\)\^\(n1 + n2 + n3 + n4 + n5 + n6\)\ \((m\^2)\)\
\^z1\ \((\(-s12\))\)\^z2\ \((\(-s123\))\)\^z3\ \((\(-s16\))\)\^z8\ \
\((\(-s23\))\)\^z4\ \((\(-s234\))\)\^z6\ \((\(-s34\))\)\^z7\ \
\((\(-s345\))\)\^z9\ \((\(-s45\))\)\^\(2 - eps - n1 - n2 - n3 - n4 - n5 - n6 \
- z1 - z2 - z3 - z4 - z5 - z6 - z7 - z8 - z9\)\ \((\(-s56\))\)\^z5\ \
Gamma[\(-z1\)]\ Gamma[\(-z10\)]\ Gamma[\(-z11\)]\ Gamma[\(-z12\)]\ \
Gamma[\(-z13\)]\ Gamma[\(-z14\)]\ Gamma[\(-2\)\ z1 + z10 + z11 + z12 + z13 + 
                  z14]\ Gamma[\(-z2\)]\ Gamma[\(-z3\)]\ Gamma[\(-z4\)]\ \
Gamma[\(-z5\)]\ Gamma[n1 + z10 + z2 + z3 + z5]\ Gamma[\(-z6\)]\ Gamma[
                2 - eps - n1 - n2 - n3 - n4 - n5 + z1 - z10 - z11 - z12 - 
                  z13 - z14 - z2 - z3 - z4 - z5 - z6 - 
                  z7]\ Gamma[\(-z7\)]\ Gamma[
                n5 + z14 + z5 + z6 + z7]\ Gamma[\(-z8\)]\ Gamma[
                n2 + z11 + z4 + z6 + z8]\ Gamma[
                2 - eps - n1 - n2 - n3 - n5 - n6 - z1 + z13 - z2 - z5 - z6 - 
                  z7 - z8 - z9]\ Gamma[\(-z9\)]\ Gamma[
                n3 + z12 + z2 + z7 + z9]\ Gamma[\(-2\) + eps + n1 + n2 + n3 + 
                  n4 + n5 + n6 + z1 + z2 + z3 + z4 + z5 + z6 + z7 + z8 + 
                  z9])\)/\((Gamma[n1]\ Gamma[n2]\ Gamma[n3]\ Gamma[n4]\ Gamma[
                n5]\ Gamma[4 - 2\ eps - n1 - n2 - n3 - n4 - n5 - n6]\ Gamma[
                n6]\ Gamma[\(-2\)\ z1])\)\),
        FontColor->RGBColor[0, 0, 1]],
      (StyleForm[ #, FontColor -> RGBColor[ 0, 0, 1]]&)]], "Print"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(fin = BarnesLemma[rep, 1]\)], "Input"],

Cell[BoxData[
    InterpretationBox[\(">> Barnes 1st Lemma will be checked for: "\
\[InvisibleSpace]{z14, z13, z12, z11, z10}\[InvisibleSpace]" <<"\),
      SequenceForm[ 
      ">> Barnes 1st Lemma will be checked for: ", {z14, z13, z12, z11, z10}, 
        " <<"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("   Starting with dim="\[InvisibleSpace]14\
\[InvisibleSpace]" representation..."\),
      SequenceForm[ "   Starting with dim=", 14, " representation..."],
      Editable->False]], "Print"],

Cell["\<\

1. Checking z14...Barnes Lemma was applied.
2. Checking z13...Barnes Lemma was applied.
3. Checking z12...Barnes Lemma was applied.
4. Checking z11...Barnes Lemma was applied.
5. Checking z10...Barnes Lemma was applied.\
\>", "Print"],

Cell[BoxData[
    \(">> Representation after 1st Barnes Lemma:  <<"\)], "Print"],

Cell[BoxData[
    InterpretationBox[
      RowBox[{
        TagBox[
          StyleBox["\<\"   1st Barnes Lemma was applied for: \"\>",
            FontColor->RGBColor[1, 0, 0]],
          (StyleForm[ #, FontColor -> RGBColor[ 1, 0, 0]]&)], 
        "\[InvisibleSpace]", \({z10, z11, z12, z13, z14}\)}],
      SequenceForm[ 
        StyleForm[ 
        "   1st Barnes Lemma was applied for: ", FontColor -> RGBColor[ 
          1, 0, 0]], {z10, z11, z12, z13, z14}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("   Obtained representation has: dim="\
\[InvisibleSpace]9\),
      SequenceForm[ "   Obtained representation has: dim=", 9],
      Editable->False]], "Print"],

Cell[BoxData[
    \(\((\((\(-1\))\)\^\(n1 + n2 + n3 + n4 + n5 + n6\)\ \((m\^2)\)\^z1\ \
\((\(-s12\))\)\^z2\ \((\(-s123\))\)\^z3\ \((\(-s16\))\)\^z8\ \
\((\(-s23\))\)\^z4\ \((\(-s234\))\)\^z6\ \((\(-s34\))\)\^z7\ \
\((\(-s345\))\)\^z9\ \((\(-s45\))\)\^\(2 - eps - n1 - n2 - n3 - n4 - n5 - n6 \
- z1 - z2 - z3 - z4 - z5 - z6 - z7 - z8 - z9\)\ \((\(-s56\))\)\^z5\ \
Gamma[\(-z1\)]\ Gamma[\(-z2\)]\ Gamma[\(-z3\)]\ Gamma[\(-z4\)]\ \
Gamma[\(-z5\)]\ Gamma[n1 + z2 + z3 + z5]\ Gamma[\(-z6\)]\ Gamma[
            2 - eps - n1 - n2 - n3 - n4 - n5 - z1 - z2 - z3 - z4 - z5 - z6 - 
              z7]\ Gamma[\(-z7\)]\ Gamma[
            n5 + z5 + z6 + z7]\ Gamma[\(-z8\)]\ Gamma[
            n2 + z4 + z6 + z8]\ Gamma[
            2 - eps - n1 - n2 - n3 - n5 - n6 - z1 - z2 - z5 - z6 - z7 - z8 - 
              z9]\ Gamma[\(-z9\)]\ Gamma[
            n3 + z2 + z7 + z9]\ Gamma[\(-2\) + eps + n1 + n2 + n3 + n4 + n5 + 
              n6 + z1 + z2 + z3 + z4 + z5 + z6 + z7 + z8 + z9])\)/\((Gamma[
            n1]\ Gamma[n2]\ Gamma[n3]\ Gamma[n4]\ Gamma[n5]\ Gamma[n6]\ Gamma[
            4 - 2\ eps - n1 - n2 - n3 - n4 - n5 - n6 - 2\ z1])\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(fin = \(\(\(\(\(fin /. n1 \[Rule] 1\) /. n2 \[Rule] 1\) /. 
              n3 \[Rule] 1\) /. n4 \[Rule] 1\) /. n5 \[Rule] 1\) /. 
        n6 \[Rule] 1\)], "Input"],

Cell[BoxData[
    \(\(\(1\/Gamma[\(-2\) - 2\ eps - 
            2\ z1]\)\((\((m\^2)\)\^z1\ \((\(-s12\))\)\^z2\ \
\((\(-s123\))\)\^z3\ \((\(-s16\))\)\^z8\ \((\(-s23\))\)\^z4\ \
\((\(-s234\))\)\^z6\ \((\(-s34\))\)\^z7\ \((\(-s345\))\)\^z9\ \
\((\(-s45\))\)\^\(\(-4\) - eps - z1 - z2 - z3 - z4 - z5 - z6 - z7 - z8 - z9\)\
\ \((\(-s56\))\)\^z5\ Gamma[\(-z1\)]\ Gamma[\(-z2\)]\ Gamma[\(-z3\)]\ \
Gamma[\(-z4\)]\ Gamma[\(-z5\)]\ Gamma[
          1 + z2 + z3 + z5]\ Gamma[\(-z6\)]\ Gamma[\(-3\) - eps - z1 - z2 - 
            z3 - z4 - z5 - z6 - z7]\ Gamma[\(-z7\)]\ Gamma[
          1 + z5 + z6 + z7]\ Gamma[\(-z8\)]\ Gamma[
          1 + z4 + z6 + z8]\ Gamma[\(-3\) - eps - z1 - z2 - z5 - z6 - z7 - 
            z8 - z9]\ Gamma[\(-z9\)]\ Gamma[1 + z2 + z7 + z9]\ Gamma[
          4 + eps + z1 + z2 + z3 + z4 + z5 + z6 + z7 + z8 + 
            z9])\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(rules = MBoptimizedRules[fin, eps \[Rule] 0, {}, {eps}]\)], "Input"],

Cell[BoxData[
    \(MBresidues::"contour" \(\(:\)\(\ \)\) 
      "contour starts and/or ends on a pole of Gamma[\!\(\(\(-2\)\) - eps - \
z1 - z5 - z6 - z8\)]"\)], "Message"],

Cell[BoxData[
    \(MBresidues::"contour" \(\(:\)\(\ \)\) 
      "contour starts and/or ends on a pole of Gamma[\!\(\(\(-2\)\) - eps - \
z1 - z5 - z6 - z8\)]"\)], "Message"],

Cell[BoxData[
    \({{eps \[Rule] 3\/32}, {z1 \[Rule] \(-2\), z2 \[Rule] \(-\(1\/2\)\), 
        z3 \[Rule] \(-\(1\/8\)\), z4 \[Rule] \(-\(5\/32\)\), 
        z5 \[Rule] \(-\(1\/4\)\), z6 \[Rule] \(-\(3\/8\)\), 
        z7 \[Rule] \(-\(3\/16\)\), z8 \[Rule] \(-\(5\/32\)\), 
        z9 \[Rule] \(-\(5\/32\)\)}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(integrals = 
      MBcontinue[fin, eps \[Rule] 0, rules, Verbose \[Rule] False]\)], "Input"],

Cell[BoxData[
    \({MBint[\(\(1\/Gamma[\(-2\) - 2\ eps - 
                2\ z1]\)\((\((m\^2)\)\^z1\ \((\(-s12\))\)\^z2\ \
\((\(-s123\))\)\^z3\ \((\(-s16\))\)\^z8\ \((\(-s23\))\)\^z4\ \
\((\(-s234\))\)\^z6\ \((\(-s34\))\)\^z7\ \((\(-s345\))\)\^z9\ \
\((\(-s45\))\)\^\(\(-4\) - eps - z1 - z2 - z3 - z4 - z5 - z6 - z7 - z8 - z9\)\
\ \((\(-s56\))\)\^z5\ Gamma[\(-z1\)]\ Gamma[\(-z2\)]\ Gamma[\(-z3\)]\ \
Gamma[\(-z4\)]\ Gamma[\(-z5\)]\ Gamma[
              1 + z2 + z3 + z5]\ Gamma[\(-z6\)]\ Gamma[\(-3\) - eps - z1 - 
                z2 - z3 - z4 - z5 - z6 - z7]\ Gamma[\(-z7\)]\ Gamma[
              1 + z5 + z6 + z7]\ Gamma[\(-z8\)]\ Gamma[
              1 + z4 + z6 + z8]\ Gamma[\(-3\) - eps - z1 - z2 - z5 - z6 - 
                z7 - z8 - z9]\ Gamma[\(-z9\)]\ Gamma[1 + z2 + z7 + z9]\ Gamma[
              4 + eps + z1 + z2 + z3 + z4 + z5 + z6 + z7 + z8 + 
                z9])\)\), {{eps \[Rule] 0}, {z1 \[Rule] \(-2\), 
            z2 \[Rule] \(-\(1\/2\)\), z3 \[Rule] \(-\(1\/8\)\), 
            z4 \[Rule] \(-\(5\/32\)\), z5 \[Rule] \(-\(1\/4\)\), 
            z6 \[Rule] \(-\(3\/8\)\), z7 \[Rule] \(-\(3\/16\)\), 
            z8 \[Rule] \(-\(5\/32\)\), 
            z9 \[Rule] \(-\(5\/32\)\)}}]}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ser = 
      MBexpand[{integrals}, Exp[eps\ EulerGamma], {eps, 0, 0}]\)], "Input"],

Cell[BoxData[
    \({MBint[\(\(1\/\(s45\^4\ Gamma[\(-2\) - 
                  2\ z1]\)\)\((\((m\^2)\)\^z1\ \((\(-s12\))\)\^z2\ \((\(-s123\
\))\)\^z3\ \((\(-s16\))\)\^z8\ \((\(-s23\))\)\^z4\ \((\(-s234\))\)\^z6\ \
\((\(-s34\))\)\^z7\ \((\(-s345\))\)\^z9\ \((\(-s45\))\)\^\(\(-z1\) - z2 - z3 \
- z4 - z5 - z6 - z7 - z8 - z9\)\ \((\(-s56\))\)\^z5\ Gamma[\(-z1\)]\ \
Gamma[\(-z2\)]\ Gamma[\(-z3\)]\ Gamma[\(-z4\)]\ Gamma[\(-z5\)]\ Gamma[
              1 + z2 + z3 + z5]\ Gamma[\(-z6\)]\ Gamma[\(-3\) - z1 - z2 - 
                z3 - z4 - z5 - z6 - z7]\ Gamma[\(-z7\)]\ Gamma[
              1 + z5 + z6 + z7]\ Gamma[\(-z8\)]\ Gamma[
              1 + z4 + z6 + z8]\ Gamma[\(-3\) - z1 - z2 - z5 - z6 - z7 - z8 - 
                z9]\ Gamma[\(-z9\)]\ Gamma[1 + z2 + z7 + z9]\ Gamma[
              4 + z1 + z2 + z3 + z4 + z5 + z6 + z7 + z8 + 
                z9])\)\), {{eps \[Rule] 0}, {z1 \[Rule] \(-2\), 
            z2 \[Rule] \(-\(1\/2\)\), z3 \[Rule] \(-\(1\/8\)\), 
            z4 \[Rule] \(-\(5\/32\)\), z5 \[Rule] \(-\(1\/4\)\), 
            z6 \[Rule] \(-\(3\/8\)\), z7 \[Rule] \(-\(3\/16\)\), 
            z8 \[Rule] \(-\(5\/32\)\), 
            z9 \[Rule] \(-\(5\/32\)\)}}]}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(MBintegrate[
      ser, {m \[Rule] 1, s12 \[Rule] \(-1\), s23 \[Rule] \(-2\), 
        s34 \[Rule] \(-3\), s45 \[Rule] \(-4\), s56 \[Rule] \(-5\), 
        s16 \[Rule] \(-6\), s123 \[Rule] \(-7\), s234 \[Rule] \(-8\), 
        s345 \[Rule] \(-9\)}]\)], "Input"],

Cell[BoxData[
    \("Shifting contours..."\)], "Print"],

Cell["Performing 0 1-dimensional integrations", "Print"],

Cell[BoxData[
    \("Higher-dimensional integrals"\)], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart10"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]9\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart10", " (dim ", 9, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart10"\),
      SequenceForm[ "Running ", "MBpart10"],
      Editable->False]], "Print"],

Cell[BoxData[
    \({0.002423102238881344`, {0.00002507902279533536`, 0}}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \( (*\ 
      sectors\[IndentingNewLine]{6.124383`\ Second, {{0.0033576676452116046`, 
              3.7100296905997188`*^-6, 
              2.7662817910409344`*^-6}}}\[IndentingNewLine]*) \)], "Input"]
},
FrontEndVersion->"5.2 for X",
ScreenRectangle->{{0, 1280}, {0, 1024}},
WindowSize->{1014, 689},
WindowMargins->{{36, Automatic}, {Automatic, 0}}
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 40, 1, 27, "Input"],
Cell[1819, 56, 41, 1, 23, "Print"],
Cell[1863, 59, 51, 1, 23, "Print"],
Cell[1917, 62, 62, 1, 23, "Print"],
Cell[1982, 65, 58, 1, 23, "Print"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2077, 71, 43, 1, 27, "Input"],
Cell[2123, 74, 56, 1, 23, "Print"],
Cell[2182, 77, 60, 1, 23, "Print"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2279, 83, 51, 1, 27, "Input"],
Cell[2333, 86, 53, 1, 23, "Print"],
Cell[2389, 89, 64, 1, 23, "Print"]
}, Open  ]],
Cell[2468, 93, 291, 5, 59, "Input"],
Cell[2762, 100, 151, 2, 27, "Input"],

Cell[CellGroupData[{
Cell[2938, 106, 106, 2, 27, "Input"],
Cell[3047, 110, 355, 5, 20, "Message"],
Cell[3405, 117, 355, 5, 20, "Message"],
Cell[3763, 124, 355, 5, 20, "Message"],
Cell[4121, 131, 333, 5, 20, "Message"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4491, 141, 69, 1, 27, "Input"],
Cell[4563, 144, 814, 13, 152, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5414, 162, 43, 1, 27, "Input"],
Cell[5460, 165, 143, 3, 23, "Print"],
Cell[5606, 170, 682, 18, 39, "Print"],
Cell[6291, 190, 143, 3, 23, "Print"],
Cell[6437, 195, 174, 3, 20, "Message"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6648, 203, 61, 1, 27, "Input"],
Cell[6712, 206, 264, 4, 23, "Print"],
Cell[6979, 212, 257, 5, 23, "Print"],
Cell[7239, 219, 50, 1, 23, "Print"],
Cell[7292, 222, 72, 1, 23, "Print"],
Cell[7367, 225, 50, 1, 23, "Print"],
Cell[7420, 228, 214, 3, 23, "Print"],
Cell[7637, 233, 56, 1, 23, "Print"],
Cell[7696, 236, 1273, 21, 111, "Print"]
}, Open  ]],

Cell[CellGroupData[{
Cell[9006, 262, 58, 1, 27, "Input"],
Cell[9067, 265, 238, 4, 23, "Print"],
Cell[9308, 271, 226, 4, 23, "Print"],
Cell[9537, 277, 80, 1, 23, "Print"],
Cell[9620, 280, 190, 4, 23, "Print"],
Cell[9813, 286, 1104, 18, 115, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10954, 309, 182, 3, 27, "Input"],
Cell[11139, 314, 785, 12, 99, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[11961, 331, 88, 1, 27, "Input"],
Cell[12052, 334, 168, 3, 20, "Message"],
Cell[12223, 339, 173, 3, 20, "Message"],
Cell[12399, 344, 128, 2, 20, "Message"],
Cell[12530, 348, 128, 2, 20, "Message"],
Cell[12661, 352, 128, 2, 20, "Message"],
Cell[12792, 356, 334, 5, 20, "Message"],
Cell[13129, 363, 159, 3, 20, "Message"],
Cell[13291, 368, 337, 5, 20, "Message"],
Cell[13631, 375, 325, 5, 44, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[13993, 385, 130, 3, 27, "Input"],
Cell[14126, 390, 365, 5, 20, "Message"]
}, Open  ]],
Cell[14506, 398, 121, 3, 27, "Input"],

Cell[CellGroupData[{
Cell[14652, 405, 300, 6, 27, "Input"],
Cell[14955, 413, 55, 1, 23, "Print"],
Cell[15013, 416, 381, 7, 68, "Print"],
Cell[15397, 425, 63, 1, 23, "Print"],
Cell[15463, 428, 250, 4, 23, "Print"],
Cell[15716, 434, 250, 4, 23, "Print"],
Cell[15969, 440, 250, 4, 23, "Print"],
Cell[16222, 446, 250, 4, 23, "Print"],
Cell[16475, 452, 250, 4, 23, "Print"],
Cell[16728, 458, 250, 4, 23, "Print"],
Cell[16981, 464, 250, 4, 23, "Print"],
Cell[17234, 470, 250, 4, 23, "Print"],
Cell[17487, 476, 250, 4, 23, "Print"],
Cell[17740, 482, 252, 4, 23, "Print"],
Cell[17995, 488, 252, 4, 23, "Print"],
Cell[18250, 494, 252, 4, 23, "Print"],
Cell[18505, 500, 252, 4, 23, "Print"],
Cell[18760, 506, 252, 4, 23, "Print"],
Cell[19015, 512, 252, 4, 23, "Print"],
Cell[19270, 518, 252, 4, 23, "Print"],
Cell[19525, 524, 252, 4, 23, "Print"],
Cell[19780, 530, 252, 4, 23, "Print"],
Cell[20035, 536, 252, 4, 23, "Print"],
Cell[20290, 542, 252, 4, 23, "Print"],
Cell[20545, 548, 252, 4, 23, "Print"],
Cell[20800, 554, 252, 4, 23, "Print"],
Cell[21055, 560, 252, 4, 23, "Print"],
Cell[21310, 566, 252, 4, 23, "Print"],
Cell[21565, 572, 252, 4, 23, "Print"],
Cell[21820, 578, 252, 4, 23, "Print"],
Cell[22075, 584, 252, 4, 23, "Print"],
Cell[22330, 590, 252, 4, 23, "Print"],
Cell[22585, 596, 252, 4, 23, "Print"],
Cell[22840, 602, 252, 4, 23, "Print"],
Cell[23095, 608, 252, 4, 23, "Print"],
Cell[23350, 614, 252, 4, 23, "Print"],
Cell[23605, 620, 252, 4, 23, "Print"],
Cell[23860, 626, 252, 4, 23, "Print"],
Cell[24115, 632, 252, 4, 23, "Print"],
Cell[24370, 638, 252, 4, 23, "Print"],
Cell[24625, 644, 252, 4, 23, "Print"],
Cell[24880, 650, 252, 4, 23, "Print"],
Cell[25135, 656, 252, 4, 23, "Print"],
Cell[25390, 662, 252, 4, 23, "Print"],
Cell[25645, 668, 252, 4, 23, "Print"],
Cell[25900, 674, 252, 4, 23, "Print"],
Cell[26155, 680, 252, 4, 23, "Print"],
Cell[26410, 686, 252, 4, 23, "Print"],
Cell[26665, 692, 252, 4, 23, "Print"],
Cell[26920, 698, 252, 4, 23, "Print"],
Cell[27175, 704, 252, 4, 23, "Print"],
Cell[27430, 710, 252, 4, 23, "Print"],
Cell[27685, 716, 252, 4, 23, "Print"],
Cell[27940, 722, 252, 4, 23, "Print"],
Cell[28195, 728, 252, 4, 23, "Print"],
Cell[28450, 734, 252, 4, 23, "Print"],
Cell[28705, 740, 252, 4, 23, "Print"],
Cell[28960, 746, 252, 4, 23, "Print"],
Cell[29215, 752, 252, 4, 23, "Print"],
Cell[29470, 758, 252, 4, 23, "Print"],
Cell[29725, 764, 252, 4, 23, "Print"],
Cell[29980, 770, 252, 4, 23, "Print"],
Cell[30235, 776, 252, 4, 23, "Print"],
Cell[30490, 782, 252, 4, 23, "Print"],
Cell[30745, 788, 252, 4, 23, "Print"],
Cell[31000, 794, 252, 4, 23, "Print"],
Cell[31255, 800, 252, 4, 23, "Print"],
Cell[31510, 806, 252, 4, 23, "Print"],
Cell[31765, 812, 252, 4, 23, "Print"],
Cell[32020, 818, 252, 4, 23, "Print"],
Cell[32275, 824, 252, 4, 23, "Print"],
Cell[32530, 830, 252, 4, 23, "Print"],
Cell[32785, 836, 252, 4, 23, "Print"],
Cell[33040, 842, 254, 4, 23, "Print"],
Cell[33297, 848, 254, 4, 23, "Print"],
Cell[33554, 854, 254, 4, 23, "Print"],
Cell[33811, 860, 254, 4, 23, "Print"],
Cell[34068, 866, 254, 4, 23, "Print"],
Cell[34325, 872, 254, 4, 23, "Print"],
Cell[34582, 878, 163, 3, 23, "Print"],
Cell[34748, 883, 163, 3, 23, "Print"],
Cell[34914, 888, 163, 3, 23, "Print"],
Cell[35080, 893, 163, 3, 23, "Print"],
Cell[35246, 898, 163, 3, 23, "Print"],
Cell[35412, 903, 163, 3, 23, "Print"],
Cell[35578, 908, 163, 3, 23, "Print"],
Cell[35744, 913, 163, 3, 23, "Print"],
Cell[35910, 918, 163, 3, 23, "Print"],
Cell[36076, 923, 165, 3, 23, "Print"],
Cell[36244, 928, 165, 3, 23, "Print"],
Cell[36412, 933, 165, 3, 23, "Print"],
Cell[36580, 938, 165, 3, 23, "Print"],
Cell[36748, 943, 165, 3, 23, "Print"],
Cell[36916, 948, 165, 3, 23, "Print"],
Cell[37084, 953, 165, 3, 23, "Print"],
Cell[37252, 958, 165, 3, 23, "Print"],
Cell[37420, 963, 165, 3, 23, "Print"],
Cell[37588, 968, 165, 3, 23, "Print"],
Cell[37756, 973, 165, 3, 23, "Print"],
Cell[37924, 978, 165, 3, 23, "Print"],
Cell[38092, 983, 165, 3, 23, "Print"],
Cell[38260, 988, 165, 3, 23, "Print"],
Cell[38428, 993, 165, 3, 23, "Print"],
Cell[38596, 998, 165, 3, 23, "Print"],
Cell[38764, 1003, 165, 3, 23, "Print"],
Cell[38932, 1008, 165, 3, 23, "Print"],
Cell[39100, 1013, 165, 3, 23, "Print"],
Cell[39268, 1018, 165, 3, 23, "Print"],
Cell[39436, 1023, 165, 3, 23, "Print"],
Cell[39604, 1028, 165, 3, 23, "Print"],
Cell[39772, 1033, 165, 3, 23, "Print"],
Cell[39940, 1038, 165, 3, 23, "Print"],
Cell[40108, 1043, 165, 3, 23, "Print"],
Cell[40276, 1048, 165, 3, 23, "Print"],
Cell[40444, 1053, 165, 3, 23, "Print"],
Cell[40612, 1058, 165, 3, 23, "Print"],
Cell[40780, 1063, 165, 3, 23, "Print"],
Cell[40948, 1068, 165, 3, 23, "Print"],
Cell[41116, 1073, 165, 3, 23, "Print"],
Cell[41284, 1078, 165, 3, 23, "Print"],
Cell[41452, 1083, 165, 3, 23, "Print"],
Cell[41620, 1088, 165, 3, 23, "Print"],
Cell[41788, 1093, 165, 3, 23, "Print"],
Cell[41956, 1098, 165, 3, 23, "Print"],
Cell[42124, 1103, 165, 3, 23, "Print"],
Cell[42292, 1108, 165, 3, 23, "Print"],
Cell[42460, 1113, 165, 3, 23, "Print"],
Cell[42628, 1118, 165, 3, 23, "Print"],
Cell[42796, 1123, 165, 3, 23, "Print"],
Cell[42964, 1128, 165, 3, 23, "Print"],
Cell[43132, 1133, 165, 3, 23, "Print"],
Cell[43300, 1138, 165, 3, 23, "Print"],
Cell[43468, 1143, 165, 3, 23, "Print"],
Cell[43636, 1148, 165, 3, 23, "Print"],
Cell[43804, 1153, 165, 3, 23, "Print"],
Cell[43972, 1158, 165, 3, 23, "Print"],
Cell[44140, 1163, 165, 3, 23, "Print"],
Cell[44308, 1168, 165, 3, 23, "Print"],
Cell[44476, 1173, 165, 3, 23, "Print"],
Cell[44644, 1178, 165, 3, 23, "Print"],
Cell[44812, 1183, 165, 3, 23, "Print"],
Cell[44980, 1188, 165, 3, 23, "Print"],
Cell[45148, 1193, 165, 3, 23, "Print"],
Cell[45316, 1198, 165, 3, 23, "Print"],
Cell[45484, 1203, 165, 3, 23, "Print"],
Cell[45652, 1208, 165, 3, 23, "Print"],
Cell[45820, 1213, 165, 3, 23, "Print"],
Cell[45988, 1218, 165, 3, 23, "Print"],
Cell[46156, 1223, 167, 3, 23, "Print"],
Cell[46326, 1228, 167, 3, 23, "Print"],
Cell[46496, 1233, 167, 3, 23, "Print"],
Cell[46666, 1238, 167, 3, 23, "Print"],
Cell[46836, 1243, 167, 3, 23, "Print"],
Cell[47006, 1248, 167, 3, 23, "Print"],
Cell[47176, 1253, 291, 4, 49, "Output"]
}, Open  ]],
Cell[47482, 1260, 534, 8, 157, "Input"],
Cell[48019, 1270, 217, 3, 27, "Input"],
Cell[48239, 1275, 291, 5, 59, "Input"],
Cell[48533, 1282, 825, 12, 91, "Input"],

Cell[CellGroupData[{
Cell[49383, 1298, 43, 1, 27, "Input"],
Cell[49429, 1301, 143, 3, 23, "Print"],
Cell[49575, 1306, 682, 18, 39, "Print"],
Cell[50260, 1326, 143, 3, 23, "Print"],
Cell[50406, 1331, 174, 3, 20, "Message"]
}, Open  ]],

Cell[CellGroupData[{
Cell[50617, 1339, 61, 1, 27, "Input"],
Cell[50681, 1342, 264, 4, 23, "Print"],
Cell[50948, 1348, 257, 5, 23, "Print"],
Cell[51208, 1355, 50, 1, 23, "Print"],
Cell[51261, 1358, 72, 1, 23, "Print"],
Cell[51336, 1361, 50, 1, 23, "Print"],
Cell[51389, 1364, 271, 4, 41, "Print"],
Cell[51663, 1370, 56, 1, 23, "Print"],
Cell[51722, 1373, 1563, 24, 131, "Print"]
}, Open  ]],

Cell[CellGroupData[{
Cell[53322, 1402, 58, 1, 27, "Input"],
Cell[53383, 1405, 300, 6, 23, "Print"],
Cell[53686, 1413, 228, 4, 23, "Print"],
Cell[53917, 1419, 245, 7, 98, "Print"],
Cell[54165, 1428, 80, 1, 23, "Print"],
Cell[54248, 1431, 500, 12, 23, "Print"],
Cell[54751, 1445, 197, 4, 23, "Print"],
Cell[54951, 1451, 1140, 17, 119, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[56128, 1473, 182, 3, 27, "Input"],
Cell[56313, 1478, 860, 14, 124, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[57210, 1497, 88, 1, 27, "Input"],
Cell[57301, 1500, 173, 3, 20, "Message"],
Cell[57477, 1505, 173, 3, 20, "Message"],
Cell[57653, 1510, 324, 5, 44, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[58014, 1520, 112, 2, 27, "Input"],
Cell[58129, 1524, 1218, 19, 159, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[59384, 1548, 102, 2, 27, "Input"],
Cell[59489, 1552, 1193, 18, 158, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[60719, 1575, 281, 5, 27, "Input"],
Cell[61003, 1582, 55, 1, 23, "Print"],
Cell[61061, 1585, 56, 0, 23, "Print"],
Cell[61120, 1587, 63, 1, 23, "Print"],
Cell[61186, 1590, 244, 4, 23, "Print"],
Cell[61433, 1596, 157, 3, 23, "Print"],
Cell[61593, 1601, 87, 1, 27, "Output"]
}, Open  ]],
Cell[61695, 1605, 220, 4, 59, "Input"]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

