(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.1'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    258207,       4967]*)
(*NotebookOutlinePosition[    259148,       4996]*)
(*  CellTagsIndexPosition[    259104,       4992]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[BoxData[
    \(<< MB.m\)], "Input"],

Cell[BoxData[
    \("MB 1.1"\)], "Print"],

Cell[BoxData[
    \("by Michal Czakon"\)], "Print"],

Cell[BoxData[
    \("more info in hep-ph/0511200"\)], "Print"],

Cell[BoxData[
    \("last modified 12 Dec 06"\)], "Print"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(<< AMBRE.m\)], "Input"],

Cell[BoxData[
    \("by K.Kajda   ver: 1.0"\)], "Print"],

Cell[BoxData[
    \("last modified 19 Apr 2007"\)], "Print"]
}, Open  ]],

Cell[BoxData[
    \( (*\[IndentingNewLine]\ \ \ \ 2 - 
        loop\ massive\ topology, \ \ Fig  .5, \ 
      with\ a\ special\ numerator\ \ \ \ \[IndentingNewLine]*) \)], "Input"],

Cell[BoxData[
    \(\(int = 
        Fullintegral[\[IndentingNewLine]{1}, \[IndentingNewLine]{PR[k1, m, 
                n1] PR[k1 + p1, 0, n2] PR[k1 + p1 + p2, m, n3] 
              PR[k1 - k2, 0, n4] PR[k2, m, n5] PR[k2 + p1 + p2, m, n6] 
              PR[k2 + p1 + p2 + p4, 0, n7]\ PR[k1 + p1 + p2 + p4, 
                0, \(-n8\)]}, {k2, k1}];\)\)], "Input"],

Cell[BoxData[
    \(\(invariants = {p1^2 \[Rule] m^2, p2^2 \[Rule] m^2, p3^2 \[Rule] m^2, 
            p4^2 \[Rule] m^2, p1*p2 \[Rule] 1/2*s - m^2, 
            p3*p4 \[Rule] 1/2\ s - m^2, p1*p3 \[Rule] 1/2*t - m^2, 
            p2*p4 \[Rule] 1/2*t - m^2, p2*p3 \[Rule] 1/2*u - m^2, 
            p1*p4 \[Rule] 1/2*u - m^2} /. 
          u \[Rule] 4\ m^2 - s - t;\)\)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(IntPart[1]\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("numerator="\[InvisibleSpace]1\),
      SequenceForm[ "numerator=", 1],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("integral="\[InvisibleSpace]\(PR[k1 - k2, 0, n4]\ PR[
            k2, m, n5]\ PR[k2 + p1 + p2, m, n6]\ PR[k2 + p1 + p2 + p4, 0, 
            n7]\)\),
      SequenceForm[ "integral=", 
        Times[ 
          PR[ 
            Plus[ k1, 
              Times[ -1, k2]], 0, n4], 
          PR[ k2, m, n5], 
          PR[ 
            Plus[ k2, p1, p2], m, n6], 
          PR[ 
            Plus[ k2, p1, p2, p4], 0, n7]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("momentum="\[InvisibleSpace]k2\),
      SequenceForm[ "momentum=", k2],
      Editable->False]], "Print"],

Cell[BoxData[
    \(Fauto::"mode" \(\(:\)\(\ \)\) 
      "U and F polynomials will be calculated in AUTO mode. In order to use \
MANUAL mode execute Fauto[0]."\)], "Message"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(SubLoop[integral]\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("Iteration nr"\[InvisibleSpace]1\[InvisibleSpace]": \
>>Integrating over "\[InvisibleSpace]k2\[InvisibleSpace]"<<"\),
      SequenceForm[ "Iteration nr", 1, ": >>Integrating over ", k2, "<<"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Computing U & F polynomial in AUTO mode >>Fauto["\
\[InvisibleSpace]1\[InvisibleSpace]"]<<"\),
      SequenceForm[ 
      "Computing U & F polynomial in AUTO mode >>Fauto[", 1, "]<<"],
      Editable->False]], "Print"],

Cell[BoxData[
    \("U polynomial..."\)], "Print"],

Cell[BoxData[
    \(X[1] + X[2] + X[3] + X[4]\)], "Print"],

Cell[BoxData[
    \("F polynomial..."\)], "Print"],

Cell[BoxData[
    \(m\^2\ FX[X[2] + X[3]]\^2 - PR[k1, m]\ X[1]\ X[2] - 
      PR[k1 + p1 + p2, m]\ X[1]\ X[3] - s\ X[2]\ X[3] - 
      PR[k1 + p1 + p2 + p4, 0]\ X[1]\ X[4]\)], "Print"],

Cell[BoxData[
    InterpretationBox[\("Representation after integrating over: "\
\[InvisibleSpace]"k2..."\),
      SequenceForm[ "Representation after integrating over: ", "k2..."],
      Editable->False]], "Print"],

Cell[BoxData[
    RowBox[{"SubLoop1", "[", 
      RowBox[{
        TagBox[
          
          StyleBox[\(\((\((\(-1\))\)\^\(2 - eps - z1 - z4\)\ \((m\^2)\)\^z1\ \
\((\(-s\))\)\^z4\ Gamma[\(-z1\)]\ Gamma[\(-z2\)]\ Gamma[\(-z3\)]\ Gamma[
                    2 - eps - n5 - n6 - n7 - z1 - z4]\ Gamma[
                    2 - eps - n4 - n5 - n6 - z1 - z2 - z3 - 
                      z4]\ Gamma[\(-z4\)]\ Gamma[\(-2\) + eps + n4 + n5 + 
                      n6 + n7 + z1 + z2 + z3 + z4]\ Gamma[
                    n6 + 2\ z1 + z3 + z4 - 
                      z5]\ Gamma[\(-z5\)]\ Gamma[\(-2\)\ z1 + z5]\ Gamma[
                    n5 + z2 + z4 + z5])\)/\((Gamma[n4]\ Gamma[n5]\ Gamma[
                    n6]\ Gamma[4 - 2\ eps - n4 - n5 - n6 - n7]\ Gamma[
                    n7]\ Gamma[\(-2\)\ z1])\)\),
            FontColor->RGBColor[0, 0, 1]],
          (StyleForm[ #, FontColor -> RGBColor[ 0, 0, 1]]&)], ",", 
        TagBox[
          
          StyleBox[\(PR[k1, m, z2]\ PR[k1 + p1 + p2, m, z3]\ PR[
                k1 + p1 + p2 + p4, 0, 
                2 - eps - n4 - n5 - n6 - n7 - z1 - z2 - z3 - z4]\),
            FontColor->RGBColor[1, 0, 0]],
          (StyleForm[ #, FontColor -> RGBColor[ 1, 0, 0]]&)]}], 
      "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(IntPart[2]\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("numerator="\[InvisibleSpace]1\),
      SequenceForm[ "numerator=", 1],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("integral="\[InvisibleSpace]\(PR[k1, m, n1 - z2]\ PR[
            k1 + p1, 0, n2]\ PR[k1 + p1 + p2, m, n3 - z3]\ PR[
            k1 + p1 + p2 + p4, 
            0, \(-2\) + eps + n4 + n5 + n6 + n7 - n8 + z1 + z2 + z3 + z4]\)\),
      
      SequenceForm[ "integral=", 
        Times[ 
          PR[ k1, m, 
            Plus[ n1, 
              Times[ -1, z2]]], 
          PR[ 
            Plus[ k1, p1], 0, n2], 
          PR[ 
            Plus[ k1, p1, p2], m, 
            Plus[ n3, 
              Times[ -1, z3]]], 
          PR[ 
            Plus[ k1, p1, p2, p4], 0, 
            Plus[ -2, eps, n4, n5, n6, n7, 
              Times[ -1, n8], z1, z2, z3, z4]]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("momentum="\[InvisibleSpace]k1\),
      SequenceForm[ "momentum=", k1],
      Editable->False]], "Print"],

Cell[BoxData[
    \(Fauto::"mode" \(\(:\)\(\ \)\) 
      "U and F polynomials will be calculated in AUTO mode. In order to use \
MANUAL mode execute Fauto[0]."\)], "Message"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(repr = SubLoop[integral];\)\)], "Input"],

Cell[BoxData[
    InterpretationBox[\("Iteration nr"\[InvisibleSpace]2\[InvisibleSpace]": \
>>Integrating over "\[InvisibleSpace]k1\[InvisibleSpace]"<<"\),
      SequenceForm[ "Iteration nr", 2, ": >>Integrating over ", k1, "<<"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Computing U & F polynomial in AUTO mode >>Fauto["\
\[InvisibleSpace]1\[InvisibleSpace]"]<<"\),
      SequenceForm[ 
      "Computing U & F polynomial in AUTO mode >>Fauto[", 1, "]<<"],
      Editable->False]], "Print"],

Cell[BoxData[
    \("U polynomial..."\)], "Print"],

Cell[BoxData[
    \(X[1] + X[2] + X[3] + X[4]\)], "Print"],

Cell[BoxData[
    \("F polynomial..."\)], "Print"],

Cell[BoxData[
    \(m\^2\ FX[X[1] + X[3]]\^2 - s\ X[1]\ X[3] - t\ X[2]\ X[4]\)], "Print"],

Cell[BoxData[
    \("Final representation:"\)], "Print"],

Cell[BoxData[
    TagBox[
      StyleBox[\(\((\((\(-1\))\)\^\(n1 + n2 + n3 + n4 + n5 + n6 + n7 - n8\)\ \
\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(z4 + z7\)\ \((\(-t\))\)\^\(4 - 2\ \
eps - n1 - n2 - n3 - n4 - n5 - n6 - n7 + n8 - z1 - z4 - z6 - z7\)\ \
Gamma[\(-z1\)]\ Gamma[\(-z2\)]\ Gamma[\(-z3\)]\ Gamma[
                2 - eps - n5 - n6 - n7 - z1 - z4]\ Gamma[
                2 - eps - n4 - n5 - n6 - z1 - z2 - z3 - 
                  z4]\ Gamma[\(-z4\)]\ Gamma[\(-2\) + eps + n4 + n5 + n6 + 
                  n7 + z1 + z2 + z3 + z4]\ Gamma[
                n6 + 2\ z1 + z3 + z4 - 
                  z5]\ Gamma[\(-z5\)]\ Gamma[\(-2\)\ z1 + z5]\ Gamma[
                n5 + z2 + z4 + z5]\ Gamma[\(-z6\)]\ Gamma[
                2 - eps - n1 - n2 - n3 + z2 + z3 - z6 - z7]\ Gamma[
                4 - 2\ eps - n1 - n3 - n4 - n5 - n6 - n7 + n8 - z1 - z4 - 
                  z6 - z7]\ Gamma[\(-z7\)]\ Gamma[\(-4\) + 2\ eps + n1 + n2 + 
                  n3 + n4 + n5 + n6 + n7 - n8 + z1 + z4 + z6 + z7]\ Gamma[
                n3 - z3 + 2\ z6 + z7 - 
                  z8]\ Gamma[\(-z8\)]\ Gamma[\(-2\)\ z6 + z8]\ Gamma[
                n1 - z2 + z7 + z8])\)/\((Gamma[n2]\ Gamma[n4]\ Gamma[
                n5]\ Gamma[n6]\ Gamma[4 - 2\ eps - n4 - n5 - n6 - n7]\ Gamma[
                n7]\ Gamma[\(-2\)\ z1]\ Gamma[n1 - z2]\ Gamma[n3 - z3]\ Gamma[
                6 - 3\ eps - n1 - n2 - n3 - n4 - n5 - n6 - n7 + n8 - z1 - 
                  z4]\ Gamma[\(-2\) + eps + n4 + n5 + n6 + n7 - n8 + z1 + 
                  z2 + z3 + z4]\ Gamma[\(-2\)\ z6])\)\),
        FontColor->RGBColor[0, 0, 1]],
      (StyleForm[ #, FontColor -> RGBColor[ 0, 0, 1]]&)]], "Print"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(rep1 = BarnesLemma[repr, 1]\)], "Input"],

Cell[BoxData[
    InterpretationBox[\(">> Barnes 1st Lemma will be checked for: "\
\[InvisibleSpace]{z8, z5, z3, z2}\[InvisibleSpace]" <<"\),
      SequenceForm[ 
      ">> Barnes 1st Lemma will be checked for: ", {z8, z5, z3, z2}, " <<"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("   Starting with dim="\[InvisibleSpace]8\
\[InvisibleSpace]" representation..."\),
      SequenceForm[ "   Starting with dim=", 8, " representation..."],
      Editable->False]], "Print"],

Cell["\<\

1. Checking z8...Barnes Lemma was applied.
2. Checking z5...Barnes Lemma was applied.
3. Checking z3
4. Checking z2\
\>", "Print"],

Cell[BoxData[
    \(">> Representation after 1st Barnes Lemma:  <<"\)], "Print"],

Cell[BoxData[
    InterpretationBox[
      RowBox[{
        TagBox[
          StyleBox["\<\"   1st Barnes Lemma was applied for: \"\>",
            FontColor->RGBColor[1, 0, 0]],
          (StyleForm[ #, FontColor -> RGBColor[ 1, 0, 0]]&)], 
        "\[InvisibleSpace]", \({z5, z8}\)}],
      SequenceForm[ 
        StyleForm[ 
        "   1st Barnes Lemma was applied for: ", FontColor -> RGBColor[ 
          1, 0, 0]], {z5, z8}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("   Obtained representation has: dim="\
\[InvisibleSpace]6\),
      SequenceForm[ "   Obtained representation has: dim=", 6],
      Editable->False]], "Print"],

Cell[BoxData[
    \(\((\((\(-1\))\)\^\(n1 + n2 + n3 + n4 + n5 + n6 + n7 - n8\)\ \
\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(z4 + z7\)\ \((\(-t\))\)\^\(4 - 2\ \
eps - n1 - n2 - n3 - n4 - n5 - n6 - n7 + n8 - z1 - z4 - z6 - z7\)\ \
Gamma[\(-z1\)]\ Gamma[\(-z2\)]\ Gamma[\(-z3\)]\ Gamma[
            2 - eps - n5 - n6 - n7 - z1 - z4]\ Gamma[
            2 - eps - n4 - n5 - n6 - z1 - z2 - z3 - 
              z4]\ Gamma[\(-z4\)]\ Gamma[n5 + z2 + z4]\ Gamma[
            n6 + z3 + z4]\ Gamma[\(-2\) + eps + n4 + n5 + n6 + n7 + z1 + z2 + 
              z3 + z4]\ Gamma[
            n5 + n6 + 2\ z1 + z2 + z3 + 2\ z4]\ Gamma[\(-z6\)]\ Gamma[
            2 - eps - n1 - n2 - n3 + z2 + z3 - z6 - z7]\ Gamma[
            4 - 2\ eps - n1 - n3 - n4 - n5 - n6 - n7 + n8 - z1 - z4 - z6 - 
              z7]\ Gamma[\(-z7\)]\ Gamma[n1 - z2 + z7]\ Gamma[
            n3 - z3 + z7]\ Gamma[\(-4\) + 2\ eps + n1 + n2 + n3 + n4 + n5 + 
              n6 + n7 - n8 + z1 + z4 + z6 + z7]\ Gamma[
            n1 + n3 - z2 - z3 + 2\ z6 + 2\ z7])\)/\((Gamma[n2]\ Gamma[
            n4]\ Gamma[n5]\ Gamma[n6]\ Gamma[
            4 - 2\ eps - n4 - n5 - n6 - n7]\ Gamma[n7]\ Gamma[n1 - z2]\ Gamma[
            n3 - z3]\ Gamma[
            6 - 3\ eps - n1 - n2 - n3 - n4 - n5 - n6 - n7 + n8 - z1 - 
              z4]\ Gamma[\(-2\) + eps + n4 + n5 + n6 + n7 - n8 + z1 + z2 + 
              z3 + z4]\ Gamma[n5 + n6 + z2 + z3 + 2\ z4]\ Gamma[
            n1 + n3 - z2 - z3 + 2\ z7])\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(fin = 
      rep1\  /. \ {n1 \[Rule] 1, n2 \[Rule] 1, n3 \[Rule] 1, n4 \[Rule] 1, 
          n5 \[Rule] 1, n6 \[Rule] 1, n7 \[Rule] 1, n7 \[Rule] 1, 
          n8 \[Rule] 1}\)], "Input"],

Cell[BoxData[
    \(\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(z4 + z7\)\ \((\(-t\))\)\^\(\
\(-2\) - 2\ eps - z1 - z4 - z6 - z7\)\ Gamma[\(-z1\)]\ Gamma[\(-z2\)]\ Gamma[\
\(-z3\)]\ Gamma[\(-1\) - eps - z1 - z4]\ Gamma[\(-1\) - eps - z1 - z2 - z3 - 
              z4]\ Gamma[\(-z4\)]\ Gamma[1 + z2 + z4]\ Gamma[
            1 + z3 + z4]\ Gamma[2 + eps + z1 + z2 + z3 + z4]\ Gamma[
            2 + 2\ z1 + z2 + z3 + 2\ z4]\ Gamma[\(-z6\)]\ Gamma[\(-1\) - 
              eps + z2 + z3 - z6 - z7]\ Gamma[\(-1\) - 2\ eps - z1 - z4 - 
              z6 - z7]\ Gamma[\(-z7\)]\ Gamma[1 - z2 + z7]\ Gamma[
            1 - z3 + z7]\ Gamma[2 + 2\ eps + z1 + z4 + z6 + z7]\ Gamma[
            2 - z2 - z3 + 2\ z6 + 2\ z7])\)/\((Gamma[\(-2\)\ eps]\ Gamma[
            1 - z2]\ Gamma[1 - z3]\ Gamma[\(-3\)\ eps - z1 - z4]\ Gamma[
            1 + eps + z1 + z2 + z3 + z4]\ Gamma[2 + z2 + z3 + 2\ z4]\ Gamma[
            2 - z2 - z3 + 2\ z7])\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(MBoptimizedRules[fin, eps \[Rule] 0, {}, {eps}]\)], "Input"],

Cell[BoxData[
    \(MBrules::"norules" \(\(:\)\(\ \)\) 
      "no rules could be found to regulate this integral"\)], "Message"],

Cell[BoxData[
    \({}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(fin = 
      rep1\  /. \ {n1 \[Rule] 1, n2 \[Rule] 1, n3 \[Rule] 1, n4 \[Rule] 1, 
          n5 \[Rule] 1, n6 \[Rule] 1, n7 \[Rule] 1, n7 \[Rule] 1, 
          n8 \[Rule] 1 + eta}\)], "Input"],

Cell[BoxData[
    \(\((\((\(-1\))\)\^\(6 - eta\)\ \((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(z4 + z7\)\ \((\(-t\))\)\^\(\(-2\) - 2\ eps + eta - z1 - z4 - \
z6 - z7\)\ Gamma[\(-z1\)]\ Gamma[\(-z2\)]\ Gamma[\(-z3\)]\ Gamma[\(-1\) - 
              eps - z1 - z4]\ Gamma[\(-1\) - eps - z1 - z2 - z3 - 
              z4]\ Gamma[\(-z4\)]\ Gamma[1 + z2 + z4]\ Gamma[
            1 + z3 + z4]\ Gamma[2 + eps + z1 + z2 + z3 + z4]\ Gamma[
            2 + 2\ z1 + z2 + z3 + 2\ z4]\ Gamma[\(-z6\)]\ Gamma[\(-1\) - 
              eps + z2 + z3 - z6 - z7]\ Gamma[\(-1\) - 2\ eps + eta - z1 - 
              z4 - z6 - z7]\ Gamma[\(-z7\)]\ Gamma[1 - z2 + z7]\ Gamma[
            1 - z3 + z7]\ Gamma[2 + 2\ eps - eta + z1 + z4 + z6 + z7]\ Gamma[
            2 - z2 - z3 + 2\ z6 + 2\ z7])\)/\((Gamma[\(-2\)\ eps]\ Gamma[
            1 - z2]\ Gamma[1 - z3]\ Gamma[\(-3\)\ eps + eta - z1 - z4]\ Gamma[
            1 + eps - eta + z1 + z2 + z3 + z4]\ Gamma[
            2 + z2 + z3 + 2\ z4]\ Gamma[2 - z2 - z3 + 2\ z7])\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(rules = 
      MBoptimizedRules[fin, 
        eta \[Rule] 0, {z1 > \(-1\)/2, z2 > z1}, {eps, eta}]\)], "Input"],

Cell[BoxData[
    \(MBrules::"norules" \(\(:\)\(\ \)\) 
      "no rules could be found to regulate this integral"\)], "Message"],

Cell[BoxData[
    \({{eps \[Rule] \(-\(265\/512\)\), 
        eta \[Rule] \(-\(1025\/1024\)\)}, {z1 \[Rule] \(-\(1\/4\)\), 
        z2 \[Rule] \(-\(1\/8\)\), z3 \[Rule] \(-\(19\/128\)\), 
        z4 \[Rule] \(-\(69\/256\)\), z6 \[Rule] \(-\(19\/64\)\), 
        z7 \[Rule] \(-\(665\/1024\)\)}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Step1cont = MBcontinue[fin, eta \[Rule] 0, rules];\)\)], "Input"],

Cell[BoxData[
    \("Level 1"\)], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"+"\[InvisibleSpace]"residue in "\[InvisibleSpace]z7\
\[InvisibleSpace]" = "\[InvisibleSpace]\(\(-2\) - 2\ eps + eta - z1 - z4 - 
          z6\)\),
      SequenceForm[ "Taking ", "+", "residue in ", z7, " = ", 
        Plus[ -2, 
          Times[ -2, eps], eta, 
          Times[ -1, z1], 
          Times[ -1, z4], 
          Times[ -1, z6]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Level "\[InvisibleSpace]2\),
      SequenceForm[ "Level ", 2],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{1}\),
      SequenceForm[ "Integral ", {1}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\(2\[InvisibleSpace]" integral(s) found"\),
      SequenceForm[ 2, " integral(s) found"],
      Editable->False]], "Print"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(after = MBexpand[Step1cont, 1, {eta, 0, 0}]\)], "Input"],

Cell[BoxData[
    RowBox[{\(General::"spell1"\), \(\(:\)\(\ \)\), "\<\"Possible spelling \
error: new symbol name \\\"\\!\\(after\\)\\\" is similar to existing symbol \
\\\"\\!\\(After\\)\\\". \\!\\(\\*ButtonBox[\\\"More\[Ellipsis]\\\", \
ButtonStyle->\\\"RefGuideLinkText\\\", ButtonFrame->None, \
ButtonData:>\\\"General::spell1\\\"]\\)\"\>"}]], "Message"],

Cell[BoxData[
    \({MBint[\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-2\)\ eps - z1 - \
z6\)\ Gamma[\(-z1\)]\ Gamma[\(-z2\)]\ Gamma[\(-z3\)]\ Gamma[\(-2\) - 4\ eps - 
                  2\ z1 - z2 - z3 - 2\ z4]\ Gamma[\(-1\) - eps - z1 - 
                  z4]\ Gamma[\(-1\) - eps - z1 - z2 - z3 - 
                  z4]\ Gamma[\(-z4\)]\ Gamma[1 + z2 + z4]\ Gamma[
                1 + z3 + z4]\ Gamma[2 + eps + z1 + z2 + z3 + z4]\ Gamma[
                2 + 2\ z1 + z2 + z3 + 2\ z4]\ Gamma[\(-1\) - 2\ eps - z1 - 
                  z2 - z4 - z6]\ Gamma[\(-1\) - 2\ eps - z1 - z3 - z4 - 
                  z6]\ Gamma[\(-z6\)]\ Gamma[
                2 + 2\ eps + z1 + z4 + 
                  z6])\)/\((s\^2\ Gamma[\(-2\)\ eps]\ Gamma[1 - z2]\ Gamma[
                1 - z3]\ Gamma[\(-3\)\ eps - z1 - z4]\ Gamma[
                2 + z2 + z3 + 2\ z4]\ Gamma[\(-2\) - 4\ eps - 2\ z1 - z2 - 
                  z3 - 2\ z4 - 2\ z6])\), {{eps \[Rule] \(-\(265\/512\)\), 
            eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
            z2 \[Rule] \(-\(1\/8\)\), z3 \[Rule] \(-\(19\/128\)\), 
            z4 \[Rule] \(-\(69\/256\)\), z6 \[Rule] \(-\(19\/64\)\)}}], 
      MBint[\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(z4 + z7\)\ \((\(-t\))\
\)\^\(\(-2\)\ eps - z1 - z4 - z6 - z7\)\ Gamma[\(-z1\)]\ Gamma[\(-z2\)]\ \
Gamma[\(-z3\)]\ Gamma[\(-1\) - eps - z1 - z4]\ Gamma[\(-1\) - eps - z1 - z2 - 
                  z3 - z4]\ Gamma[\(-z4\)]\ Gamma[1 + z2 + z4]\ Gamma[
                1 + z3 + z4]\ Gamma[2 + eps + z1 + z2 + z3 + z4]\ Gamma[
                2 + 2\ z1 + z2 + z3 + 2\ z4]\ Gamma[\(-z6\)]\ Gamma[\(-1\) - 
                  eps + z2 + z3 - z6 - z7]\ Gamma[\(-1\) - 2\ eps - z1 - z4 - 
                  z6 - z7]\ Gamma[\(-z7\)]\ Gamma[1 - z2 + z7]\ Gamma[
                1 - z3 + z7]\ Gamma[2 + 2\ eps + z1 + z4 + z6 + z7]\ Gamma[
                2 - z2 - z3 + 2\ z6 + 
                  2\ z7])\)/\((t\^2\ Gamma[\(-2\)\ eps]\ Gamma[1 - z2]\ Gamma[
                1 - z3]\ Gamma[\(-3\)\ eps - z1 - z4]\ Gamma[
                1 + eps + z1 + z2 + z3 + z4]\ Gamma[
                2 + z2 + z3 + 2\ z4]\ Gamma[
                2 - z2 - z3 + 2\ z7])\), {{eps \[Rule] \(-\(265\/512\)\), 
            eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
            z2 \[Rule] \(-\(1\/8\)\), z3 \[Rule] \(-\(19\/128\)\), 
            z4 \[Rule] \(-\(69\/256\)\), z6 \[Rule] \(-\(19\/64\)\), 
            z7 \[Rule] \(-\(665\/1024\)\)}}]}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(integrals = 
      after /. MBint[integrand_, rules_] \[RuleDelayed] 
          MBcontinue[integrand, eps \[Rule] 0, rules]\)], "Input"],

Cell[BoxData[
    RowBox[{\(General::"spell1"\), \(\(:\)\(\ \)\), "\<\"Possible spelling \
error: new symbol name \\\"\\!\\(integrals\\)\\\" is similar to existing \
symbol \\\"\\!\\(integral\\)\\\". \\!\\(\\*ButtonBox[\\\"More\[Ellipsis]\\\", \
ButtonStyle->\\\"RefGuideLinkText\\\", ButtonFrame->None, \
ButtonData:>\\\"General::spell1\\\"]\\)\"\>"}]], "Message"],

Cell[BoxData[
    \("Level 1"\)], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"-"\[InvisibleSpace]"residue in "\[InvisibleSpace]z3\
\[InvisibleSpace]" = "\[InvisibleSpace]\(\(-2\) - 4\ eps - 2\ z1 - z2 - 
          2\ z4\)\),
      SequenceForm[ "Taking ", "-", "residue in ", z3, " = ", 
        Plus[ -2, 
          Times[ -4, eps], 
          Times[ -2, z1], 
          Times[ -1, z2], 
          Times[ -2, z4]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"-"\[InvisibleSpace]"residue in "\[InvisibleSpace]z4\
\[InvisibleSpace]" = "\[InvisibleSpace]\(\(-1\) - eps - z1\)\),
      SequenceForm[ "Taking ", "-", "residue in ", z4, " = ", 
        Plus[ -1, 
          Times[ -1, eps], 
          Times[ -1, z1]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"-"\[InvisibleSpace]"residue in "\[InvisibleSpace]z4\
\[InvisibleSpace]" = "\[InvisibleSpace]\(\(-1\) - eps - z1 - z2 - z3\)\),
      SequenceForm[ "Taking ", "-", "residue in ", z4, " = ", 
        Plus[ -1, 
          Times[ -1, eps], 
          Times[ -1, z1], 
          Times[ -1, z2], 
          Times[ -1, z3]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"-"\[InvisibleSpace]"residue in "\[InvisibleSpace]z6\
\[InvisibleSpace]" = "\[InvisibleSpace]\(\(-1\) - 2\ eps - z1 - z2 - z4\)\),
      SequenceForm[ "Taking ", "-", "residue in ", z6, " = ", 
        Plus[ -1, 
          Times[ -2, eps], 
          Times[ -1, z1], 
          Times[ -1, z2], 
          Times[ -1, z4]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"-"\[InvisibleSpace]"residue in "\[InvisibleSpace]z6\
\[InvisibleSpace]" = "\[InvisibleSpace]\(\(-1\) - 2\ eps - z1 - z3 - z4\)\),
      SequenceForm[ "Taking ", "-", "residue in ", z6, " = ", 
        Plus[ -1, 
          Times[ -2, eps], 
          Times[ -1, z1], 
          Times[ -1, z3], 
          Times[ -1, z4]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Level "\[InvisibleSpace]2\),
      SequenceForm[ "Level ", 2],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{1}\),
      SequenceForm[ "Integral ", {1}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"-"\[InvisibleSpace]"residue in "\[InvisibleSpace]z4\
\[InvisibleSpace]" = "\[InvisibleSpace]\(\(-1\) - 4\ eps - 2\ z1 - z2\)\),
      SequenceForm[ "Taking ", "-", "residue in ", z4, " = ", 
        Plus[ -1, 
          Times[ -4, eps], 
          Times[ -2, z1], 
          Times[ -1, z2]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"-"\[InvisibleSpace]"residue in "\[InvisibleSpace]z4\
\[InvisibleSpace]" = "\[InvisibleSpace]\(\(-1\) - 3\ eps - z1\)\),
      SequenceForm[ "Taking ", "-", "residue in ", z4, " = ", 
        Plus[ -1, 
          Times[ -3, eps], 
          Times[ -1, z1]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"-"\[InvisibleSpace]"residue in "\[InvisibleSpace]z6\
\[InvisibleSpace]" = "\[InvisibleSpace]\(\(-1\) - 2\ eps - z1 - z2 - z4\)\),
      SequenceForm[ "Taking ", "-", "residue in ", z6, " = ", 
        Plus[ -1, 
          Times[ -2, eps], 
          Times[ -1, z1], 
          Times[ -1, z2], 
          Times[ -1, z4]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{2}\),
      SequenceForm[ "Integral ", {2}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"+"\[InvisibleSpace]"residue in "\[InvisibleSpace]z3\
\[InvisibleSpace]" = "\[InvisibleSpace]\(2\ eps - z2\)\),
      SequenceForm[ "Taking ", "+", "residue in ", z3, " = ", 
        Plus[ 
          Times[ 2, eps], 
          Times[ -1, z2]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{3}\),
      SequenceForm[ "Integral ", {3}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"+"\[InvisibleSpace]"residue in "\[InvisibleSpace]z3\
\[InvisibleSpace]" = "\[InvisibleSpace]\(2\ eps - z2\)\),
      SequenceForm[ "Taking ", "+", "residue in ", z3, " = ", 
        Plus[ 
          Times[ 2, eps], 
          Times[ -1, z2]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{4}\),
      SequenceForm[ "Integral ", {4}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{5}\),
      SequenceForm[ "Integral ", {5}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Level "\[InvisibleSpace]3\),
      SequenceForm[ "Level ", 3],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{1, 1}\),
      SequenceForm[ "Integral ", {1, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"+"\[InvisibleSpace]"residue in "\[InvisibleSpace]z6\
\[InvisibleSpace]" = "\[InvisibleSpace]\(2\ eps + z1\)\),
      SequenceForm[ "Taking ", "+", "residue in ", z6, " = ", 
        Plus[ 
          Times[ 2, eps], z1]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{1, 2}\),
      SequenceForm[ "Integral ", {1, 2}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"+"\[InvisibleSpace]"residue in "\[InvisibleSpace]z2\
\[InvisibleSpace]" = "\[InvisibleSpace]\(2\ eps\)\),
      SequenceForm[ "Taking ", "+", "residue in ", z2, " = ", 
        Times[ 2, eps]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{1, 3}\),
      SequenceForm[ "Integral ", {1, 3}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"-"\[InvisibleSpace]"residue in "\[InvisibleSpace]z4\
\[InvisibleSpace]" = "\[InvisibleSpace]\(\(-1\) - 4\ eps - 2\ z1 - z2\)\),
      SequenceForm[ "Taking ", "-", "residue in ", z4, " = ", 
        Plus[ -1, 
          Times[ -4, eps], 
          Times[ -2, z1], 
          Times[ -1, z2]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{2, 1}\),
      SequenceForm[ "Integral ", {2, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"+"\[InvisibleSpace]"residue in "\[InvisibleSpace]z2\
\[InvisibleSpace]" = "\[InvisibleSpace]\(2\ eps\)\),
      SequenceForm[ "Taking ", "+", "residue in ", z2, " = ", 
        Times[ 2, eps]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{3, 1}\),
      SequenceForm[ "Integral ", {3, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"+"\[InvisibleSpace]"residue in "\[InvisibleSpace]z2\
\[InvisibleSpace]" = "\[InvisibleSpace]\(2\ eps\)\),
      SequenceForm[ "Taking ", "+", "residue in ", z2, " = ", 
        Times[ 2, eps]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Level "\[InvisibleSpace]4\),
      SequenceForm[ "Level ", 4],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{1, 1, 1}\),
      SequenceForm[ "Integral ", {1, 1, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{1, 2, 1}\),
      SequenceForm[ "Integral ", {1, 2, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{1, 3, 1}\),
      SequenceForm[ "Integral ", {1, 3, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{2, 1, 1}\),
      SequenceForm[ "Integral ", {2, 1, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{3, 1, 1}\),
      SequenceForm[ "Integral ", {3, 1, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\(16\[InvisibleSpace]" integral(s) found"\),
      SequenceForm[ 16, " integral(s) found"],
      Editable->False]], "Print"],

Cell[BoxData[
    \("Level 1"\)], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"-"\[InvisibleSpace]"residue in "\[InvisibleSpace]z4\
\[InvisibleSpace]" = "\[InvisibleSpace]\(\(-1\) - eps - z1\)\),
      SequenceForm[ "Taking ", "-", "residue in ", z4, " = ", 
        Plus[ -1, 
          Times[ -1, eps], 
          Times[ -1, z1]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"-"\[InvisibleSpace]"residue in "\[InvisibleSpace]z4\
\[InvisibleSpace]" = "\[InvisibleSpace]\(\(-1\) - eps - z1 - z2 - z3\)\),
      SequenceForm[ "Taking ", "-", "residue in ", z4, " = ", 
        Plus[ -1, 
          Times[ -1, eps], 
          Times[ -1, z1], 
          Times[ -1, z2], 
          Times[ -1, z3]]],
      Editable->False]], "Print"],

Cell[BoxData[
    \("...no contribution"\)], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"-"\[InvisibleSpace]"residue in "\[InvisibleSpace]z7\
\[InvisibleSpace]" = "\[InvisibleSpace]\(\(-1\) - eps + z2 + z3 - z6\)\),
      SequenceForm[ "Taking ", "-", "residue in ", z7, " = ", 
        Plus[ -1, 
          Times[ -1, eps], z2, z3, 
          Times[ -1, z6]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"-"\[InvisibleSpace]"residue in "\[InvisibleSpace]z7\
\[InvisibleSpace]" = "\[InvisibleSpace]\(\(-2\) - 2\ eps - z1 - z4 - z6\)\),
      SequenceForm[ "Taking ", "-", "residue in ", z7, " = ", 
        Plus[ -2, 
          Times[ -2, eps], 
          Times[ -1, z1], 
          Times[ -1, z4], 
          Times[ -1, z6]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Level "\[InvisibleSpace]2\),
      SequenceForm[ "Level ", 2],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{1}\),
      SequenceForm[ "Integral ", {1}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"+"\[InvisibleSpace]"residue in "\[InvisibleSpace]z3\
\[InvisibleSpace]" = "\[InvisibleSpace]\(2\ eps - z2\)\),
      SequenceForm[ "Taking ", "+", "residue in ", z3, " = ", 
        Plus[ 
          Times[ 2, eps], 
          Times[ -1, z2]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"-"\[InvisibleSpace]"residue in "\[InvisibleSpace]z7\
\[InvisibleSpace]" = "\[InvisibleSpace]\(\(-1\) - eps + z2 + z3 - z6\)\),
      SequenceForm[ "Taking ", "-", "residue in ", z7, " = ", 
        Plus[ -1, 
          Times[ -1, eps], z2, z3, 
          Times[ -1, z6]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"-"\[InvisibleSpace]"residue in "\[InvisibleSpace]z7\
\[InvisibleSpace]" = "\[InvisibleSpace]\(\(-1\) - eps - z6\)\),
      SequenceForm[ "Taking ", "-", "residue in ", z7, " = ", 
        Plus[ -1, 
          Times[ -1, eps], 
          Times[ -1, z6]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{2}\),
      SequenceForm[ "Integral ", {2}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"+"\[InvisibleSpace]"residue in "\[InvisibleSpace]z3\
\[InvisibleSpace]" = "\[InvisibleSpace]\(2\ eps - z2\)\),
      SequenceForm[ "Taking ", "+", "residue in ", z3, " = ", 
        Plus[ 
          Times[ 2, eps], 
          Times[ -1, z2]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"-"\[InvisibleSpace]"residue in "\[InvisibleSpace]z4\
\[InvisibleSpace]" = "\[InvisibleSpace]\(\(-1\) - eps - z1 - z2 - z3\)\),
      SequenceForm[ "Taking ", "-", "residue in ", z4, " = ", 
        Plus[ -1, 
          Times[ -1, eps], 
          Times[ -1, z1], 
          Times[ -1, z2], 
          Times[ -1, z3]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{3}\),
      SequenceForm[ "Integral ", {3}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"-"\[InvisibleSpace]"residue in "\[InvisibleSpace]z3\
\[InvisibleSpace]" = "\[InvisibleSpace]\(\(-2\) - 4\ eps - 2\ z1 - z2 - 
          2\ z4\)\),
      SequenceForm[ "Taking ", "-", "residue in ", z3, " = ", 
        Plus[ -2, 
          Times[ -4, eps], 
          Times[ -2, z1], 
          Times[ -1, z2], 
          Times[ -2, z4]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"-"\[InvisibleSpace]"residue in "\[InvisibleSpace]z4\
\[InvisibleSpace]" = "\[InvisibleSpace]\(\(-1\) - eps - z1 - z2 - z3\)\),
      SequenceForm[ "Taking ", "-", "residue in ", z4, " = ", 
        Plus[ -1, 
          Times[ -1, eps], 
          Times[ -1, z1], 
          Times[ -1, z2], 
          Times[ -1, z3]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"-"\[InvisibleSpace]"residue in "\[InvisibleSpace]z6\
\[InvisibleSpace]" = "\[InvisibleSpace]\(\(-1\) - 2\ eps - z1 - z2 - z4\)\),
      SequenceForm[ "Taking ", "-", "residue in ", z6, " = ", 
        Plus[ -1, 
          Times[ -2, eps], 
          Times[ -1, z1], 
          Times[ -1, z2], 
          Times[ -1, z4]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"-"\[InvisibleSpace]"residue in "\[InvisibleSpace]z6\
\[InvisibleSpace]" = "\[InvisibleSpace]\(\(-1\) - 2\ eps - z1 - z3 - z4\)\),
      SequenceForm[ "Taking ", "-", "residue in ", z6, " = ", 
        Plus[ -1, 
          Times[ -2, eps], 
          Times[ -1, z1], 
          Times[ -1, z3], 
          Times[ -1, z4]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Level "\[InvisibleSpace]3\),
      SequenceForm[ "Level ", 3],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{1, 1}\),
      SequenceForm[ "Integral ", {1, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"+"\[InvisibleSpace]"residue in "\[InvisibleSpace]z2\
\[InvisibleSpace]" = "\[InvisibleSpace]\(2\ eps\)\),
      SequenceForm[ "Taking ", "+", "residue in ", z2, " = ", 
        Times[ 2, eps]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"-"\[InvisibleSpace]"residue in "\[InvisibleSpace]z7\
\[InvisibleSpace]" = "\[InvisibleSpace]\(\(-1\) - eps - z6\)\),
      SequenceForm[ "Taking ", "-", "residue in ", z7, " = ", 
        Plus[ -1, 
          Times[ -1, eps], 
          Times[ -1, z6]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{1, 2}\),
      SequenceForm[ "Integral ", {1, 2}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"+"\[InvisibleSpace]"residue in "\[InvisibleSpace]z3\
\[InvisibleSpace]" = "\[InvisibleSpace]\(2\ eps - z2\)\),
      SequenceForm[ "Taking ", "+", "residue in ", z3, " = ", 
        Plus[ 
          Times[ 2, eps], 
          Times[ -1, z2]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{1, 3}\),
      SequenceForm[ "Integral ", {1, 3}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{2, 1}\),
      SequenceForm[ "Integral ", {2, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"+"\[InvisibleSpace]"residue in "\[InvisibleSpace]z2\
\[InvisibleSpace]" = "\[InvisibleSpace]\(2\ eps\)\),
      SequenceForm[ "Taking ", "+", "residue in ", z2, " = ", 
        Times[ 2, eps]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{2, 2}\),
      SequenceForm[ "Integral ", {2, 2}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"+"\[InvisibleSpace]"residue in "\[InvisibleSpace]z3\
\[InvisibleSpace]" = "\[InvisibleSpace]\(2\ eps - z2\)\),
      SequenceForm[ "Taking ", "+", "residue in ", z3, " = ", 
        Plus[ 
          Times[ 2, eps], 
          Times[ -1, z2]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{3, 1}\),
      SequenceForm[ "Integral ", {3, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"-"\[InvisibleSpace]"residue in "\[InvisibleSpace]z4\
\[InvisibleSpace]" = "\[InvisibleSpace]\(\(-1\) - 4\ eps - 2\ z1 - z2\)\),
      SequenceForm[ "Taking ", "-", "residue in ", z4, " = ", 
        Plus[ -1, 
          Times[ -4, eps], 
          Times[ -2, z1], 
          Times[ -1, z2]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"-"\[InvisibleSpace]"residue in "\[InvisibleSpace]z4\
\[InvisibleSpace]" = "\[InvisibleSpace]\(\(-1\) - 3\ eps - z1\)\),
      SequenceForm[ "Taking ", "-", "residue in ", z4, " = ", 
        Plus[ -1, 
          Times[ -3, eps], 
          Times[ -1, z1]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"-"\[InvisibleSpace]"residue in "\[InvisibleSpace]z6\
\[InvisibleSpace]" = "\[InvisibleSpace]\(\(-1\) - 2\ eps - z1 - z2 - z4\)\),
      SequenceForm[ "Taking ", "-", "residue in ", z6, " = ", 
        Plus[ -1, 
          Times[ -2, eps], 
          Times[ -1, z1], 
          Times[ -1, z2], 
          Times[ -1, z4]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{3, 2}\),
      SequenceForm[ "Integral ", {3, 2}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"+"\[InvisibleSpace]"residue in "\[InvisibleSpace]z3\
\[InvisibleSpace]" = "\[InvisibleSpace]\(2\ eps - z2\)\),
      SequenceForm[ "Taking ", "+", "residue in ", z3, " = ", 
        Plus[ 
          Times[ 2, eps], 
          Times[ -1, z2]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{3, 3}\),
      SequenceForm[ "Integral ", {3, 3}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{3, 4}\),
      SequenceForm[ "Integral ", {3, 4}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Level "\[InvisibleSpace]4\),
      SequenceForm[ "Level ", 4],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{1, 1, 1}\),
      SequenceForm[ "Integral ", {1, 1, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"-"\[InvisibleSpace]"residue in "\[InvisibleSpace]z7\
\[InvisibleSpace]" = "\[InvisibleSpace]\(\(-1\) - eps - z6\)\),
      SequenceForm[ "Taking ", "-", "residue in ", z7, " = ", 
        Plus[ -1, 
          Times[ -1, eps], 
          Times[ -1, z6]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{1, 1, 2}\),
      SequenceForm[ "Integral ", {1, 1, 2}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{1, 2, 1}\),
      SequenceForm[ "Integral ", {1, 2, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"+"\[InvisibleSpace]"residue in "\[InvisibleSpace]z2\
\[InvisibleSpace]" = "\[InvisibleSpace]\(2\ eps\)\),
      SequenceForm[ "Taking ", "+", "residue in ", z2, " = ", 
        Times[ 2, eps]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{2, 1, 1}\),
      SequenceForm[ "Integral ", {2, 1, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{2, 2, 1}\),
      SequenceForm[ "Integral ", {2, 2, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"+"\[InvisibleSpace]"residue in "\[InvisibleSpace]z2\
\[InvisibleSpace]" = "\[InvisibleSpace]\(2\ eps\)\),
      SequenceForm[ "Taking ", "+", "residue in ", z2, " = ", 
        Times[ 2, eps]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{3, 1, 1}\),
      SequenceForm[ "Integral ", {3, 1, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"+"\[InvisibleSpace]"residue in "\[InvisibleSpace]z6\
\[InvisibleSpace]" = "\[InvisibleSpace]\(2\ eps + z1\)\),
      SequenceForm[ "Taking ", "+", "residue in ", z6, " = ", 
        Plus[ 
          Times[ 2, eps], z1]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{3, 1, 2}\),
      SequenceForm[ "Integral ", {3, 1, 2}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"+"\[InvisibleSpace]"residue in "\[InvisibleSpace]z2\
\[InvisibleSpace]" = "\[InvisibleSpace]\(2\ eps\)\),
      SequenceForm[ "Taking ", "+", "residue in ", z2, " = ", 
        Times[ 2, eps]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{3, 1, 3}\),
      SequenceForm[ "Integral ", {3, 1, 3}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"-"\[InvisibleSpace]"residue in "\[InvisibleSpace]z4\
\[InvisibleSpace]" = "\[InvisibleSpace]\(\(-1\) - 4\ eps - 2\ z1 - z2\)\),
      SequenceForm[ "Taking ", "-", "residue in ", z4, " = ", 
        Plus[ -1, 
          Times[ -4, eps], 
          Times[ -2, z1], 
          Times[ -1, z2]]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{3, 2, 1}\),
      SequenceForm[ "Integral ", {3, 2, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Taking \
"\[InvisibleSpace]"+"\[InvisibleSpace]"residue in "\[InvisibleSpace]z2\
\[InvisibleSpace]" = "\[InvisibleSpace]\(2\ eps\)\),
      SequenceForm[ "Taking ", "+", "residue in ", z2, " = ", 
        Times[ 2, eps]],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Level "\[InvisibleSpace]5\),
      SequenceForm[ "Level ", 5],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{1, 1, 1, 1}\),
      SequenceForm[ "Integral ", {1, 1, 1, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{1, 2, 1, 1}\),
      SequenceForm[ "Integral ", {1, 2, 1, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{2, 2, 1, 1}\),
      SequenceForm[ "Integral ", {2, 2, 1, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{3, 1, 1, 1}\),
      SequenceForm[ "Integral ", {3, 1, 1, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{3, 1, 2, 1}\),
      SequenceForm[ "Integral ", {3, 1, 2, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{3, 1, 3, 1}\),
      SequenceForm[ "Integral ", {3, 1, 3, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Integral "\[InvisibleSpace]{3, 2, 1, 1}\),
      SequenceForm[ "Integral ", {3, 2, 1, 1}],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\(29\[InvisibleSpace]" integral(s) found"\),
      SequenceForm[ 29, " integral(s) found"],
      Editable->False]], "Print"],

Cell[BoxData[
    \({{{{{MBint[\(-\(\((\((m\^2)\)\^\(2\ eps + 2\ z1\)\ \((\(-s\))\)\^\(\(-4\
\)\ eps - 2\ z1\)\ Gamma[\(-4\)\ eps]\ Gamma[\(-2\)\ eps - 
                            z1]\ Gamma[\(-z1\)]\ Gamma[\(-4\)\ eps - 2\ z1 - 
                            z2]\ Gamma[\(-eps\) - z1 - 
                            z2]\ Gamma[\(-z2\)]\ Gamma[
                          3\ eps + z1 + z2]\ Gamma[
                          1 + 4\ eps + 2\ z1 + 
                            z2])\)/\((s\^2\ Gamma[\(-2\)\ eps]\ Gamma[
                          1 - 4\ eps - 2\ z1 - z2])\)\)\), {{eps \[Rule] 0, 
                    eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                    z2 \[Rule] \(-\(1\/8\)\)}}]}, 
            MBint[\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-2\)\ eps - \
z1 - z6\)\ Gamma[\(-4\)\ eps]\ Gamma[\(-z1\)]\ Gamma[\(-4\)\ eps - 2\ z1 - 
                        z2]\ Gamma[\(-eps\) - z1 - z2]\ Gamma[\(-z2\)]\ Gamma[
                      3\ eps + z1 + z2]\ Gamma[
                      1 + 4\ eps + 2\ z1 + z2]\ Gamma[\(-2\)\ eps - z1 - 
                        z6]\ Gamma[2\ eps + z1 - z6]\ Gamma[\(-z6\)]\ Gamma[
                      1 - 2\ eps - z1 - z2 + 
                        z6])\)/\((s\^2\ Gamma[\(-2\)\ eps]\ Gamma[
                      1 - z2]\ Gamma[
                      1 - 4\ eps - 2\ z1 - 
                        z2]\ Gamma[\(-2\)\ z6])\), {{eps \[Rule] 0, 
                  eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                  z2 \[Rule] \(-\(1\/8\)\), 
                  z6 \[Rule] \(-\(19\/64\)\)}}]}, \
{{MBint[\(-\(\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-2\)\ eps - z1 - \
z6\)\ Gamma[\(-4\)\ eps]\ Gamma[
                          2\ eps]\ Gamma[\(-3\)\ eps - z1]\ Gamma[\(-eps\) - 
                            z1]\ Gamma[\(-z1\)]\ Gamma[
                          1 + 3\ eps + z1]\ Gamma[\(-eps\) - z6]\ Gamma[
                          eps - z6]\ Gamma[\(-z6\)]\ Gamma[
                          1 - eps + z6])\)/\((s\^2\ Gamma[
                          1 - 2\ eps]\ Gamma[\(-4\)\ eps - 
                            2\ z1]\ Gamma[\(-2\)\ z6])\)\)\), {{eps \[Rule] 
                      0, eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                    z6 \[Rule] \(-\(19\/64\)\)}}]}, 
            MBint[\(-\(\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-2\)\ \
eps - z1 - z6\)\ Gamma[\(-4\)\ eps]\ Gamma[2\ eps]\ Gamma[\(-z1\)]\ Gamma[
                        1 + 3\ eps + z1]\ Gamma[\(-eps\) - z1 - 
                          z2]\ Gamma[\(-z2\)]\ Gamma[\(-2\)\ eps + 
                          z2]\ Gamma[\(-3\)\ eps - z1 + z2]\ Gamma[
                        eps - z2 - z6]\ Gamma[\(-eps\) + z2 - 
                          z6]\ Gamma[\(-z6\)]\ Gamma[
                        1 - eps + 
                          z6])\)/\((s\^2\ Gamma[\(-2\)\ eps]\ Gamma[\(-4\)\ \
eps - 2\ z1]\ Gamma[1 - z2]\ Gamma[
                        1 - 2\ eps + 
                          z2]\ Gamma[\(-2\)\ z6])\)\)\), {{eps \[Rule] 0, 
                  eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                  z2 \[Rule] \(-\(1\/8\)\), 
                  z6 \[Rule] \(-\(19\/64\)\)}}]}, {{MBint[\((\((m\^2)\)\^\(2\ \
eps + 2\ z1\)\ \((\(-s\))\)\^\(\(-4\)\ eps - 2\ z1\)\ Gamma[\(-4\)\ eps]\ \
Gamma[\(-2\)\ eps - z1]\ Gamma[\(-z1\)]\ Gamma[\(-4\)\ eps - 2\ z1 - 
                          z2]\ Gamma[\(-eps\) - z1 - 
                          z2]\ Gamma[\(-z2\)]\ Gamma[3\ eps + z1 + z2]\ Gamma[
                        1 + 4\ eps + 2\ z1 + 
                          z2])\)/\((s\^2\ Gamma[\(-2\)\ eps]\ Gamma[
                        1 - 4\ eps - 2\ z1 - z2])\), {{eps \[Rule] 0, 
                    eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                    z2 \[Rule] \(-\(1\/8\)\)}}]}, 
            MBint[\(-\(\((\((m\^2)\)\^\(\(-1\) - 2\ eps - z2 - z4\)\ \
\((\(-s\))\)\^\(z2 + z4\)\ Gamma[\(-4\)\ eps]\ Gamma[\(-z1\)]\ Gamma[\(-z2\)]\
\ Gamma[\(-1\) - eps - z1 - z4]\ Gamma[\(-1\) - 4\ eps - 2\ z1 - z2 - 
                          z4]\ Gamma[\(-z4\)]\ Gamma[
                        1 + 3\ eps + z1 + z4]\ Gamma[1 + z2 + z4]\ Gamma[
                        1 + 2\ eps + z1 + z2 + z4]\ Gamma[
                        2 + 4\ eps + 2\ z1 + z2 + 
                          2\ z4])\)/\((s\ Gamma[\(-2\)\ eps]\ Gamma[\(-4\)\ \
eps - 2\ z1]\ Gamma[3 + 4\ eps + 2\ z1 + z2 + 2\ z4])\)\)\), {{eps \[Rule] 0, 
                  eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                  z2 \[Rule] \(-\(1\/8\)\), z4 \[Rule] \(-\(69\/256\)\)}}]}, 
          MBint[\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-2\)\ eps - z1 \
- z6\)\ Gamma[\(-4\)\ eps]\ Gamma[\(-z1\)]\ Gamma[\(-z2\)]\ Gamma[\(-1\) - 
                      eps - z1 - z4]\ Gamma[\(-1\) - 4\ eps - 2\ z1 - z2 - 
                      z4]\ Gamma[\(-z4\)]\ Gamma[1 + 3\ eps + z1 + z4]\ Gamma[
                    1 + z2 + z4]\ Gamma[
                    2 + 4\ eps + 2\ z1 + z2 + 2\ z4]\ Gamma[\(-1\) - 2\ eps - 
                      z1 - z2 - z4 - z6]\ Gamma[
                    1 + 2\ eps + z1 + z2 + z4 - z6]\ Gamma[\(-z6\)]\ Gamma[
                    2 + 2\ eps + z1 + z4 + 
                      z6])\)/\((s\^2\ Gamma[\(-2\)\ eps]\ Gamma[\(-4\)\ eps - 
                      2\ z1]\ Gamma[1 - z2]\ Gamma[
                    3 + 4\ eps + 2\ z1 + z2 + 
                      2\ z4]\ Gamma[\(-2\)\ z6])\), {{eps \[Rule] 0, 
                eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                z2 \[Rule] \(-\(1\/8\)\), z4 \[Rule] \(-\(69\/256\)\), 
                
                z6 \[Rule] \(-\(19\/64\)\)}}]}, {{{MBint[\((\((m\^2)\)\^\(z1 \
+ z6\)\ \((\(-s\))\)\^\(\(-2\)\ eps - z1 - z6\)\ Gamma[\(-4\)\ eps]\ \
Gamma[\(-2\)\ eps]\ Gamma[1 + 2\ eps]\ Gamma[\(-eps\) - z1]\ Gamma[
                        eps - z1]\ Gamma[\(-z1\)]\ Gamma[
                        1 + eps + z1]\ Gamma[\(-3\)\ eps - 
                          z6]\ Gamma[\(-eps\) - z6]\ Gamma[\(-z6\)]\ Gamma[
                        1 + eps + 
                          z6])\)/\((s\^2\ Gamma[1 - 2\ eps]\^2\ Gamma[\(-2\)\ \
z1]\ Gamma[\(-4\)\ eps - 2\ z6])\), {{eps \[Rule] 0, 
                    eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                    z6 \[Rule] \(-\(19\/64\)\)}}]}, 
            MBint[\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-2\)\ eps - \
z1 - z6\)\ Gamma[\(-4\)\ eps]\ Gamma[1 + 2\ eps]\ Gamma[\(-z1\)]\ Gamma[
                      1 + eps + z1]\ Gamma[
                      eps - z1 - z2]\ Gamma[\(-z2\)]\ Gamma[\(-2\)\ eps + 
                        z2]\ Gamma[\(-eps\) - z1 + z2]\ Gamma[\(-eps\) - z2 - 
                        z6]\ Gamma[\(-3\)\ eps + z2 - 
                        z6]\ Gamma[\(-z6\)]\ Gamma[
                      1 + eps + z6])\)/\((s\^2\ Gamma[
                      1 - 2\ eps]\ Gamma[\(-2\)\ z1]\ Gamma[1 - z2]\ Gamma[
                      1 - 2\ eps + z2]\ Gamma[\(-4\)\ eps - 
                        2\ z6])\), {{eps \[Rule] 0, 
                  eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                  z2 \[Rule] \(-\(1\/8\)\), z6 \[Rule] \(-\(19\/64\)\)}}]}, 
          MBint[\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-2\)\ eps - z1 \
- z6\)\ Gamma[\(-z1\)]\ Gamma[
                    1 + eps + z1]\ Gamma[\(-z2\)]\ Gamma[\(-eps\) - z1 + 
                      z2]\ Gamma[\(-z2\) - z3]\ Gamma[\(-2\)\ eps - z2 - 
                      z3]\ Gamma[\(-z3\)]\ Gamma[\(-eps\) - z1 + z3]\ Gamma[
                    1 + z2 + z3]\ Gamma[\(-2\)\ eps + z2 + 
                      z3]\ Gamma[\(-eps\) - z2 - z6]\ Gamma[\(-eps\) - z3 - 
                      z6]\ Gamma[\(-z6\)]\ Gamma[
                    1 + eps + z6])\)/\((s\^2\ Gamma[
                    1 - 2\ eps]\ Gamma[\(-2\)\ eps]\ Gamma[1 - z2]\ Gamma[
                    1 - z3]\ Gamma[\(-2\)\ eps - 2\ z1 + z2 + 
                      z3]\ Gamma[\(-2\)\ eps - z2 - z3 - 
                      2\ z6])\), {{eps \[Rule] 0, 
                eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                z2 \[Rule] \(-\(1\/8\)\), z3 \[Rule] \(-\(19\/128\)\), 
                z6 \[Rule] \(-\(19\/64\)\)}}]}, {{{MBint[\((\((m\^2)\)\^\(z1 \
+ z6\)\ \((\(-s\))\)\^\(\(-2\)\ eps - z1 - z6\)\ Gamma[\(-4\)\ eps]\ Gamma[
                        2\ eps]\ Gamma[\(-3\)\ eps - z1]\ Gamma[\(-eps\) - 
                          z1]\ Gamma[\(-z1\)]\ Gamma[
                        1 + 3\ eps + z1]\ Gamma[\(-eps\) - z6]\ Gamma[
                        eps - z6]\ Gamma[\(-z6\)]\ Gamma[
                        1 - eps + z6])\)/\((s\^2\ Gamma[
                        1 - 2\ eps]\ Gamma[\(-4\)\ eps - 
                          2\ z1]\ Gamma[\(-2\)\ z6])\), {{eps \[Rule] 0, 
                    eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                    z6 \[Rule] \(-\(19\/64\)\)}}]}, 
            MBint[\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-2\)\ eps - \
z1 - z6\)\ Gamma[\(-4\)\ eps]\ Gamma[2\ eps]\ Gamma[\(-z1\)]\ Gamma[
                      1 + 3\ eps + z1]\ Gamma[\(-eps\) - z1 - 
                        z2]\ Gamma[\(-z2\)]\ Gamma[\(-2\)\ eps + 
                        z2]\ Gamma[\(-3\)\ eps - z1 + z2]\ Gamma[
                      eps - z2 - z6]\ Gamma[\(-eps\) + z2 - 
                        z6]\ Gamma[\(-z6\)]\ Gamma[
                      1 - eps + 
                        z6])\)/\((s\^2\ Gamma[\(-2\)\ eps]\ Gamma[\(-4\)\ eps \
- 2\ z1]\ Gamma[1 - z2]\ Gamma[
                      1 - 2\ eps + z2]\ Gamma[\(-2\)\ z6])\), {{eps \[Rule] 
                    0, eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                  z2 \[Rule] \(-\(1\/8\)\), z6 \[Rule] \(-\(19\/64\)\)}}]}, 
          MBint[\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-2\)\ eps - z1 \
- z6\)\ Gamma[\(-z1\)]\ Gamma[\(-eps\) - z1 - 
                      z2]\ Gamma[\(-z2\)]\ Gamma[\(-eps\) - z1 - 
                      z3]\ Gamma[\(-2\)\ eps - z2 - 
                      z3]\ Gamma[\(-z3\)]\ Gamma[
                    z2 + z3]\ Gamma[\(-2\)\ eps + z2 + z3]\ Gamma[
                    1 + eps + z1 + z2 + z3]\ Gamma[\(-eps\) + z2 - 
                      z6]\ Gamma[\(-eps\) + z3 - z6]\ Gamma[\(-z6\)]\ Gamma[
                    1 + eps - z2 - z3 + 
                      z6])\)/\((s\^2\ Gamma[\(-2\)\ eps]\ Gamma[
                    1 - z2]\ Gamma[
                    1 - z3]\ Gamma[\(-2\)\ eps - 2\ z1 - z2 - z3]\ Gamma[
                    1 - 2\ eps + z2 + z3]\ Gamma[\(-2\)\ eps + z2 + z3 - 
                      2\ z6])\), {{eps \[Rule] 0, 
                eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                z2 \[Rule] \(-\(1\/8\)\), z3 \[Rule] \(-\(19\/128\)\), 
                z6 \[Rule] \(-\(19\/64\)\)}}]}, {MBint[\(-\(\((\((m\^2)\)\^\(\
\(-1\) - 2\ eps - z2 - z4\)\ \((\(-s\))\)\^\(z2 + z4\)\ Gamma[\(-z1\)]\ \
Gamma[\(-z2\)]\ Gamma[\(-z3\)]\ Gamma[\(-2\) - 4\ eps - 2\ z1 - z2 - z3 - 
                        2\ z4]\ Gamma[\(-1\) - eps - z1 - z4]\ Gamma[\(-1\) - 
                        eps - z1 - z2 - z3 - z4]\ Gamma[\(-z4\)]\ Gamma[
                      1 + z2 + z4]\ Gamma[1 + 2\ eps + z1 + z2 + z4]\ Gamma[
                      1 + z3 + z4]\ Gamma[2 + eps + z1 + z2 + z3 + z4]\ Gamma[
                      2 + 2\ z1 + z2 + z3 + 
                        2\ z4])\)/\((s\ Gamma[\(-2\)\ eps]\ Gamma[
                      1 - z3]\ Gamma[\(-3\)\ eps - z1 - z4]\ Gamma[
                      2 + z2 + z3 + 2\ z4])\)\)\), {{eps \[Rule] 0, 
                eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                z2 \[Rule] \(-\(1\/8\)\), z3 \[Rule] \(-\(19\/128\)\), 
                z4 \[Rule] \(-\(69\/256\)\)}}]}, \
{MBint[\(-\(\((\((m\^2)\)\^\(\(-1\) - 2\ eps - z3 - z4\)\ \((\(-s\))\)\^\(z3 \
+ z4\)\ Gamma[\(-z1\)]\ Gamma[\(-z2\)]\ Gamma[\(-z3\)]\ Gamma[\(-2\) - 
                        4\ eps - 2\ z1 - z2 - z3 - 2\ z4]\ Gamma[\(-1\) - 
                        eps - z1 - z4]\ Gamma[\(-1\) - eps - z1 - z2 - z3 - 
                        z4]\ Gamma[\(-z4\)]\ Gamma[1 + z2 + z4]\ Gamma[
                      1 + z3 + z4]\ Gamma[1 + 2\ eps + z1 + z3 + z4]\ Gamma[
                      2 + eps + z1 + z2 + z3 + z4]\ Gamma[
                      2 + 2\ z1 + z2 + z3 + 
                        2\ z4])\)/\((s\ Gamma[\(-2\)\ eps]\ Gamma[
                      1 - z2]\ Gamma[\(-3\)\ eps - z1 - z4]\ Gamma[
                      2 + z2 + z3 + 2\ z4])\)\)\), {{eps \[Rule] 0, 
                eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                z2 \[Rule] \(-\(1\/8\)\), z3 \[Rule] \(-\(19\/128\)\), 
                z4 \[Rule] \(-\(69\/256\)\)}}]}, 
        MBint[\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-2\)\ eps - z1 - \
z6\)\ Gamma[\(-z1\)]\ Gamma[\(-z2\)]\ Gamma[\(-z3\)]\ Gamma[\(-2\) - 4\ eps - 
                    2\ z1 - z2 - z3 - 2\ z4]\ Gamma[\(-1\) - eps - z1 - 
                    z4]\ Gamma[\(-1\) - eps - z1 - z2 - z3 - 
                    z4]\ Gamma[\(-z4\)]\ Gamma[1 + z2 + z4]\ Gamma[
                  1 + z3 + z4]\ Gamma[2 + eps + z1 + z2 + z3 + z4]\ Gamma[
                  2 + 2\ z1 + z2 + z3 + 2\ z4]\ Gamma[\(-1\) - 2\ eps - z1 - 
                    z2 - z4 - z6]\ Gamma[\(-1\) - 2\ eps - z1 - z3 - z4 - 
                    z6]\ Gamma[\(-z6\)]\ Gamma[
                  2 + 2\ eps + z1 + z4 + 
                    z6])\)/\((s\^2\ Gamma[\(-2\)\ eps]\ Gamma[1 - z2]\ Gamma[
                  1 - z3]\ Gamma[\(-3\)\ eps - z1 - z4]\ Gamma[
                  2 + z2 + z3 + 2\ z4]\ Gamma[\(-2\) - 4\ eps - 2\ z1 - z2 - 
                    z3 - 2\ z4 - 2\ z6])\), {{eps \[Rule] 0, 
              eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
              z2 \[Rule] \(-\(1\/8\)\), z3 \[Rule] \(-\(19\/128\)\), 
              z4 \[Rule] \(-\(69\/256\)\), 
              z6 \[Rule] \(-\(19\/64\)\)}}]}, {{{{{MBint[\(-\(\((\((m\^2)\)\^\
\(z1 + z6\)\ \((\(-s\))\)\^\(\(-2\)\ eps - z1 - z6\)\ Gamma[\(-4\)\ eps]\ \
Gamma[\(-2\)\ eps]\ Gamma[1 + 2\ eps]\ Gamma[\(-eps\) - z1]\ Gamma[
                            eps - z1]\ Gamma[\(-z1\)]\ Gamma[
                            1 + eps + z1]\ Gamma[\(-3\)\ eps - 
                              z6]\ Gamma[\(-eps\) - 
                              z6]\ Gamma[\(-z6\)]\ Gamma[
                            1 + eps + 
                              z6])\)/\((s\^2\ Gamma[1 - 2\ eps]\^2\ \
Gamma[\(-2\)\ z1]\ Gamma[\(-4\)\ eps - 2\ z6])\)\)\), {{eps \[Rule] 0, 
                      eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                      z6 \[Rule] \(-\(19\/64\)\)}}]}, 
              MBint[\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-eps\) - z1 \
+ z7\)\ \((\(-t\))\)\^\(\(-eps\) - z6 - z7\)\ Gamma[\(-2\)\ eps]\ Gamma[
                        1 + 2\ eps]\ Gamma[\(-eps\) - z1]\ Gamma[
                        eps - z1]\ Gamma[\(-z1\)]\ Gamma[
                        1 + eps + z1]\ Gamma[\(-z6\)]\ Gamma[\(-eps\) - z6 - 
                          z7]\ Gamma[\(-1\) + eps - z6 - 
                          z7]\ Gamma[\(-z7\)]\ Gamma[1 + z7]\ Gamma[
                        1 - 2\ eps + z7]\ Gamma[1 + eps + z6 + z7]\ Gamma[
                        2 - 2\ eps + 2\ z6 + 
                          2\ z7])\)/\((s\ t\ Gamma[1 - 2\ eps]\^2\ Gamma[
                        2\ eps]\ Gamma[\(-2\)\ z1]\ Gamma[
                        2 - 2\ eps + 2\ z7])\), {{eps \[Rule] 0, 
                    eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                    z6 \[Rule] \(-\(19\/64\)\), 
                    z7 \[Rule] \(-\(665\/1024\)\)}}]}, \
{MBint[\(-\(\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-2\)\ eps - z1 - z6\
\)\ Gamma[\(-4\)\ eps]\ Gamma[1 + 2\ eps]\ Gamma[\(-z1\)]\ Gamma[
                          1 + eps + z1]\ Gamma[
                          eps - z1 - z2]\ Gamma[\(-z2\)]\ Gamma[\(-2\)\ eps + 
                            z2]\ Gamma[\(-eps\) - z1 + z2]\ Gamma[\(-eps\) - 
                            z2 - z6]\ Gamma[\(-3\)\ eps + z2 - 
                            z6]\ Gamma[\(-z6\)]\ Gamma[
                          1 + eps + z6])\)/\((s\^2\ Gamma[
                          1 - 2\ eps]\ Gamma[\(-2\)\ z1]\ Gamma[
                          1 - z2]\ Gamma[
                          1 - 2\ eps + z2]\ Gamma[\(-4\)\ eps - 
                            2\ z6])\)\)\), {{eps \[Rule] 0, 
                    eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                    z2 \[Rule] \(-\(1\/8\)\), z6 \[Rule] \(-\(19\/64\)\)}}]}, 
            MBint[\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-eps\) - z1 + \
z7\)\ \((\(-t\))\)\^\(\(-eps\) - z6 - z7\)\ Gamma[
                      1 + 2\ eps]\ Gamma[\(-z1\)]\ Gamma[1 + eps + z1]\ Gamma[
                      eps - z1 - z2]\ Gamma[\(-z2\)]\ Gamma[\(-2\)\ eps + 
                        z2]\ Gamma[\(-eps\) - z1 + 
                        z2]\ Gamma[\(-z6\)]\ Gamma[\(-eps\) - z6 - 
                        z7]\ Gamma[\(-1\) + eps - z6 - 
                        z7]\ Gamma[\(-z7\)]\ Gamma[1 - z2 + z7]\ Gamma[
                      1 - 2\ eps + z2 + z7]\ Gamma[1 + eps + z6 + z7]\ Gamma[
                      2 - 2\ eps + 2\ z6 + 2\ z7])\)/\((s\ t\ Gamma[
                      1 - 2\ eps]\ Gamma[2\ eps]\ Gamma[\(-2\)\ z1]\ Gamma[
                      1 - z2]\ Gamma[1 - 2\ eps + z2]\ Gamma[
                      2 - 2\ eps + 2\ z7])\), {{eps \[Rule] 0, 
                  eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                  z2 \[Rule] \(-\(1\/8\)\), z6 \[Rule] \(-\(19\/64\)\), 
                  z7 \[Rule] \(-\(665\/1024\)\)}}]}, {{{MBint[\(-\(\((2\ \
EulerGamma\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
\((\(-t\))\)\^\(\(-2\)\ eps\)\ Gamma[\(-2\)\ eps]\ Gamma[
                              1 + 2\ eps]\ Gamma[\(-eps\) - z1]\ Gamma[
                              eps - z1]\ Gamma[\(-z1\)]\ Gamma[
                              1 + eps + z1]\ Gamma[\(-eps\) - z6]\ Gamma[
                              eps - z6]\ Gamma[\(-z6\)]\ Gamma[
                              1 - eps + z6])\)/\((s\^2\ Gamma[
                              1 - 2\ eps]\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ \
z6])\)\)\) + \((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
\((\(-t\))\)\^\(\(-2\)\ eps\)\ Gamma[\(-2\)\ eps]\ Gamma[
                            1 + 2\ eps]\ Gamma[\(-eps\) - z1]\ Gamma[
                            eps - z1]\ Gamma[\(-z1\)]\ Gamma[
                            1 + eps + z1]\ Gamma[\(-eps\) - z6]\ Gamma[
                            eps - z6]\ Gamma[\(-z6\)]\ Gamma[
                            1 - eps + z6]\ Log[\(-s\)])\)/\((s\^2\ Gamma[
                            1 - 2\ eps]\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ \
z6])\) - \((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
\((\(-t\))\)\^\(\(-2\)\ eps\)\ Gamma[\(-2\)\ eps]\ Gamma[
                            1 + 2\ eps]\ Gamma[\(-eps\) - z1]\ Gamma[
                            eps - z1]\ Gamma[\(-z1\)]\ Gamma[
                            1 + eps + z1]\ Gamma[\(-eps\) - z6]\ Gamma[
                            eps - z6]\ Gamma[\(-z6\)]\ Gamma[
                            1 - eps + z6]\ Log[\(-t\)])\)/\((s\^2\ Gamma[
                            1 - 2\ eps]\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ \
z6])\) - \((2\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
\((\(-t\))\)\^\(\(-2\)\ eps\)\ Gamma[\(-2\)\ eps]\ Gamma[
                            1 + 2\ eps]\ Gamma[\(-eps\) - z1]\ Gamma[
                            eps - z1]\ Gamma[\(-z1\)]\ Gamma[
                            1 + eps + z1]\ Gamma[\(-eps\) - z6]\ Gamma[
                            eps - z6]\ Gamma[\(-z6\)]\ Gamma[
                            1 - eps + z6]\ PolyGamma[
                            0, \(-2\)\ eps])\)/\((s\^2\ Gamma[
                            1 - 2\ eps]\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ \
z6])\) + \((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
\((\(-t\))\)\^\(\(-2\)\ eps\)\ Gamma[\(-2\)\ eps]\ Gamma[
                            1 + 2\ eps]\ Gamma[\(-eps\) - z1]\ Gamma[
                            eps - z1]\ Gamma[\(-z1\)]\ Gamma[
                            1 + eps + z1]\ Gamma[\(-eps\) - z6]\ Gamma[
                            eps - z6]\ Gamma[\(-z6\)]\ Gamma[
                            1 - eps + z6]\ PolyGamma[0, 
                            1 + 2\ eps])\)/\((s\^2\ Gamma[
                            1 - 2\ eps]\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ \
z6])\) + \((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
\((\(-t\))\)\^\(\(-2\)\ eps\)\ Gamma[\(-2\)\ eps]\ Gamma[
                            1 + 2\ eps]\ Gamma[\(-eps\) - z1]\ Gamma[
                            eps - z1]\ Gamma[\(-z1\)]\ Gamma[
                            1 + eps + z1]\ Gamma[\(-eps\) - z6]\ Gamma[
                            eps - z6]\ Gamma[\(-z6\)]\ Gamma[
                            1 - eps + z6]\ PolyGamma[0, 
                            eps - z1])\)/\((s\^2\ Gamma[
                            1 - 2\ eps]\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ \
z6])\) - \((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
\((\(-t\))\)\^\(\(-2\)\ eps\)\ Gamma[\(-2\)\ eps]\ Gamma[
                            1 + 2\ eps]\ Gamma[\(-eps\) - z1]\ Gamma[
                            eps - z1]\ Gamma[\(-z1\)]\ Gamma[
                            1 + eps + z1]\ Gamma[\(-eps\) - z6]\ Gamma[
                            eps - z6]\ Gamma[\(-z6\)]\ Gamma[
                            1 - eps + z6]\ PolyGamma[
                            0, \(-2\)\ z1])\)/\((s\^2\ Gamma[
                            1 - 2\ eps]\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ \
z6])\) + \((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
\((\(-t\))\)\^\(\(-2\)\ eps\)\ Gamma[\(-2\)\ eps]\ Gamma[
                            1 + 2\ eps]\ Gamma[\(-eps\) - z1]\ Gamma[
                            eps - z1]\ Gamma[\(-z1\)]\ Gamma[
                            1 + eps + z1]\ Gamma[\(-eps\) - z6]\ Gamma[
                            eps - z6]\ Gamma[\(-z6\)]\ Gamma[
                            1 - eps + z6]\ PolyGamma[0, 
                            eps - z6])\)/\((s\^2\ Gamma[
                            1 - 2\ eps]\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ \
z6])\) - \((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
\((\(-t\))\)\^\(\(-2\)\ eps\)\ Gamma[\(-2\)\ eps]\ Gamma[
                            1 + 2\ eps]\ Gamma[\(-eps\) - z1]\ Gamma[
                            eps - z1]\ Gamma[\(-z1\)]\ Gamma[
                            1 + eps + z1]\ Gamma[\(-eps\) - z6]\ Gamma[
                            eps - z6]\ Gamma[\(-z6\)]\ Gamma[
                            1 - eps + z6]\ PolyGamma[
                            0, \(-2\)\ z6])\)/\((s\^2\ Gamma[
                            1 - 2\ eps]\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ \
z6])\) - \((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
\((\(-t\))\)\^\(\(-2\)\ eps\)\ Gamma[\(-2\)\ eps]\ Gamma[
                            1 + 2\ eps]\ Gamma[\(-eps\) - z1]\ Gamma[
                            eps - z1]\ Gamma[\(-z1\)]\ Gamma[
                            1 + eps + z1]\ Gamma[\(-eps\) - z6]\ Gamma[
                            eps - z6]\ Gamma[\(-z6\)]\ Gamma[
                            1 - eps + z6]\ PolyGamma[0, 
                            1 - eps + z6])\)/\((s\^2\ Gamma[
                            1 - 2\ eps]\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ \
z6])\), {{eps \[Rule] 0, eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                      z6 \[Rule] \(-\(19\/64\)\)}}]}, 
              MBint[\(-\(\((2\ EulerGamma\ \((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ \((\(-t\))\)\^\(\(-2\)\ eps\)\ Gamma[
                            1 + 2\ eps]\ Gamma[\(-z1\)]\ Gamma[
                            1 + eps + z1]\ Gamma[
                            eps - z1 - 
                              z2]\ Gamma[\(-z2\)]\ Gamma[\(-2\)\ eps + 
                              z2]\ Gamma[\(-eps\) - z1 + z2]\ Gamma[
                            eps - z2 - z6]\ Gamma[\(-eps\) + z2 - 
                              z6]\ Gamma[\(-z6\)]\ Gamma[
                            1 - eps + 
                              z6])\)/\((s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                            1 - z2]\ Gamma[
                            1 - 2\ eps + 
                              z2]\ Gamma[\(-2\)\ z6])\)\)\) + \((\((m\^2)\)\^\
\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \((\(-t\))\)\^\(\(-2\)\ eps\)\ \
Gamma[1 + 2\ eps]\ Gamma[\(-z1\)]\ Gamma[1 + eps + z1]\ Gamma[
                          eps - z1 - z2]\ Gamma[\(-z2\)]\ Gamma[\(-2\)\ eps + 
                            z2]\ Gamma[\(-eps\) - z1 + z2]\ Gamma[
                          eps - z2 - z6]\ Gamma[\(-eps\) + z2 - 
                            z6]\ Gamma[\(-z6\)]\ Gamma[
                          1 - eps + 
                            z6]\ Log[\(-s\)])\)/\((s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[1 - z2]\ Gamma[1 - 2\ eps + 
                            z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + \
z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \((\(-t\))\)\^\(\(-2\)\ eps\)\ Gamma[
                          1 + 2\ eps]\ Gamma[\(-z1\)]\ Gamma[
                          1 + eps + z1]\ Gamma[
                          eps - z1 - z2]\ Gamma[\(-z2\)]\ Gamma[\(-2\)\ eps + 
                            z2]\ Gamma[\(-eps\) - z1 + z2]\ Gamma[
                          eps - z2 - z6]\ Gamma[\(-eps\) + z2 - 
                            z6]\ Gamma[\(-z6\)]\ Gamma[
                          1 - eps + 
                            z6]\ Log[\(-t\)])\)/\((s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[1 - z2]\ Gamma[1 - 2\ eps + 
                            z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + \
z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \((\(-t\))\)\^\(\(-2\)\ eps\)\ Gamma[
                          1 + 2\ eps]\ Gamma[\(-z1\)]\ Gamma[
                          1 + eps + z1]\ Gamma[
                          eps - z1 - z2]\ Gamma[\(-z2\)]\ Gamma[\(-2\)\ eps + 
                            z2]\ Gamma[\(-eps\) - z1 + z2]\ Gamma[
                          eps - z2 - z6]\ Gamma[\(-eps\) + z2 - 
                            z6]\ Gamma[\(-z6\)]\ Gamma[
                          1 - eps + z6]\ PolyGamma[0, 
                          1 - 2\ eps])\)/\((s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                          1 - z2]\ Gamma[
                          1 - 2\ eps + 
                            z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + \
z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \((\(-t\))\)\^\(\(-2\)\ eps\)\ Gamma[
                          1 + 2\ eps]\ Gamma[\(-z1\)]\ Gamma[
                          1 + eps + z1]\ Gamma[
                          eps - z1 - z2]\ Gamma[\(-z2\)]\ Gamma[\(-2\)\ eps + 
                            z2]\ Gamma[\(-eps\) - z1 + z2]\ Gamma[
                          eps - z2 - z6]\ Gamma[\(-eps\) + z2 - 
                            z6]\ Gamma[\(-z6\)]\ Gamma[
                          1 - eps + z6]\ PolyGamma[
                          0, \(-2\)\ eps])\)/\((s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[1 - z2]\ Gamma[1 - 2\ eps + 
                            z2]\ Gamma[\(-2\)\ z6])\) + \((\((m\^2)\)\^\(z1 + \
z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \((\(-t\))\)\^\(\(-2\)\ eps\)\ Gamma[
                          1 + 2\ eps]\ Gamma[\(-z1\)]\ Gamma[
                          1 + eps + z1]\ Gamma[
                          eps - z1 - z2]\ Gamma[\(-z2\)]\ Gamma[\(-2\)\ eps + 
                            z2]\ Gamma[\(-eps\) - z1 + z2]\ Gamma[
                          eps - z2 - z6]\ Gamma[\(-eps\) + z2 - 
                            z6]\ Gamma[\(-z6\)]\ Gamma[
                          1 - eps + z6]\ PolyGamma[0, 
                          1 + 2\ eps])\)/\((s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                          1 - z2]\ Gamma[
                          1 - 2\ eps + 
                            z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + \
z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \((\(-t\))\)\^\(\(-2\)\ eps\)\ Gamma[
                          1 + 2\ eps]\ Gamma[\(-z1\)]\ Gamma[
                          1 + eps + z1]\ Gamma[
                          eps - z1 - z2]\ Gamma[\(-z2\)]\ Gamma[\(-2\)\ eps + 
                            z2]\ Gamma[\(-eps\) - z1 + z2]\ Gamma[
                          eps - z2 - z6]\ Gamma[\(-eps\) + z2 - 
                            z6]\ Gamma[\(-z6\)]\ Gamma[
                          1 - eps + z6]\ PolyGamma[
                          0, \(-2\)\ z1])\)/\((s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                          1 - z2]\ Gamma[
                          1 - 2\ eps + 
                            z2]\ Gamma[\(-2\)\ z6])\) + \((\((m\^2)\)\^\(z1 + \
z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \((\(-t\))\)\^\(\(-2\)\ eps\)\ Gamma[
                          1 + 2\ eps]\ Gamma[\(-z1\)]\ Gamma[
                          1 + eps + z1]\ Gamma[
                          eps - z1 - z2]\ Gamma[\(-z2\)]\ Gamma[\(-2\)\ eps + 
                            z2]\ Gamma[\(-eps\) - z1 + z2]\ Gamma[
                          eps - z2 - z6]\ Gamma[\(-eps\) + z2 - 
                            z6]\ Gamma[\(-z6\)]\ Gamma[
                          1 - eps + z6]\ PolyGamma[0, 
                          eps - z1 - z2])\)/\((s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                          1 - z2]\ Gamma[
                          1 - 2\ eps + 
                            z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + \
z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \((\(-t\))\)\^\(\(-2\)\ eps\)\ Gamma[
                          1 + 2\ eps]\ Gamma[\(-z1\)]\ Gamma[
                          1 + eps + z1]\ Gamma[
                          eps - z1 - z2]\ Gamma[\(-z2\)]\ Gamma[\(-2\)\ eps + 
                            z2]\ Gamma[\(-eps\) - z1 + z2]\ Gamma[
                          eps - z2 - z6]\ Gamma[\(-eps\) + z2 - 
                            z6]\ Gamma[\(-z6\)]\ Gamma[
                          1 - eps + z6]\ PolyGamma[
                          0, \(-2\)\ eps + 
                            z2])\)/\((s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                          1 - z2]\ Gamma[
                          1 - 2\ eps + 
                            z2]\ Gamma[\(-2\)\ z6])\) + \((\((m\^2)\)\^\(z1 + \
z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \((\(-t\))\)\^\(\(-2\)\ eps\)\ Gamma[
                          1 + 2\ eps]\ Gamma[\(-z1\)]\ Gamma[
                          1 + eps + z1]\ Gamma[
                          eps - z1 - z2]\ Gamma[\(-z2\)]\ Gamma[\(-2\)\ eps + 
                            z2]\ Gamma[\(-eps\) - z1 + z2]\ Gamma[
                          eps - z2 - z6]\ Gamma[\(-eps\) + z2 - 
                            z6]\ Gamma[\(-z6\)]\ Gamma[
                          1 - eps + z6]\ PolyGamma[0, 
                          1 - 2\ eps + 
                            z2])\)/\((s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                          1 - z2]\ Gamma[
                          1 - 2\ eps + 
                            z2]\ Gamma[\(-2\)\ z6])\) + \((\((m\^2)\)\^\(z1 + \
z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \((\(-t\))\)\^\(\(-2\)\ eps\)\ Gamma[
                          1 + 2\ eps]\ Gamma[\(-z1\)]\ Gamma[
                          1 + eps + z1]\ Gamma[
                          eps - z1 - z2]\ Gamma[\(-z2\)]\ Gamma[\(-2\)\ eps + 
                            z2]\ Gamma[\(-eps\) - z1 + z2]\ Gamma[
                          eps - z2 - z6]\ Gamma[\(-eps\) + z2 - 
                            z6]\ Gamma[\(-z6\)]\ Gamma[
                          1 - eps + z6]\ PolyGamma[0, 
                          eps - z2 - z6])\)/\((s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                          1 - z2]\ Gamma[
                          1 - 2\ eps + 
                            z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + \
z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \((\(-t\))\)\^\(\(-2\)\ eps\)\ Gamma[
                          1 + 2\ eps]\ Gamma[\(-z1\)]\ Gamma[
                          1 + eps + z1]\ Gamma[
                          eps - z1 - z2]\ Gamma[\(-z2\)]\ Gamma[\(-2\)\ eps + 
                            z2]\ Gamma[\(-eps\) - z1 + z2]\ Gamma[
                          eps - z2 - z6]\ Gamma[\(-eps\) + z2 - 
                            z6]\ Gamma[\(-z6\)]\ Gamma[
                          1 - eps + z6]\ PolyGamma[
                          0, \(-2\)\ z6])\)/\((s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                          1 - z2]\ Gamma[
                          1 - 2\ eps + 
                            z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + \
z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \((\(-t\))\)\^\(\(-2\)\ eps\)\ Gamma[
                          1 + 2\ eps]\ Gamma[\(-z1\)]\ Gamma[
                          1 + eps + z1]\ Gamma[
                          eps - z1 - z2]\ Gamma[\(-z2\)]\ Gamma[\(-2\)\ eps + 
                            z2]\ Gamma[\(-eps\) - z1 + z2]\ Gamma[
                          eps - z2 - z6]\ Gamma[\(-eps\) + z2 - 
                            z6]\ Gamma[\(-z6\)]\ Gamma[
                          1 - eps + z6]\ PolyGamma[0, 
                          1 - eps + z6])\)/\((s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                          1 - z2]\ Gamma[
                          1 - 2\ eps + 
                            z2]\ Gamma[\(-2\)\ z6])\), {{eps \[Rule] 0, 
                    eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                    z2 \[Rule] \(-\(1\/8\)\), z6 \[Rule] \(-\(19\/64\)\)}}]}, 
            MBint[\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-2\)\ eps - \
z1 + z2 + z3 - z6\)\ \((\(-t\))\)\^\(\(-z2\) - z3\)\ Gamma[\(-z1\)]\ Gamma[
                      1 + eps + z1]\ Gamma[\(-z2\)]\ Gamma[\(-eps\) - z1 + 
                        z2]\ Gamma[\(-z2\) - z3]\ Gamma[
                      1 - z2 - z3]\ Gamma[\(-z3\)]\ Gamma[\(-eps\) - z1 + 
                        z3]\ Gamma[
                      1 + z2 + 
                        z3]\ Gamma[\(-2\)\ eps + z2 + z3]\^2\ Gamma[\(-eps\) \
+ z2 - z6]\ Gamma[\(-eps\) + z3 - z6]\ Gamma[\(-z6\)]\ Gamma[
                      1 + eps - z2 - z3 + z6])\)/\((s\^2\ Gamma[
                      1 - 2\ eps]\ Gamma[\(-2\)\ eps]\ Gamma[1 - z2]\ Gamma[
                      1 - z3]\ Gamma[\(-2\)\ eps - 2\ z1 + z2 + 
                        z3]\ Gamma[\(-2\)\ eps + z2 + z3 - 
                        2\ z6])\), {{eps \[Rule] 0, 
                  eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                  z2 \[Rule] \(-\(1\/8\)\), z3 \[Rule] \(-\(19\/128\)\), 
                  z6 \[Rule] \(-\(19\/64\)\)}}]}, {MBint[\(-\(\((\((m\^2)\)\^\
\(z1 + z6\)\ \((\(-s\))\)\^\(\(-2\)\ eps - z1 - z6\)\ Gamma[\(-z1\)]\ Gamma[
                        1 + eps + z1]\ Gamma[\(-z2\)]\ Gamma[\(-eps\) - z1 + 
                          z2]\ Gamma[\(-z2\) - z3]\ Gamma[\(-2\)\ eps - z2 - 
                          z3]\ Gamma[\(-z3\)]\ Gamma[\(-eps\) - z1 + 
                          z3]\ Gamma[
                        1 + z2 + z3]\ Gamma[\(-2\)\ eps + z2 + 
                          z3]\ Gamma[\(-eps\) - z2 - z6]\ Gamma[\(-eps\) - 
                          z3 - z6]\ Gamma[\(-z6\)]\ Gamma[
                        1 + eps + z6])\)/\((s\^2\ Gamma[
                        1 - 2\ eps]\ Gamma[\(-2\)\ eps]\ Gamma[1 - z2]\ Gamma[
                        1 - z3]\ Gamma[\(-2\)\ eps - 2\ z1 + z2 + 
                          z3]\ Gamma[\(-2\)\ eps - z2 - z3 - 
                          2\ z6])\)\)\), {{eps \[Rule] 0, 
                  eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                  z2 \[Rule] \(-\(1\/8\)\), z3 \[Rule] \(-\(19\/128\)\), 
                  z6 \[Rule] \(-\(19\/64\)\)}}]}, 
          MBint[\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-eps\) - z1 + \
z7\)\ \((\(-t\))\)\^\(\(-eps\) - z6 - z7\)\ Gamma[\(-z1\)]\ Gamma[
                    1 + eps + z1]\ Gamma[\(-z2\)]\ Gamma[\(-eps\) - z1 + 
                      z2]\ Gamma[\(-z2\) - 
                      z3]\ Gamma[\(-z3\)]\ Gamma[\(-eps\) - z1 + z3]\ Gamma[
                    1 + z2 + z3]\ Gamma[\(-2\)\ eps + z2 + 
                      z3]\ Gamma[\(-z6\)]\ Gamma[\(-eps\) - z6 - 
                      z7]\ Gamma[\(-1\) - eps + z2 + z3 - z6 - 
                      z7]\ Gamma[\(-z7\)]\ Gamma[1 - z2 + z7]\ Gamma[
                    1 - z3 + z7]\ Gamma[1 + eps + z6 + z7]\ Gamma[
                    2 - z2 - z3 + 2\ z6 + 2\ z7])\)/\((s\ t\ Gamma[
                    1 - 2\ eps]\ Gamma[\(-2\)\ eps]\ Gamma[1 - z2]\ Gamma[
                    1 - z3]\ Gamma[
                    z2 + z3]\ Gamma[\(-2\)\ eps - 2\ z1 + z2 + z3]\ Gamma[
                    2 - z2 - z3 + 2\ z7])\), {{eps \[Rule] 0, 
                eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                z2 \[Rule] \(-\(1\/8\)\), z3 \[Rule] \(-\(19\/128\)\), 
                z6 \[Rule] \(-\(19\/64\)\), 
                z7 \[Rule] \(-\(665\/1024\)\)}}]}, \
{{{MBint[\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(eps + z4 - z6\)\ \((\(-t\
\))\)\^\(\(-3\)\ eps - z1 - z4\)\ Gamma[\(-z1\)]\ Gamma[\(-1\) - 3\ eps - 
                          z1 - z4]\ Gamma[\(-1\) - eps - z1 - 
                          z4]\ Gamma[\(-z4\)]\ Gamma[1 + z4]\ Gamma[
                        1 + 2\ eps + z4]\ Gamma[2 + 3\ eps + z1 + z4]\ Gamma[
                        2 + 2\ eps + 2\ z1 + 2\ z4]\ Gamma[\(-eps\) - 
                          z6]\ Gamma[eps - z6]\ Gamma[\(-z6\)]\ Gamma[
                        1 - eps + z6])\)/\((s\ t\ Gamma[1 - 2\ eps]\ Gamma[
                        2 + 2\ eps + 
                          2\ z4]\ Gamma[\(-2\)\ z6])\), {{eps \[Rule] 0, 
                    eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                    z4 \[Rule] \(-\(69\/256\)\), 
                    z6 \[Rule] \(-\(19\/64\)\)}}]}, 
            MBint[\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(eps + z4 - z6\)\
\ \((\(-t\))\)\^\(\(-3\)\ eps - z1 - z4\)\ Gamma[\(-z1\)]\ Gamma[\(-z2\)]\ \
Gamma[\(-2\)\ eps + z2]\ Gamma[\(-1\) - 3\ eps - z1 - z4]\ Gamma[\(-1\) - 
                        eps - z1 - z4]\ Gamma[\(-z4\)]\ Gamma[
                      2 + 3\ eps + z1 + z4]\ Gamma[
                      1 + 2\ eps - z2 + z4]\ Gamma[1 + z2 + z4]\ Gamma[
                      2 + 2\ eps + 2\ z1 + 2\ z4]\ Gamma[
                      eps - z2 - z6]\ Gamma[\(-eps\) + z2 - 
                        z6]\ Gamma[\(-z6\)]\ Gamma[
                      1 - eps + z6])\)/\((s\ t\ Gamma[\(-2\)\ eps]\ Gamma[
                      1 - z2]\ Gamma[1 - 2\ eps + z2]\ Gamma[
                      2 + 2\ eps + 
                        2\ z4]\ Gamma[\(-2\)\ z6])\), {{eps \[Rule] 0, 
                  eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                  z2 \[Rule] \(-\(1\/8\)\), z4 \[Rule] \(-\(69\/256\)\), 
                  z6 \[Rule] \(-\(19\/64\)\)}}]}, \
{{{MBint[\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-2\)\ eps - z1 - z6\)\ \
Gamma[\(-4\)\ eps]\ Gamma[
                          2\ eps]\ Gamma[\(-3\)\ eps - z1]\ Gamma[\(-eps\) - 
                            z1]\ Gamma[\(-z1\)]\ Gamma[
                          1 + 3\ eps + z1]\ Gamma[\(-eps\) - z6]\ Gamma[
                          eps - z6]\ Gamma[\(-z6\)]\ Gamma[
                          1 - eps + z6])\)/\((s\^2\ Gamma[
                          1 - 2\ eps]\ Gamma[\(-4\)\ eps - 
                            2\ z1]\ Gamma[\(-2\)\ z6])\), {{eps \[Rule] 0, 
                      eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                      z6 \[Rule] \(-\(19\/64\)\)}}]}, 
              MBint[\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-2\)\ eps - \
z1 - z6\)\ Gamma[\(-4\)\ eps]\ Gamma[2\ eps]\ Gamma[\(-z1\)]\ Gamma[
                        1 + 3\ eps + z1]\ Gamma[\(-eps\) - z1 - 
                          z2]\ Gamma[\(-z2\)]\ Gamma[\(-2\)\ eps + 
                          z2]\ Gamma[\(-3\)\ eps - z1 + z2]\ Gamma[
                        eps - z2 - z6]\ Gamma[\(-eps\) + z2 - 
                          z6]\ Gamma[\(-z6\)]\ Gamma[
                        1 - eps + 
                          z6])\)/\((s\^2\ Gamma[\(-2\)\ eps]\ Gamma[\(-4\)\ \
eps - 2\ z1]\ Gamma[1 - z2]\ Gamma[
                        1 - 2\ eps + z2]\ Gamma[\(-2\)\ z6])\), {{eps \[Rule] 
                      0, eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                    z2 \[Rule] \(-\(1\/8\)\), z6 \[Rule] \(-\(19\/64\)\)}}]}, 
            MBint[\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-2\)\ eps - \
z1 - z6\)\ Gamma[\(-z1\)]\ Gamma[\(-eps\) - z1 - 
                        z2]\ Gamma[\(-z2\)]\ Gamma[\(-eps\) - z1 - 
                        z3]\ Gamma[\(-2\)\ eps - z2 - 
                        z3]\ Gamma[\(-z3\)]\ Gamma[
                      z2 + z3]\ Gamma[\(-2\)\ eps + z2 + z3]\ Gamma[
                      1 + eps + z1 + z2 + z3]\ Gamma[\(-eps\) + z2 - 
                        z6]\ Gamma[\(-eps\) + z3 - z6]\ Gamma[\(-z6\)]\ Gamma[
                      1 + eps - z2 - z3 + 
                        z6])\)/\((s\^2\ Gamma[\(-2\)\ eps]\ Gamma[
                      1 - z2]\ Gamma[
                      1 - z3]\ Gamma[\(-2\)\ eps - 2\ z1 - z2 - z3]\ Gamma[
                      1 - 2\ eps + z2 + z3]\ Gamma[\(-2\)\ eps + z2 + z3 - 
                        2\ z6])\), {{eps \[Rule] 0, 
                  eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                  z2 \[Rule] \(-\(1\/8\)\), z3 \[Rule] \(-\(19\/128\)\), 
                  z6 \[Rule] \(-\(19\/64\)\)}}]}, 
          MBint[\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-eps\) + z2 + \
z3 + z4 - z6\)\ \((\(-t\))\)\^\(\(-eps\) - z1 - z2 - z3 - z4\)\ \
Gamma[\(-z1\)]\ Gamma[\(-z2\)]\ Gamma[\(-z3\)]\ Gamma[\(-2\)\ eps + z2 + 
                      z3]\ Gamma[\(-1\) - eps - z1 - z4]\ Gamma[\(-1\) - 
                      eps - z1 - z2 - z3 - z4]\ Gamma[\(-eps\) - z1 - z2 - 
                      z3 - z4]\ Gamma[\(-z4\)]\ Gamma[1 + z2 + z4]\ Gamma[
                    1 + z3 + z4]\ Gamma[2 + eps + z1 + z2 + z3 + z4]\ Gamma[
                    2 + 2\ z1 + z2 + z3 + 2\ z4]\ Gamma[\(-eps\) + z2 - 
                      z6]\ Gamma[\(-eps\) + z3 - z6]\ Gamma[\(-z6\)]\ Gamma[
                    1 + eps - z2 - z3 + 
                      z6])\)/\((s\ t\ Gamma[\(-2\)\ eps]\ Gamma[
                    1 - z2]\ Gamma[
                    1 - z3]\ Gamma[\(-3\)\ eps - z1 - z4]\ Gamma[
                    2 + z2 + z3 + 2\ z4]\ Gamma[\(-2\)\ eps + z2 + z3 - 
                      2\ z6])\), {{eps \[Rule] 0, 
                eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                z2 \[Rule] \(-\(1\/8\)\), z3 \[Rule] \(-\(19\/128\)\), 
                z4 \[Rule] \(-\(69\/256\)\), 
                z6 \[Rule] \(-\(19\/64\)\)}}]}, {{{{MBint[\((\((m\^2)\)\^\(2\ \
eps + 2\ z1\)\ \((\(-s\))\)\^\(\(-4\)\ eps - 2\ z1\)\ Gamma[\(-4\)\ eps]\ \
Gamma[\(-2\)\ eps - z1]\ Gamma[\(-z1\)]\ Gamma[\(-4\)\ eps - 2\ z1 - 
                            z2]\ Gamma[\(-eps\) - z1 - 
                            z2]\ Gamma[\(-z2\)]\ Gamma[
                          3\ eps + z1 + z2]\ Gamma[
                          1 + 4\ eps + 2\ z1 + 
                            z2])\)/\((s\^2\ Gamma[\(-2\)\ eps]\ Gamma[
                          1 - 4\ eps - 2\ z1 - z2])\), {{eps \[Rule] 0, 
                      eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                      z2 \[Rule] \(-\(1\/8\)\)}}]}, 
              MBint[\(-\(\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-2\)\ \
eps - z1 - z6\)\ Gamma[\(-4\)\ eps]\ Gamma[\(-z1\)]\ Gamma[\(-4\)\ eps - 
                            2\ z1 - z2]\ Gamma[\(-eps\) - z1 - 
                            z2]\ Gamma[\(-z2\)]\ Gamma[
                          3\ eps + z1 + z2]\ Gamma[
                          1 + 4\ eps + 2\ z1 + z2]\ Gamma[\(-2\)\ eps - z1 - 
                            z6]\ Gamma[
                          2\ eps + z1 - z6]\ Gamma[\(-z6\)]\ Gamma[
                          1 - 2\ eps - z1 - z2 + 
                            z6])\)/\((s\^2\ Gamma[\(-2\)\ eps]\ Gamma[
                          1 - z2]\ Gamma[
                          1 - 4\ eps - 2\ z1 - 
                            z2]\ Gamma[\(-2\)\ z6])\)\)\), {{eps \[Rule] 0, 
                    eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                    z2 \[Rule] \(-\(1\/8\)\), 
                    z6 \[Rule] \(-\(19\/64\)\)}}]}, \
{{MBint[\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-2\)\ eps - z1 - z6\)\ \
Gamma[\(-4\)\ eps]\ Gamma[
                          2\ eps]\ Gamma[\(-3\)\ eps - z1]\ Gamma[\(-eps\) - 
                            z1]\ Gamma[\(-z1\)]\ Gamma[
                          1 + 3\ eps + z1]\ Gamma[\(-eps\) - z6]\ Gamma[
                          eps - z6]\ Gamma[\(-z6\)]\ Gamma[
                          1 - eps + z6])\)/\((s\^2\ Gamma[
                          1 - 2\ eps]\ Gamma[\(-4\)\ eps - 
                            2\ z1]\ Gamma[\(-2\)\ z6])\), {{eps \[Rule] 0, 
                      eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                      z6 \[Rule] \(-\(19\/64\)\)}}]}, 
              MBint[\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-2\)\ eps - \
z1 - z6\)\ Gamma[\(-4\)\ eps]\ Gamma[2\ eps]\ Gamma[\(-z1\)]\ Gamma[
                        1 + 3\ eps + z1]\ Gamma[\(-eps\) - z1 - 
                          z2]\ Gamma[\(-z2\)]\ Gamma[\(-2\)\ eps + 
                          z2]\ Gamma[\(-3\)\ eps - z1 + z2]\ Gamma[
                        eps - z2 - z6]\ Gamma[\(-eps\) + z2 - 
                          z6]\ Gamma[\(-z6\)]\ Gamma[
                        1 - eps + 
                          z6])\)/\((s\^2\ Gamma[\(-2\)\ eps]\ Gamma[\(-4\)\ \
eps - 2\ z1]\ Gamma[1 - z2]\ Gamma[
                        1 - 2\ eps + z2]\ Gamma[\(-2\)\ z6])\), {{eps \[Rule] 
                      0, eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                    z2 \[Rule] \(-\(1\/8\)\), 
                    z6 \[Rule] \(-\(19\/64\)\)}}]}, \
{{MBint[\(-\(\((\((m\^2)\)\^\(2\ eps + 2\ z1\)\ \((\(-s\))\)\^\(\(-4\)\ eps - \
2\ z1\)\ Gamma[\(-4\)\ eps]\ Gamma[\(-2\)\ eps - 
                              z1]\ Gamma[\(-z1\)]\ Gamma[\(-4\)\ eps - 
                              2\ z1 - z2]\ Gamma[\(-eps\) - z1 - 
                              z2]\ Gamma[\(-z2\)]\ Gamma[
                            3\ eps + z1 + z2]\ Gamma[
                            1 + 4\ eps + 2\ z1 + 
                              z2])\)/\((s\^2\ Gamma[\(-2\)\ eps]\ Gamma[
                            1 - 4\ eps - 2\ z1 - z2])\)\)\), {{eps \[Rule] 0, 
                      eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                      z2 \[Rule] \(-\(1\/8\)\)}}]}, 
              MBint[\((\((m\^2)\)\^\(\(-1\) - 2\ eps - z2 - z4\)\ \
\((\(-s\))\)\^\(z2 + z4\)\ Gamma[\(-4\)\ eps]\ Gamma[\(-z1\)]\ Gamma[\(-z2\)]\
\ Gamma[\(-1\) - eps - z1 - z4]\ Gamma[\(-1\) - 4\ eps - 2\ z1 - z2 - 
                          z4]\ Gamma[\(-z4\)]\ Gamma[
                        1 + 3\ eps + z1 + z4]\ Gamma[1 + z2 + z4]\ Gamma[
                        1 + 2\ eps + z1 + z2 + z4]\ Gamma[
                        2 + 4\ eps + 2\ z1 + z2 + 
                          2\ z4])\)/\((s\ Gamma[\(-2\)\ eps]\ Gamma[\(-4\)\ \
eps - 2\ z1]\ Gamma[3 + 4\ eps + 2\ z1 + z2 + 2\ z4])\), {{eps \[Rule] 0, 
                    eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                    z2 \[Rule] \(-\(1\/8\)\), 
                    z4 \[Rule] \(-\(69\/256\)\)}}]}, 
            MBint[\(-\(\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-2\)\ \
eps - z1 - z6\)\ Gamma[\(-4\)\ eps]\ Gamma[\(-z1\)]\ Gamma[\(-z2\)]\ \
Gamma[\(-1\) - eps - z1 - z4]\ Gamma[\(-1\) - 4\ eps - 2\ z1 - z2 - 
                          z4]\ Gamma[\(-z4\)]\ Gamma[
                        1 + 3\ eps + z1 + z4]\ Gamma[1 + z2 + z4]\ Gamma[
                        2 + 4\ eps + 2\ z1 + z2 + 2\ z4]\ Gamma[\(-1\) - 
                          2\ eps - z1 - z2 - z4 - z6]\ Gamma[
                        1 + 2\ eps + z1 + z2 + z4 - 
                          z6]\ Gamma[\(-z6\)]\ Gamma[
                        2 + 2\ eps + z1 + z4 + 
                          z6])\)/\((s\^2\ Gamma[\(-2\)\ eps]\ Gamma[\(-4\)\ \
eps - 2\ z1]\ Gamma[1 - z2]\ Gamma[
                        3 + 4\ eps + 2\ z1 + z2 + 
                          2\ z4]\ Gamma[\(-2\)\ z6])\)\)\), {{eps \[Rule] 0, 
                  eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                  z2 \[Rule] \(-\(1\/8\)\), z4 \[Rule] \(-\(69\/256\)\), 
                  z6 \[Rule] \(-\(19\/64\)\)}}]}, {{{MBint[\(-\(\((\((m\^2)\)\
\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-2\)\ eps - z1 - z6\)\ Gamma[\(-4\)\ eps]\ \
Gamma[2\ eps]\ Gamma[\(-3\)\ eps - z1]\ Gamma[\(-eps\) - 
                              z1]\ Gamma[\(-z1\)]\ Gamma[
                            1 + 3\ eps + z1]\ Gamma[\(-eps\) - z6]\ Gamma[
                            eps - z6]\ Gamma[\(-z6\)]\ Gamma[
                            1 - eps + z6])\)/\((s\^2\ Gamma[
                            1 - 2\ eps]\ Gamma[\(-4\)\ eps - 
                              2\ z1]\ Gamma[\(-2\)\ z6])\)\)\), {{eps \[Rule] 
                        0, eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                      z6 \[Rule] \(-\(19\/64\)\)}}]}, 
              MBint[\(-\(\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-2\)\ \
eps - z1 - z6\)\ Gamma[\(-4\)\ eps]\ Gamma[2\ eps]\ Gamma[\(-z1\)]\ Gamma[
                          1 + 3\ eps + z1]\ Gamma[\(-eps\) - z1 - 
                            z2]\ Gamma[\(-z2\)]\ Gamma[\(-2\)\ eps + 
                            z2]\ Gamma[\(-3\)\ eps - z1 + z2]\ Gamma[
                          eps - z2 - z6]\ Gamma[\(-eps\) + z2 - 
                            z6]\ Gamma[\(-z6\)]\ Gamma[
                          1 - eps + 
                            z6])\)/\((s\^2\ Gamma[\(-2\)\ eps]\ Gamma[\(-4\)\ \
eps - 2\ z1]\ Gamma[1 - z2]\ Gamma[
                          1 - 2\ eps + 
                            z2]\ Gamma[\(-2\)\ z6])\)\)\), {{eps \[Rule] 0, 
                    eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                    z2 \[Rule] \(-\(1\/8\)\), z6 \[Rule] \(-\(19\/64\)\)}}]}, 
            MBint[\(-\(\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-2\)\ \
eps - z1 - z6\)\ Gamma[\(-z1\)]\ Gamma[\(-eps\) - z1 - 
                          z2]\ Gamma[\(-z2\)]\ Gamma[\(-eps\) - z1 - 
                          z3]\ Gamma[\(-2\)\ eps - z2 - 
                          z3]\ Gamma[\(-z3\)]\ Gamma[
                        z2 + z3]\ Gamma[\(-2\)\ eps + z2 + z3]\ Gamma[
                        1 + eps + z1 + z2 + z3]\ Gamma[\(-eps\) + z2 - 
                          z6]\ Gamma[\(-eps\) + z3 - 
                          z6]\ Gamma[\(-z6\)]\ Gamma[
                        1 + eps - z2 - z3 + 
                          z6])\)/\((s\^2\ Gamma[\(-2\)\ eps]\ Gamma[
                        1 - z2]\ Gamma[
                        1 - z3]\ Gamma[\(-2\)\ eps - 2\ z1 - z2 - z3]\ Gamma[
                        1 - 2\ eps + z2 + z3]\ Gamma[\(-2\)\ eps + z2 + z3 - 
                          2\ z6])\)\)\), {{eps \[Rule] 0, 
                  eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                  z2 \[Rule] \(-\(1\/8\)\), z3 \[Rule] \(-\(19\/128\)\), 
                  z6 \[Rule] \(-\(19\/64\)\)}}]}, \
{MBint[\((\((m\^2)\)\^\(\(-1\) - 2\ eps - z2 - z4\)\ \((\(-s\))\)\^\(z2 + \
z4\)\ Gamma[\(-z1\)]\ Gamma[\(-z2\)]\ Gamma[\(-z3\)]\ Gamma[\(-2\) - 4\ eps - 
                        2\ z1 - z2 - z3 - 2\ z4]\ Gamma[\(-1\) - eps - z1 - 
                        z4]\ Gamma[\(-1\) - eps - z1 - z2 - z3 - 
                        z4]\ Gamma[\(-z4\)]\ Gamma[1 + z2 + z4]\ Gamma[
                      1 + 2\ eps + z1 + z2 + z4]\ Gamma[1 + z3 + z4]\ Gamma[
                      2 + eps + z1 + z2 + z3 + z4]\ Gamma[
                      2 + 2\ z1 + z2 + z3 + 
                        2\ z4])\)/\((s\ Gamma[\(-2\)\ eps]\ Gamma[
                      1 - z3]\ Gamma[\(-3\)\ eps - z1 - z4]\ Gamma[
                      2 + z2 + z3 + 2\ z4])\), {{eps \[Rule] 0, 
                  eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                  z2 \[Rule] \(-\(1\/8\)\), z3 \[Rule] \(-\(19\/128\)\), 
                  z4 \[Rule] \(-\(69\/256\)\)}}]}, \
{MBint[\((\((m\^2)\)\^\(\(-1\) - 2\ eps - z3 - z4\)\ \((\(-s\))\)\^\(z3 + \
z4\)\ Gamma[\(-z1\)]\ Gamma[\(-z2\)]\ Gamma[\(-z3\)]\ Gamma[\(-2\) - 4\ eps - 
                        2\ z1 - z2 - z3 - 2\ z4]\ Gamma[\(-1\) - eps - z1 - 
                        z4]\ Gamma[\(-1\) - eps - z1 - z2 - z3 - 
                        z4]\ Gamma[\(-z4\)]\ Gamma[1 + z2 + z4]\ Gamma[
                      1 + z3 + z4]\ Gamma[1 + 2\ eps + z1 + z3 + z4]\ Gamma[
                      2 + eps + z1 + z2 + z3 + z4]\ Gamma[
                      2 + 2\ z1 + z2 + z3 + 
                        2\ z4])\)/\((s\ Gamma[\(-2\)\ eps]\ Gamma[
                      1 - z2]\ Gamma[\(-3\)\ eps - z1 - z4]\ Gamma[
                      2 + z2 + z3 + 2\ z4])\), {{eps \[Rule] 0, 
                  eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                  z2 \[Rule] \(-\(1\/8\)\), z3 \[Rule] \(-\(19\/128\)\), 
                  z4 \[Rule] \(-\(69\/256\)\)}}]}, 
          MBint[\(-\(\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-2\)\ eps \
- z1 - z6\)\ Gamma[\(-z1\)]\ Gamma[\(-z2\)]\ Gamma[\(-z3\)]\ Gamma[\(-2\) - 
                        4\ eps - 2\ z1 - z2 - z3 - 2\ z4]\ Gamma[\(-1\) - 
                        eps - z1 - z4]\ Gamma[\(-1\) - eps - z1 - z2 - z3 - 
                        z4]\ Gamma[\(-z4\)]\ Gamma[1 + z2 + z4]\ Gamma[
                      1 + z3 + z4]\ Gamma[2 + eps + z1 + z2 + z3 + z4]\ Gamma[
                      2 + 2\ z1 + z2 + z3 + 2\ z4]\ Gamma[\(-1\) - 2\ eps - 
                        z1 - z2 - z4 - z6]\ Gamma[\(-1\) - 2\ eps - z1 - z3 - 
                        z4 - z6]\ Gamma[\(-z6\)]\ Gamma[
                      2 + 2\ eps + z1 + z4 + 
                        z6])\)/\((s\^2\ Gamma[\(-2\)\ eps]\ Gamma[
                      1 - z2]\ Gamma[
                      1 - z3]\ Gamma[\(-3\)\ eps - z1 - z4]\ Gamma[
                      2 + z2 + z3 + 2\ z4]\ Gamma[\(-2\) - 4\ eps - 2\ z1 - 
                        z2 - z3 - 2\ z4 - 2\ z6])\)\)\), {{eps \[Rule] 0, 
                eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
                z2 \[Rule] \(-\(1\/8\)\), z3 \[Rule] \(-\(19\/128\)\), 
                z4 \[Rule] \(-\(69\/256\)\), z6 \[Rule] \(-\(19\/64\)\)}}]}, 
        MBint[\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(z4 + z7\)\ \
\((\(-t\))\)\^\(\(-2\)\ eps - z1 - z4 - z6 - z7\)\ Gamma[\(-z1\)]\ \
Gamma[\(-z2\)]\ Gamma[\(-z3\)]\ Gamma[\(-1\) - eps - z1 - z4]\ Gamma[\(-1\) - 
                    eps - z1 - z2 - z3 - z4]\ Gamma[\(-z4\)]\ Gamma[
                  1 + z2 + z4]\ Gamma[1 + z3 + z4]\ Gamma[
                  2 + eps + z1 + z2 + z3 + z4]\ Gamma[
                  2 + 2\ z1 + z2 + z3 + 
                    2\ z4]\ Gamma[\(-z6\)]\ Gamma[\(-1\) - eps + z2 + z3 - 
                    z6 - z7]\ Gamma[\(-1\) - 2\ eps - z1 - z4 - z6 - 
                    z7]\ Gamma[\(-z7\)]\ Gamma[1 - z2 + z7]\ Gamma[
                  1 - z3 + z7]\ Gamma[2 + 2\ eps + z1 + z4 + z6 + z7]\ Gamma[
                  2 - z2 - z3 + 2\ z6 + 
                    2\ z7])\)/\((t\^2\ Gamma[\(-2\)\ eps]\ Gamma[
                  1 - z2]\ Gamma[1 - z3]\ Gamma[\(-3\)\ eps - z1 - z4]\ Gamma[
                  1 + eps + z1 + z2 + z3 + z4]\ Gamma[
                  2 + z2 + z3 + 2\ z4]\ Gamma[
                  2 - z2 - z3 + 2\ z7])\), {{eps \[Rule] 0, 
              eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
              z2 \[Rule] \(-\(1\/8\)\), z3 \[Rule] \(-\(19\/128\)\), 
              z4 \[Rule] \(-\(69\/256\)\), z6 \[Rule] \(-\(19\/64\)\), 
              z7 \[Rule] \(-\(665\/1024\)\)}}]}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(ser = 
      MBexpand[{integrals}, Exp[2*eps*EulerGamma], {eps, 0, 0}]\)], "Input"],

Cell[BoxData[
    \({MBint[\(-\(\(\((m\^2)\)\^\(2\ z1\)\ \((\(-s\))\)\^\(\(-2\)\ z1\)\ \
Gamma[\(-z1\)]\^2\ Gamma[\(-2\)\ z1 - z2]\ Gamma[\(-z1\) - 
                    z2]\ Gamma[\(-z2\)]\ Gamma[z1 + z2]\ Gamma[
                  1 + 2\ z1 + z2]\)\/\(2\ s\^2\ Gamma[
                  1 - 2\ z1 - z2]\)\)\), {{eps \[Rule] 0, 
            eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
            z2 \[Rule] \(-\(1\/8\)\)}}], 
      MBint[\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\ Gamma[\(-2\)\ z1 - z2]\ Gamma[\(-z1\) - 
                  z2]\ Gamma[\(-z2\)]\ Gamma[z1 + z2]\ Gamma[
                1 + 2\ z1 + z2]\ Gamma[\(-z1\) - z6]\ Gamma[
                z1 - z6]\ Gamma[\(-z6\)]\ Gamma[
                1 - z1 - z2 + z6])\)/\((2\ s\^2\ Gamma[1 - z2]\ Gamma[
                1 - 2\ z1 - z2]\ Gamma[\(-2\)\ z6])\), {{eps \[Rule] 0, 
            eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
            z2 \[Rule] \(-\(1\/8\)\), z6 \[Rule] \(-\(19\/64\)\)}}], 
      MBint[\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\
\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\)\/\(8\ eps\^2\ \
s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(EulerGamma\ \((m\^2)\)\^\(z1 \
+ z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ \
Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\)\/\(4\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\
\(-2\)\ z6]\) + \(EulerGamma\^2\ \((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\)\/\(4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(\((m\^2)\)\^\(z1 + z6\)\ \[Pi]\^2\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\)\/\(6\ \
s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \((\
\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\
\^3\ Gamma[1 + z6]\ Log[\(-s\)]\)\/\(4\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[\(-2\)\ z6]\) - \(EulerGamma\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\
\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 \
+ z6]\ Log[\(-s\)]\)\/\(2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ Log[\(-s\)]\^2\)\/\(4\ s\^2\ \
Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-2\)\ z1]\)\/\(2\ eps\ s\^2\ Gamma[\(-2\
\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(EulerGamma\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\
\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ \
Gamma[1 + z6]\ PolyGamma[0, \(-2\)\ z1]\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\
\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
Log[\(-s\)]\ PolyGamma[0, \(-2\)\ z1]\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\
\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, \(-2\)\ z1]\^2\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) \
- \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, \(-z1\)]\)\/\(2\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\
\) - \(EulerGamma\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, \(-z1\)]\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ Log[\(-s\)]\ PolyGamma[0, \
\(-z1\)]\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(2\ \
\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-2\)\ z1]\ \
PolyGamma[0, \(-z1\)]\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-z1\)]\^2\)\/\
\(s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(3\ \((m\^2)\)\^\(z1 + z6\)\
\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ \
Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, 1 + z1]\)\/\(8\ eps\ s\^2\ \
Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(3\ EulerGamma\ \((m\^2)\)\^\(z1 + \
z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ \
Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, 1 + z1]\)\/\(4\ s\^2\ \
Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(3\ \((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ Log[\(-s\)]\ PolyGamma[0, 1 + z1]\)\/\(4\ s\^2\ Gamma[\
\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(3\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\
\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ \
Gamma[1 + z6]\ PolyGamma[0, \(-2\)\ z1]\ PolyGamma[0, 1 + z1]\)\/\(2\ s\^2\ \
Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(3\ \((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-z1\)]\ PolyGamma[0, 1 + z1]\)\/\(2\ \
s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(9\ \((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, 1 + z1]\^2\)\/\(16\ s\^2\ Gamma[\(-2\)\ \
z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) \
- z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, 1 + z6]\)\/\(8\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ \
z6]\) - \(EulerGamma\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\
\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, 1 + z6]\)\/\(4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ Log[\(-s\)]\ PolyGamma[0, 1 \
+ z6]\)\/\(4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-2\)\ z1]\ \
PolyGamma[0, 1 + z6]\)\/\(2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-z1\)]\ \
PolyGamma[0, 1 + z6]\)\/\(2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \
\(3\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, 1 + z1]\ PolyGamma[0, 1 + z6]\)\/\(8\ s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\
\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, 1 + z6]\^2\)\/\(16\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) \
- \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[1, \(-2\)\ z1]\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(5\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[1, \(-z1\)]\)\/\(8\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(9\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[1, 1 + z1]\)\/\(16\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[1, \(-z6\)]\)\/\(8\
\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ PolyGamma[1, 1 + z6]\)\/\(16\ s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[\(-2\)\ z6]\), {{eps \[Rule] 0, 
            eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
            z6 \[Rule] \(-\(19\/64\)\)}}], 
      MBint[\(-\(\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\ Gamma[1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                    z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                    z2 - z6]\ Gamma[\(-z6\)]\ Gamma[
                    1 + z6])\)/\((4\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                    1 - z2]\ Gamma[
                    1 + z2]\ Gamma[\(-2\)\ z6])\)\)\) - \((EulerGamma\ \
\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ \
Gamma[1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[
                  1 + z6])\)/\((2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[
                  1 + z6]\ Log[\(-s\)])\)/\((2\ s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[
                  0, \(-2\)\ z1])\)/\((s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((3\ \((m\^2)\)\^\(z1 + z6\
\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[0, 
                  1 + z1])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[
                  0, \(-z1\) - z2])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[0, 
                  z2])\)/\((2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[0, 
                  1 + z2])\)/\((2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((3\ \((m\^2)\)\^\(z1 + z6\
\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[
                  0, \(-z1\) + z2])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[
                  0, \(-z2\) - z6])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[0, 
                  z2 - z6])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[0, 
                  1 + z6])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[1 + z2]\ Gamma[\(-2\)\ z6])\), {{eps \[Rule] 
              0, eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
            z2 \[Rule] \(-\(1\/8\)\), z6 \[Rule] \(-\(19\/64\)\)}}], 
      MBint[\(\((m\^2)\)\^\(2\ z1\)\ \((\(-s\))\)\^\(\(-2\)\ z1\)\ \
Gamma[\(-z1\)]\^2\ Gamma[\(-2\)\ z1 - z2]\ Gamma[\(-z1\) - z2]\ \
Gamma[\(-z2\)]\ Gamma[z1 + z2]\ Gamma[1 + 2\ z1 + z2]\)\/\(2\ s\^2\ Gamma[1 - \
2\ z1 - z2]\), {{eps \[Rule] 0, eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
            z2 \[Rule] \(-\(1\/8\)\)}}], 
      MBint[\(-\(\((\((m\^2)\)\^\(\(-1\) - z2 - z4\)\ \((\(-s\))\)\^\(z2 + z4\
\)\ Gamma[\(-z1\)]\ Gamma[\(-z2\)]\ Gamma[\(-1\) - z1 - z4]\ Gamma[\(-1\) - 
                    2\ z1 - z2 - z4]\ Gamma[\(-z4\)]\ Gamma[
                  1 + z1 + z4]\ Gamma[1 + z2 + z4]\ Gamma[
                  1 + z1 + z2 + z4]\ Gamma[
                  2 + 2\ z1 + z2 + 
                    2\ z4])\)/\((2\ s\ Gamma[\(-2\)\ z1]\ Gamma[
                  3 + 2\ z1 + z2 + 2\ z4])\)\)\), {{eps \[Rule] 0, 
            eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
            z2 \[Rule] \(-\(1\/8\)\), z4 \[Rule] \(-\(69\/256\)\)}}], 
      MBint[\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\ Gamma[\(-z2\)]\ Gamma[\(-1\) - z1 - z4]\ Gamma[\(-1\) - 
                  2\ z1 - z2 - z4]\ Gamma[\(-z4\)]\ Gamma[1 + z1 + z4]\ Gamma[
                1 + z2 + z4]\ Gamma[
                2 + 2\ z1 + z2 + 2\ z4]\ Gamma[\(-1\) - z1 - z2 - z4 - 
                  z6]\ Gamma[1 + z1 + z2 + z4 - z6]\ Gamma[\(-z6\)]\ Gamma[
                2 + z1 + z4 + z6])\)/\((2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                1 - z2]\ Gamma[
                3 + 2\ z1 + z2 + 2\ z4]\ Gamma[\(-2\)\ z6])\), {{eps \[Rule] 
              0, eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
            z2 \[Rule] \(-\(1\/8\)\), z4 \[Rule] \(-\(69\/256\)\), 
            z6 \[Rule] \(-\(19\/64\)\)}}], 
      MBint[\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\
\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\)\/\(8\ eps\^2\ \
s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(EulerGamma\ \((m\^2)\)\^\(z1 \
+ z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ \
Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\)\/\(4\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\
\(-2\)\ z6]\) + \(EulerGamma\^2\ \((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\)\/\(4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(\((m\^2)\)\^\(z1 + z6\)\ \[Pi]\^2\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\)\/\(6\ \
s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \((\
\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\
\^3\ Gamma[1 + z6]\ Log[\(-s\)]\)\/\(4\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[\(-2\)\ z6]\) - \(EulerGamma\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\
\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 \
+ z6]\ Log[\(-s\)]\)\/\(2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ Log[\(-s\)]\^2\)\/\(4\ s\^2\ \
Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, 1 + z1]\)\/\(8\ eps\ s\^2\ Gamma[\(-2\)\ \
z1]\ Gamma[\(-2\)\ z6]\) + \(EulerGamma\ \((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, 1 + z1]\)\/\(4\ s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\
\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
Log[\(-s\)]\ PolyGamma[0, 1 + z1]\)\/\(4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\
\)\ z6]\) + \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\
\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, 1 \
+ z1]\^2\)\/\(16\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\
\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + \
z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-2\)\ z6]\)\/\(2\ eps\ \
s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(EulerGamma\ \((m\^2)\)\^\(z1 \
+ z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ \
Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-2\)\ z6]\)\/\(s\^2\ \
Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ Log[\(-s\)]\ PolyGamma[0, \(-2\)\ z6]\)\/\(s\^2\ \
Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, 1 + z1]\ PolyGamma[0, \(-2\)\ z6]\)\/\(2\ \
s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\^\(z1 + z6\)\ \((\
\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\
\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-2\)\ z6]\^2\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\
\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, \(-z6\)]\)\/\(2\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\
\) - \(EulerGamma\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, \(-z6\)]\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ Log[\(-s\)]\ PolyGamma[0, \
\(-z6\)]\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, 1 + z1]\ \
PolyGamma[0, \(-z6\)]\)\/\(2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \
\(2\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, \(-2\)\ z6]\ PolyGamma[0, \(-z6\)]\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\
\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, \(-z6\)]\^2\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, 1 + z6]\)\/\(8\ \
eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(EulerGamma\ \
\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, 1 + z6]\)\/\(4\ \
s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \((\
\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\
\^3\ Gamma[1 + z6]\ Log[\(-s\)]\ PolyGamma[0, 1 + z6]\)\/\(4\ s\^2\ \
Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, 1 + z1]\ PolyGamma[0, 1 + z6]\)\/\(8\ \
s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\^\(z1 + z6\)\ \((\
\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\
\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-2\)\ z6]\ PolyGamma[0, 1 + z6]\)\/\(2\ \
s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \((\
\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\
\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-z6\)]\ PolyGamma[0, 1 + z6]\)\/\(2\ s\^2\ \
Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, 1 + z6]\^2\)\/\(16\ s\^2\ Gamma[\(-2\)\ \
z1]\ Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) \
- z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[1, \(-z1\)]\)\/\(8\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[1, 1 + z1]\)\/\(16\
\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ PolyGamma[1, \(-2\)\ z6]\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[\(-2\)\ z6]\) + \(5\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - \
z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[1, \(-z6\)]\)\/\(8\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[1, 1 + z6]\)\/\(16\
\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\), {{eps \[Rule] 0, 
            eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
            z6 \[Rule] \(-\(19\/64\)\)}}], 
      MBint[\(-\(\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\ Gamma[1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                    z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                    z2 - z6]\ Gamma[\(-z6\)]\ Gamma[
                    1 + z6])\)/\((4\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                    1 - z2]\ Gamma[
                    1 + z2]\ Gamma[\(-2\)\ z6])\)\)\) - \((EulerGamma\ \
\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ \
Gamma[1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[
                  1 + z6])\)/\((2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[
                  1 + z6]\ Log[\(-s\)])\)/\((2\ s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[0, 
                  1 + z1])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[
                  0, \(-z1\) - z2])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[0, 
                  z2])\)/\((2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[0, 
                  1 + z2])\)/\((2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[
                  0, \(-z1\) + z2])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[
                  0, \(-z2\) - z6])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((3\ \((m\^2)\)\^\(z1 + z6\
\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[0, 
                  z2 - z6])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[
                  0, \(-2\)\ z6])\)/\((s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[0, 
                  1 + z6])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[1 + z2]\ Gamma[\(-2\)\ z6])\), {{eps \[Rule] 
              0, eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
            z2 \[Rule] \(-\(1\/8\)\), z6 \[Rule] \(-\(19\/64\)\)}}], 
      MBint[\(-\(\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[
                    1 + z6]\)\/\(8\ eps\^2\ s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[\(-2\)\ z6]\)\)\) - \(EulerGamma\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\
\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ \
Gamma[1 + z6]\)\/\(4\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \
\(EulerGamma\^2\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\)\/\(4\ \
s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \
\[Pi]\^2\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ \
Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\)\/\(6\ s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\
\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
Log[\(-s\)]\)\/\(4\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(EulerGamma\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\
\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ Log[\(-s\)]\)\/\
\(2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\
\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ \
Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ Log[\(-s\)]\^2\)\/\(4\ s\^2\ Gamma[\(-2\)\ \
z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) \
- z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, \(-2\)\ z1]\)\/\(2\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ \
z6]\) - \(EulerGamma\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\
\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, \(-2\)\ z1]\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ Log[\(-s\)]\ PolyGamma[0, \
\(-2\)\ z1]\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-2\)\ \
z1]\^2\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\^\(z1 \
+ z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ \
Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-z1\)]\)\/\(2\ eps\ s\^2\ \
Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(EulerGamma\ \((m\^2)\)\^\(z1 + \
z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ \
Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-z1\)]\)\/\(s\^2\ Gamma[\(-2\
\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ Log[\(-s\)]\ PolyGamma[0, \(-z1\)]\)\/\(s\^2\ \
Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(2\ \((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-2\)\ z1]\ PolyGamma[0, \(-z1\)]\)\/\(s\
\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-z1\)]\^2\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[\(-2\)\ z6]\) - \(3\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - \
z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, 1 + z1]\)\/\(8\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ \
z6]\) - \(3\ EulerGamma\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - \
z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, 1 + z1]\)\/\(4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(3\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
Log[\(-s\)]\ PolyGamma[0, 1 + z1]\)\/\(4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\
\)\ z6]\) - \(3\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, \(-2\)\ z1]\ PolyGamma[0, 1 + z1]\)\/\(2\ s\^2\ Gamma[\(-2\)\ \
z1]\ Gamma[\(-2\)\ z6]\) + \(3\ \((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-z1\)]\ PolyGamma[0, 1 + z1]\)\/\(2\ \
s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(9\ \((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, 1 + z1]\^2\)\/\(16\ s\^2\ Gamma[\(-2\)\ \
z1]\ Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) \
- z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, 1 + z6]\)\/\(8\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ \
z6]\) + \(EulerGamma\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\
\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, 1 + z6]\)\/\(4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ Log[\(-s\)]\ PolyGamma[0, 1 \
+ z6]\)\/\(4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-2\)\ z1]\ \
PolyGamma[0, 1 + z6]\)\/\(2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-z1\)]\ \
PolyGamma[0, 1 + z6]\)\/\(2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(3\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, 1 + z1]\ PolyGamma[0, 1 + z6]\)\/\(8\ s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\
\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, 1 + z6]\^2\)\/\(16\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) \
+ \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[1, \(-2\)\ z1]\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \
\(5\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[1, \(-z1\)]\)\/\(8\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \
\(9\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[1, 1 + z1]\)\/\(16\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[1, \(-z6\)]\)\/\(8\
\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ PolyGamma[1, 1 + z6]\)\/\(16\ s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[\(-2\)\ z6]\), {{eps \[Rule] 0, 
            eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
            z6 \[Rule] \(-\(19\/64\)\)}}], 
      MBint[\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\ Gamma[1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[
                  1 + z6])\)/\((4\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((EulerGamma\ \((m\^2)\)\^\
\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[
                  1 + z6])\)/\((2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[
                  1 + z6]\ Log[\(-s\)])\)/\((2\ s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[
                  0, \(-2\)\ z1])\)/\((s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((3\ \((m\^2)\)\^\(z1 + z6\
\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[0, 
                  1 + z1])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[
                  0, \(-z1\) - z2])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[0, 
                  z2])\)/\((2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[0, 
                  1 + z2])\)/\((2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((3\ \((m\^2)\)\^\(z1 + z6\
\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[
                  0, \(-z1\) + z2])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[
                  0, \(-z2\) - z6])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[0, 
                  z2 - z6])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[0, 
                  1 + z6])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[1 + z2]\ Gamma[\(-2\)\ z6])\), {{eps \[Rule] 
              0, eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
            z2 \[Rule] \(-\(1\/8\)\), z6 \[Rule] \(-\(19\/64\)\)}}], 
      MBint[\(-\(\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[
                    1 + z6]\)\/\(8\ eps\^2\ s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[\(-2\)\ z6]\)\)\) - \(EulerGamma\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\
\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ \
Gamma[1 + z6]\)\/\(4\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \
\(EulerGamma\^2\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\)\/\(4\ \
s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \
\[Pi]\^2\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ \
Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\)\/\(6\ s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\
\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
Log[\(-s\)]\)\/\(4\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(EulerGamma\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\
\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ Log[\(-s\)]\)\/\
\(2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\
\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ \
Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ Log[\(-s\)]\^2\)\/\(4\ s\^2\ Gamma[\(-2\)\ \
z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) \
- z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, 1 + z1]\)\/\(8\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ \
z6]\) - \(EulerGamma\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\
\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, 1 + z1]\)\/\(4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ Log[\(-s\)]\ PolyGamma[0, 1 \
+ z1]\)\/\(4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, 1 + \
z1]\^2\)\/\(16\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\
\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\
\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-2\)\ z6]\)\/\(2\ eps\ \
s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(EulerGamma\ \((m\^2)\)\^\(z1 \
+ z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ \
Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-2\)\ z6]\)\/\(s\^2\ \
Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ Log[\(-s\)]\ PolyGamma[0, \(-2\)\ z6]\)\/\(s\^2\ \
Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, 1 + z1]\ PolyGamma[0, \(-2\)\ z6]\)\/\(2\ \
s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \((\
\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\
\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-2\)\ z6]\^2\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\
\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, \(-z6\)]\)\/\(2\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\
\) + \(EulerGamma\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, \(-z6\)]\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ Log[\(-s\)]\ PolyGamma[0, \
\(-z6\)]\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, 1 + z1]\ \
PolyGamma[0, \(-z6\)]\)\/\(2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(2\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, \(-2\)\ z6]\ PolyGamma[0, \(-z6\)]\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\
\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, \(-z6\)]\^2\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, 1 + z6]\)\/\(8\ \
eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(EulerGamma\ \
\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, 1 + z6]\)\/\(4\ \
s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\^\(z1 + z6\)\ \((\
\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\
\^3\ Gamma[1 + z6]\ Log[\(-s\)]\ PolyGamma[0, 1 + z6]\)\/\(4\ s\^2\ \
Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, 1 + z1]\ PolyGamma[0, 1 + z6]\)\/\(8\ \
s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \((\
\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\
\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-2\)\ z6]\ PolyGamma[0, 1 + z6]\)\/\(2\ \
s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\^\(z1 + z6\)\ \((\
\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\
\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-z6\)]\ PolyGamma[0, 1 + z6]\)\/\(2\ s\^2\ \
Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, 1 + z6]\^2\)\/\(16\ s\^2\ Gamma[\(-2\)\ \
z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) \
- z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[1, \(-z1\)]\)\/\(8\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[1, 1 + z1]\)\/\(16\
\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ PolyGamma[1, \(-2\)\ z6]\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[\(-2\)\ z6]\) - \(5\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - \
z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[1, \(-z6\)]\)\/\(8\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[1, 1 + z6]\)\/\(16\
\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\), {{eps \[Rule] 0, 
            eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
            z6 \[Rule] \(-\(19\/64\)\)}}], 
      MBint[\(-\(\(1\/\(s\ t\ Gamma[\(-2\)\ z1]\ Gamma[
                  2 + 2\ z7]\)\)\((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) + z7\)\ \((\(-t\))\)\^\(\(-z6\) - z7\)\ \
Gamma[\(-z1\)]\^3\ Gamma[
                1 + z1]\ Gamma[\(-z6\)]\ Gamma[\(-1\) - z6 - 
                  z7]\ Gamma[\(-z6\) - 
                  z7]\ Gamma[\(-z7\)]\ Gamma[1 + z7]\^2\ Gamma[
                1 + z6 + z7]\ Gamma[2 + 2\ z6 + 2\ z7])\)\)\), {{eps \[Rule] 
              0, eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
            z6 \[Rule] \(-\(19\/64\)\), z7 \[Rule] \(-\(665\/1024\)\)}}], 
      MBint[\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\ Gamma[1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[
                  1 + z6])\)/\((4\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((EulerGamma\ \((m\^2)\)\^\
\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[
                  1 + z6])\)/\((2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[
                  1 + z6]\ Log[\(-s\)])\)/\((2\ s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[0, 
                  1 + z1])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[
                  0, \(-z1\) - z2])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[0, 
                  z2])\)/\((2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[0, 
                  1 + z2])\)/\((2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[
                  0, \(-z1\) + z2])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[
                  0, \(-z2\) - z6])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((3\ \((m\^2)\)\^\(z1 + z6\
\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[0, 
                  z2 - z6])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[
                  0, \(-2\)\ z6])\)/\((s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[0, 
                  1 + z6])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[1 + z2]\ Gamma[\(-2\)\ z6])\), {{eps \[Rule] 
              0, eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
            z2 \[Rule] \(-\(1\/8\)\), z6 \[Rule] \(-\(19\/64\)\)}}], 
      MBint[\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\
\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\)\/\(2\ eps\^2\ \
s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(EulerGamma\ \((m\^2)\)\^\(z1 \
+ z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ \
Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\)\/\(2\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\
\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \[Pi]\^2\ \((\(-s\))\)\^\(\(-z1\) \
- z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + \
z6]\)\/\(3\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 \
+ z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ \
Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ Log[\(-s\)]\)\/\(2\ eps\ s\^2\ Gamma[\(-2\)\
\ z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\
\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\
\ Log[\(-t\)]\)\/\(2\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \
\(EulerGamma\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\
\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ Log[\(-t\)]\)\/\
\(s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ Log[\(-s\)]\ Log[\(-t\)]\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\
\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, \(-2\)\ z1]\)\/\(2\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ \
z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
Log[\(-t\)]\ PolyGamma[0, \(-2\)\ z1]\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\
\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, \(-z1\)]\)\/\(2\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\
\) + \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
Log[\(-t\)]\ PolyGamma[0, \(-z1\)]\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\
\ z6]\) + \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, 1 + z1]\)\/\(2\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ \
z6]\) + \(EulerGamma\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\
\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, 1 + z1]\)\/\(2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ Log[\(-s\)]\ PolyGamma[0, 1 \
+ z1]\)\/\(2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ Log[\(-t\)]\ PolyGamma[0, 1 \
+ z1]\)\/\(2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-2\)\ z1]\ \
PolyGamma[0, 1 + z1]\)\/\(2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-z1\)]\ \
PolyGamma[0, 1 + z1]\)\/\(2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, 1 + \
z1]\^2\)\/\(4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-2\)\ z6]\)\/\
\(2\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + \
z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ \
Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ Log[\(-t\)]\ PolyGamma[0, \(-2\)\ \
z6]\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\^\(z1 + \
z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ \
Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, 1 + z1]\ PolyGamma[0, \(-2\)\ \
z6]\)\/\(2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 \
+ z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ \
Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-z6\)]\)\/\(2\ eps\ s\^2\ \
Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ Log[\(-t\)]\ PolyGamma[0, \(-z6\)]\)\/\(s\^2\ \
Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, 1 + z1]\ PolyGamma[0, \(-z6\)]\)\/\(2\ \
s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(EulerGamma\ \((m\^2)\)\^\(z1 \
+ z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ \
Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, 1 + z6]\)\/\(2\ s\^2\ \
Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ Log[\(-s\)]\ PolyGamma[0, 1 + z6]\)\/\(2\ s\^2\ Gamma[\
\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ Log[\(-t\)]\ PolyGamma[0, 1 + z6]\)\/\(2\ s\^2\ Gamma[\
\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-2\)\ z1]\ PolyGamma[0, 1 + z6]\)\/\(2\ \
s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\^\(z1 + z6\)\ \((\
\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\
\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-z1\)]\ PolyGamma[0, 1 + z6]\)\/\(2\ s\^2\ \
Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-2\)\ z6]\ PolyGamma[0, 1 + z6]\)\/\(2\ \
s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\^\(z1 + z6\)\ \((\
\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\
\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-z6\)]\ PolyGamma[0, 1 + z6]\)\/\(2\ s\^2\ \
Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, 1 + z6]\^2\)\/\(4\ s\^2\ Gamma[\(-2\)\ \
z1]\ Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) \
- z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[1, 1 + z1]\)\/\(4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[1, 1 + z6]\)\/\(4\ \
s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\), {{eps \[Rule] 0, 
            eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
            z6 \[Rule] \(-\(19\/64\)\)}}], 
      MBint[\(-\(\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\ Gamma[1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                    z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                    z2 - z6]\ Gamma[\(-z6\)]\ Gamma[
                    1 + z6])\)/\((2\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                    1 - z2]\ Gamma[
                    1 + z2]\ Gamma[\(-2\)\ z6])\)\)\) - \((EulerGamma\ \
\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ \
Gamma[1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[
                  1 + z6])\)/\((s\^2\ Gamma[\(-2\)\ z1]\ Gamma[1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[
                  1 + z6]\ Log[\(-s\)])\)/\((s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[
                  0, \(-2\)\ z1])\)/\((s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[0, 
                  1 + z1])\)/\((2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[
                  0, \(-z1\) - z2])\)/\((2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[
                  0, \(-z1\) + z2])\)/\((2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[
                  0, \(-z2\) - z6])\)/\((2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[0, 
                  z2 - z6])\)/\((2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[
                  0, \(-2\)\ z6])\)/\((s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[0, 
                  1 + z6])\)/\((2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[1 + z2]\ Gamma[\(-2\)\ z6])\), {{eps \[Rule] 
              0, eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
            z2 \[Rule] \(-\(1\/8\)\), z6 \[Rule] \(-\(19\/64\)\)}}], 
      MBint[\(\(1\/\(s\ t\ Gamma[
                2 + 2\ z4]\ Gamma[\(-2\)\ z6]\)\)\((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(z4 - z6\)\ \((\(-t\))\)\^\(\(-z1\) - z4\)\ Gamma[\(-z1\)]\ \
Gamma[\(-1\) - z1 - z4]\^2\ Gamma[\(-z4\)]\ Gamma[1 + z4]\^2\ Gamma[
              2 + z1 + z4]\ Gamma[
              2 + 2\ z1 + 2\ z4]\ Gamma[\(-z6\)]\^3\ Gamma[
              1 + z6])\)\), {{eps \[Rule] 0, 
            eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
            z4 \[Rule] \(-\(69\/256\)\), z6 \[Rule] \(-\(19\/64\)\)}}], 
      MBint[\(-\(\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[
                    1 + z6]\)\/\(8\ eps\^2\ s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[\(-2\)\ z6]\)\)\) - \(EulerGamma\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\
\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ \
Gamma[1 + z6]\)\/\(4\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \
\(EulerGamma\^2\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\)\/\(4\ \
s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \
\[Pi]\^2\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ \
Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\)\/\(6\ s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\
\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
Log[\(-s\)]\)\/\(4\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(EulerGamma\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\
\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ Log[\(-s\)]\)\/\
\(2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\
\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ \
Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ Log[\(-s\)]\^2\)\/\(4\ s\^2\ Gamma[\(-2\)\ \
z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) \
- z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, \(-2\)\ z1]\)\/\(2\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ \
z6]\) - \(EulerGamma\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\
\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, \(-2\)\ z1]\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ Log[\(-s\)]\ PolyGamma[0, \
\(-2\)\ z1]\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-2\)\ \
z1]\^2\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\^\(z1 \
+ z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ \
Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-z1\)]\)\/\(2\ eps\ s\^2\ \
Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(EulerGamma\ \((m\^2)\)\^\(z1 + \
z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ \
Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-z1\)]\)\/\(s\^2\ Gamma[\(-2\
\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ Log[\(-s\)]\ PolyGamma[0, \(-z1\)]\)\/\(s\^2\ \
Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(2\ \((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-2\)\ z1]\ PolyGamma[0, \(-z1\)]\)\/\(s\
\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-z1\)]\^2\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[\(-2\)\ z6]\) - \(3\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - \
z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, 1 + z1]\)\/\(8\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ \
z6]\) - \(3\ EulerGamma\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - \
z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, 1 + z1]\)\/\(4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(3\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
Log[\(-s\)]\ PolyGamma[0, 1 + z1]\)\/\(4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\
\)\ z6]\) - \(3\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, \(-2\)\ z1]\ PolyGamma[0, 1 + z1]\)\/\(2\ s\^2\ Gamma[\(-2\)\ \
z1]\ Gamma[\(-2\)\ z6]\) + \(3\ \((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-z1\)]\ PolyGamma[0, 1 + z1]\)\/\(2\ \
s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(9\ \((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, 1 + z1]\^2\)\/\(16\ s\^2\ Gamma[\(-2\)\ \
z1]\ Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) \
- z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, 1 + z6]\)\/\(8\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ \
z6]\) + \(EulerGamma\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\
\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, 1 + z6]\)\/\(4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ Log[\(-s\)]\ PolyGamma[0, 1 \
+ z6]\)\/\(4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-2\)\ z1]\ \
PolyGamma[0, 1 + z6]\)\/\(2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-z1\)]\ \
PolyGamma[0, 1 + z6]\)\/\(2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(3\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, 1 + z1]\ PolyGamma[0, 1 + z6]\)\/\(8\ s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\
\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, 1 + z6]\^2\)\/\(16\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) \
+ \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[1, \(-2\)\ z1]\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \
\(5\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[1, \(-z1\)]\)\/\(8\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \
\(9\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[1, 1 + z1]\)\/\(16\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[1, \(-z6\)]\)\/\(8\
\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ PolyGamma[1, 1 + z6]\)\/\(16\ s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[\(-2\)\ z6]\), {{eps \[Rule] 0, 
            eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
            z6 \[Rule] \(-\(19\/64\)\)}}], 
      MBint[\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\ Gamma[1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[
                  1 + z6])\)/\((4\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((EulerGamma\ \((m\^2)\)\^\
\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[
                  1 + z6])\)/\((2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[
                  1 + z6]\ Log[\(-s\)])\)/\((2\ s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[
                  0, \(-2\)\ z1])\)/\((s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((3\ \((m\^2)\)\^\(z1 + z6\
\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[0, 
                  1 + z1])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[
                  0, \(-z1\) - z2])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[0, 
                  z2])\)/\((2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[0, 
                  1 + z2])\)/\((2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((3\ \((m\^2)\)\^\(z1 + z6\
\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[
                  0, \(-z1\) + z2])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[
                  0, \(-z2\) - z6])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[0, 
                  z2 - z6])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[0, 
                  1 + z6])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[1 + z2]\ Gamma[\(-2\)\ z6])\), {{eps \[Rule] 
              0, eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
            z2 \[Rule] \(-\(1\/8\)\), z6 \[Rule] \(-\(19\/64\)\)}}], 
      MBint[\(\((m\^2)\)\^\(2\ z1\)\ \((\(-s\))\)\^\(\(-2\)\ z1\)\ \
Gamma[\(-z1\)]\^2\ Gamma[\(-2\)\ z1 - z2]\ Gamma[\(-z1\) - z2]\ \
Gamma[\(-z2\)]\ Gamma[z1 + z2]\ Gamma[1 + 2\ z1 + z2]\)\/\(2\ s\^2\ Gamma[1 - \
2\ z1 - z2]\), {{eps \[Rule] 0, eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
            z2 \[Rule] \(-\(1\/8\)\)}}], 
      MBint[\(-\(\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\ Gamma[\(-2\)\ z1 - z2]\ Gamma[\(-z1\) - 
                    z2]\ Gamma[\(-z2\)]\ Gamma[z1 + z2]\ Gamma[
                  1 + 2\ z1 + z2]\ Gamma[\(-z1\) - z6]\ Gamma[
                  z1 - z6]\ Gamma[\(-z6\)]\ Gamma[
                  1 - z1 - z2 + z6])\)/\((2\ s\^2\ Gamma[1 - z2]\ Gamma[
                  1 - 2\ z1 - z2]\ Gamma[\(-2\)\ z6])\)\)\), {{eps \[Rule] 0, 
            eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
            z2 \[Rule] \(-\(1\/8\)\), z6 \[Rule] \(-\(19\/64\)\)}}], 
      MBint[\(-\(\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[
                    1 + z6]\)\/\(8\ eps\^2\ s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[\(-2\)\ z6]\)\)\) - \(EulerGamma\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\
\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ \
Gamma[1 + z6]\)\/\(4\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \
\(EulerGamma\^2\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\)\/\(4\ \
s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \
\[Pi]\^2\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ \
Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\)\/\(6\ s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\
\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
Log[\(-s\)]\)\/\(4\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(EulerGamma\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\
\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ Log[\(-s\)]\)\/\
\(2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\
\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ \
Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ Log[\(-s\)]\^2\)\/\(4\ s\^2\ Gamma[\(-2\)\ \
z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) \
- z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, \(-2\)\ z1]\)\/\(2\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ \
z6]\) - \(EulerGamma\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\
\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, \(-2\)\ z1]\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ Log[\(-s\)]\ PolyGamma[0, \
\(-2\)\ z1]\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-2\)\ \
z1]\^2\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\^\(z1 \
+ z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ \
Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-z1\)]\)\/\(2\ eps\ s\^2\ \
Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(EulerGamma\ \((m\^2)\)\^\(z1 + \
z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ \
Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-z1\)]\)\/\(s\^2\ Gamma[\(-2\
\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ Log[\(-s\)]\ PolyGamma[0, \(-z1\)]\)\/\(s\^2\ \
Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(2\ \((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-2\)\ z1]\ PolyGamma[0, \(-z1\)]\)\/\(s\
\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-z1\)]\^2\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[\(-2\)\ z6]\) - \(3\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - \
z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, 1 + z1]\)\/\(8\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ \
z6]\) - \(3\ EulerGamma\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - \
z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, 1 + z1]\)\/\(4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(3\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
Log[\(-s\)]\ PolyGamma[0, 1 + z1]\)\/\(4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\
\)\ z6]\) - \(3\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, \(-2\)\ z1]\ PolyGamma[0, 1 + z1]\)\/\(2\ s\^2\ Gamma[\(-2\)\ \
z1]\ Gamma[\(-2\)\ z6]\) + \(3\ \((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-z1\)]\ PolyGamma[0, 1 + z1]\)\/\(2\ \
s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(9\ \((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, 1 + z1]\^2\)\/\(16\ s\^2\ Gamma[\(-2\)\ \
z1]\ Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) \
- z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, 1 + z6]\)\/\(8\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ \
z6]\) + \(EulerGamma\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\
\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, 1 + z6]\)\/\(4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ Log[\(-s\)]\ PolyGamma[0, 1 \
+ z6]\)\/\(4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-2\)\ z1]\ \
PolyGamma[0, 1 + z6]\)\/\(2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-z1\)]\ \
PolyGamma[0, 1 + z6]\)\/\(2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(3\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, 1 + z1]\ PolyGamma[0, 1 + z6]\)\/\(8\ s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\
\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, 1 + z6]\^2\)\/\(16\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) \
+ \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[1, \(-2\)\ z1]\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \
\(5\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[1, \(-z1\)]\)\/\(8\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \
\(9\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[1, 1 + z1]\)\/\(16\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[1, \(-z6\)]\)\/\(8\
\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ PolyGamma[1, 1 + z6]\)\/\(16\ s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[\(-2\)\ z6]\), {{eps \[Rule] 0, 
            eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
            z6 \[Rule] \(-\(19\/64\)\)}}], 
      MBint[\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\ Gamma[1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[
                  1 + z6])\)/\((4\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((EulerGamma\ \((m\^2)\)\^\
\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[
                  1 + z6])\)/\((2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[
                  1 + z6]\ Log[\(-s\)])\)/\((2\ s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[
                  0, \(-2\)\ z1])\)/\((s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((3\ \((m\^2)\)\^\(z1 + z6\
\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[0, 
                  1 + z1])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[
                  0, \(-z1\) - z2])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[0, 
                  z2])\)/\((2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[0, 
                  1 + z2])\)/\((2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((3\ \((m\^2)\)\^\(z1 + z6\
\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[
                  0, \(-z1\) + z2])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[
                  0, \(-z2\) - z6])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[0, 
                  z2 - z6])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[0, 
                  1 + z6])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[1 + z2]\ Gamma[\(-2\)\ z6])\), {{eps \[Rule] 
              0, eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
            z2 \[Rule] \(-\(1\/8\)\), z6 \[Rule] \(-\(19\/64\)\)}}], 
      MBint[\(-\(\(\((m\^2)\)\^\(2\ z1\)\ \((\(-s\))\)\^\(\(-2\)\ z1\)\ \
Gamma[\(-z1\)]\^2\ Gamma[\(-2\)\ z1 - z2]\ Gamma[\(-z1\) - 
                    z2]\ Gamma[\(-z2\)]\ Gamma[z1 + z2]\ Gamma[
                  1 + 2\ z1 + z2]\)\/\(2\ s\^2\ Gamma[
                  1 - 2\ z1 - z2]\)\)\), {{eps \[Rule] 0, 
            eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
            z2 \[Rule] \(-\(1\/8\)\)}}], 
      MBint[\((\((m\^2)\)\^\(\(-1\) - z2 - z4\)\ \((\(-s\))\)\^\(z2 + z4\)\ \
Gamma[\(-z1\)]\ Gamma[\(-z2\)]\ Gamma[\(-1\) - z1 - z4]\ Gamma[\(-1\) - 
                  2\ z1 - z2 - z4]\ Gamma[\(-z4\)]\ Gamma[1 + z1 + z4]\ Gamma[
                1 + z2 + z4]\ Gamma[1 + z1 + z2 + z4]\ Gamma[
                2 + 2\ z1 + z2 + 2\ z4])\)/\((2\ s\ Gamma[\(-2\)\ z1]\ Gamma[
                3 + 2\ z1 + z2 + 2\ z4])\), {{eps \[Rule] 0, 
            eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
            z2 \[Rule] \(-\(1\/8\)\), z4 \[Rule] \(-\(69\/256\)\)}}], 
      MBint[\(-\(\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\ Gamma[\(-z2\)]\ Gamma[\(-1\) - z1 - z4]\ Gamma[\(-1\) - 
                    2\ z1 - z2 - z4]\ Gamma[\(-z4\)]\ Gamma[
                  1 + z1 + z4]\ Gamma[1 + z2 + z4]\ Gamma[
                  2 + 2\ z1 + z2 + 2\ z4]\ Gamma[\(-1\) - z1 - z2 - z4 - 
                    z6]\ Gamma[1 + z1 + z2 + z4 - z6]\ Gamma[\(-z6\)]\ Gamma[
                  2 + z1 + z4 + z6])\)/\((2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  3 + 2\ z1 + z2 + 
                    2\ z4]\ Gamma[\(-2\)\ z6])\)\)\), {{eps \[Rule] 0, 
            eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
            z2 \[Rule] \(-\(1\/8\)\), z4 \[Rule] \(-\(69\/256\)\), 
            z6 \[Rule] \(-\(19\/64\)\)}}], 
      MBint[\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\
\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\)\/\(8\ eps\^2\ \
s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(EulerGamma\ \((m\^2)\)\^\(z1 \
+ z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ \
Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\)\/\(4\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\
\(-2\)\ z6]\) + \(EulerGamma\^2\ \((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\)\/\(4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(\((m\^2)\)\^\(z1 + z6\)\ \[Pi]\^2\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\)\/\(6\ \
s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \((\
\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\
\^3\ Gamma[1 + z6]\ Log[\(-s\)]\)\/\(4\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[\(-2\)\ z6]\) - \(EulerGamma\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\
\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 \
+ z6]\ Log[\(-s\)]\)\/\(2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ Log[\(-s\)]\^2\)\/\(4\ s\^2\ \
Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-2\)\ z1]\)\/\(2\ eps\ s\^2\ Gamma[\(-2\
\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(EulerGamma\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\
\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ \
Gamma[1 + z6]\ PolyGamma[0, \(-2\)\ z1]\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\
\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
Log[\(-s\)]\ PolyGamma[0, \(-2\)\ z1]\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\
\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, \(-2\)\ z1]\^2\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) \
- \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, \(-z1\)]\)\/\(2\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\
\) - \(EulerGamma\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, \(-z1\)]\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ Log[\(-s\)]\ PolyGamma[0, \
\(-z1\)]\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(2\ \
\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-2\)\ z1]\ \
PolyGamma[0, \(-z1\)]\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-z1\)]\^2\)\/\
\(s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(3\ \((m\^2)\)\^\(z1 + z6\)\
\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ \
Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, 1 + z1]\)\/\(8\ eps\ s\^2\ \
Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(3\ EulerGamma\ \((m\^2)\)\^\(z1 + \
z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ \
Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, 1 + z1]\)\/\(4\ s\^2\ \
Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(3\ \((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ Log[\(-s\)]\ PolyGamma[0, 1 + z1]\)\/\(4\ s\^2\ Gamma[\
\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(3\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\
\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ \
Gamma[1 + z6]\ PolyGamma[0, \(-2\)\ z1]\ PolyGamma[0, 1 + z1]\)\/\(2\ s\^2\ \
Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \(3\ \((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-z1\)]\ PolyGamma[0, 1 + z1]\)\/\(2\ \
s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(9\ \((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, 1 + z1]\^2\)\/\(16\ s\^2\ Gamma[\(-2\)\ \
z1]\ Gamma[\(-2\)\ z6]\) - \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) \
- z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, 1 + z6]\)\/\(8\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ \
z6]\) - \(EulerGamma\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\
\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, 1 + z6]\)\/\(4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ Log[\(-s\)]\ PolyGamma[0, 1 \
+ z6]\)\/\(4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-2\)\ z1]\ \
PolyGamma[0, 1 + z6]\)\/\(2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[0, \(-z1\)]\ \
PolyGamma[0, 1 + z6]\)\/\(2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) - \
\(3\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, 1 + z1]\ PolyGamma[0, 1 + z6]\)\/\(8\ s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\
\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[0, 1 + z6]\^2\)\/\(16\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) \
- \(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[1, \(-2\)\ z1]\)\/\(s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(5\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[1, \(-z1\)]\)\/\(8\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(9\ \((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ \
PolyGamma[1, 1 + z1]\)\/\(16\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \
\(\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ \
Gamma[1 + z1]\ Gamma[\(-z6\)]\^3\ Gamma[1 + z6]\ PolyGamma[1, \(-z6\)]\)\/\(8\
\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[\(-2\)\ z6]\) + \(\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\^3\ Gamma[1 + z1]\ Gamma[\(-z6\
\)]\^3\ Gamma[1 + z6]\ PolyGamma[1, 1 + z6]\)\/\(16\ s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[\(-2\)\ z6]\), {{eps \[Rule] 0, 
            eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
            z6 \[Rule] \(-\(19\/64\)\)}}], 
      MBint[\(-\(\((\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ \
Gamma[\(-z1\)]\ Gamma[1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                    z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                    z2 - z6]\ Gamma[\(-z6\)]\ Gamma[
                    1 + z6])\)/\((4\ eps\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                    1 - z2]\ Gamma[
                    1 + z2]\ Gamma[\(-2\)\ z6])\)\)\) - \((EulerGamma\ \
\((m\^2)\)\^\(z1 + z6\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ \
Gamma[1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[
                  1 + z6])\)/\((2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[
                  1 + z6]\ Log[\(-s\)])\)/\((2\ s\^2\ Gamma[\(-2\)\ z1]\ \
Gamma[1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[
                  0, \(-2\)\ z1])\)/\((s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((3\ \((m\^2)\)\^\(z1 + z6\
\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[0, 
                  1 + z1])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[
                  0, \(-z1\) - z2])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[0, 
                  z2])\)/\((2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[0, 
                  1 + z2])\)/\((2\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((3\ \((m\^2)\)\^\(z1 + z6\
\)\ \((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[
                  0, \(-z1\) + z2])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) - \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[
                  0, \(-z2\) - z6])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[0, 
                  z2 - z6])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[
                  1 + z2]\ Gamma[\(-2\)\ z6])\) + \((\((m\^2)\)\^\(z1 + z6\)\ \
\((\(-s\))\)\^\(\(-z1\) - z6\)\ Gamma[\(-z1\)]\ Gamma[
                  1 + z1]\ Gamma[\(-z1\) - z2]\ Gamma[\(-z2\)]\ Gamma[
                  z2]\ Gamma[\(-z1\) + z2]\ Gamma[\(-z2\) - z6]\ Gamma[
                  z2 - z6]\ Gamma[\(-z6\)]\ Gamma[1 + z6]\ PolyGamma[0, 
                  1 + z6])\)/\((4\ s\^2\ Gamma[\(-2\)\ z1]\ Gamma[
                  1 - z2]\ Gamma[1 + z2]\ Gamma[\(-2\)\ z6])\), {{eps \[Rule] 
              0, eta \[Rule] 0}, {z1 \[Rule] \(-\(1\/4\)\), 
            z2 \[Rule] \(-\(1\/8\)\), 
            z6 \[Rule] \(-\(19\/64\)\)}}]}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(MBintegrate[
      ser, {s \[Rule] \(-1\)/2, t \[Rule] \(-3\), m \[Rule] 1}]\)], "Input"],

Cell[BoxData[
    \("Shifting contours..."\)], "Print"],

Cell["Performing 0 1-dimensional integrations", "Print"],

Cell[BoxData[
    \("Higher-dimensional integrals"\)], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart1eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]4\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart1eps0", " (dim ", 4, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart2eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]4\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart2eps0", " (dim ", 4, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart3eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart3eps0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart4eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart4eps0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart5eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart5eps0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart6eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart6eps0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart7eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart7eps0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart8eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart8eps0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart9eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart9eps0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart10eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart10eps0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart11eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart11eps0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart12eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart12eps0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart13eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart13eps0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart14eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart14eps0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart15eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart15eps0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart16eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart16eps0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart17eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart17eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart18eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart18eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart19eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart19eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart20eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart20eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart21eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart21eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart22eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart22eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart23eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart23eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart24eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart24eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart25eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart25eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart26eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart26eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart27eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart27eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart28eps0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart28eps0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart29eps-1"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart29eps-1", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart30eps-1"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart30eps-1", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart31eps-1"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart31eps-1", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart32eps-1"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart32eps-1", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart33eps-1"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart33eps-1", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart34eps-1"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart34eps-1", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart35eps-1"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart35eps-1", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart36eps-1"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart36eps-1", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart37eps-1"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart37eps-1", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart38eps-1"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart38eps-1", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart39eps-1"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart39eps-1", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart40eps-1"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart40eps-1", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart41eps-1"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart41eps-1", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart42eps-1"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart42eps-1", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart43eps-1"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart43eps-1", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart44eps-1"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart44eps-1", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart45eps-2"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart45eps-2", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart46eps-2"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart46eps-2", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart47eps-2"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart47eps-2", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart48eps-2"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart48eps-2", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart49eps-2"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart49eps-2", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart50eps-2"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart50eps-2", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart51eps-2"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart51eps-2", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart52eps-2"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart52eps-2", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart1eps0"\),
      SequenceForm[ "Running ", "MBpart1eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart2eps0"\),
      SequenceForm[ "Running ", "MBpart2eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart3eps0"\),
      SequenceForm[ "Running ", "MBpart3eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart4eps0"\),
      SequenceForm[ "Running ", "MBpart4eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart5eps0"\),
      SequenceForm[ "Running ", "MBpart5eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart6eps0"\),
      SequenceForm[ "Running ", "MBpart6eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart7eps0"\),
      SequenceForm[ "Running ", "MBpart7eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart8eps0"\),
      SequenceForm[ "Running ", "MBpart8eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart9eps0"\),
      SequenceForm[ "Running ", "MBpart9eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart10eps0"\),
      SequenceForm[ "Running ", "MBpart10eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart11eps0"\),
      SequenceForm[ "Running ", "MBpart11eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart12eps0"\),
      SequenceForm[ "Running ", "MBpart12eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart13eps0"\),
      SequenceForm[ "Running ", "MBpart13eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart14eps0"\),
      SequenceForm[ "Running ", "MBpart14eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart15eps0"\),
      SequenceForm[ "Running ", "MBpart15eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart16eps0"\),
      SequenceForm[ "Running ", "MBpart16eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart17eps0"\),
      SequenceForm[ "Running ", "MBpart17eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart18eps0"\),
      SequenceForm[ "Running ", "MBpart18eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart19eps0"\),
      SequenceForm[ "Running ", "MBpart19eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart20eps0"\),
      SequenceForm[ "Running ", "MBpart20eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart21eps0"\),
      SequenceForm[ "Running ", "MBpart21eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart22eps0"\),
      SequenceForm[ "Running ", "MBpart22eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart23eps0"\),
      SequenceForm[ "Running ", "MBpart23eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart24eps0"\),
      SequenceForm[ "Running ", "MBpart24eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart25eps0"\),
      SequenceForm[ "Running ", "MBpart25eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart26eps0"\),
      SequenceForm[ "Running ", "MBpart26eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart27eps0"\),
      SequenceForm[ "Running ", "MBpart27eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart28eps0"\),
      SequenceForm[ "Running ", "MBpart28eps0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart29eps-1"\),
      SequenceForm[ "Running ", "MBpart29eps-1"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart30eps-1"\),
      SequenceForm[ "Running ", "MBpart30eps-1"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart31eps-1"\),
      SequenceForm[ "Running ", "MBpart31eps-1"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart32eps-1"\),
      SequenceForm[ "Running ", "MBpart32eps-1"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart33eps-1"\),
      SequenceForm[ "Running ", "MBpart33eps-1"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart34eps-1"\),
      SequenceForm[ "Running ", "MBpart34eps-1"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart35eps-1"\),
      SequenceForm[ "Running ", "MBpart35eps-1"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart36eps-1"\),
      SequenceForm[ "Running ", "MBpart36eps-1"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart37eps-1"\),
      SequenceForm[ "Running ", "MBpart37eps-1"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart38eps-1"\),
      SequenceForm[ "Running ", "MBpart38eps-1"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart39eps-1"\),
      SequenceForm[ "Running ", "MBpart39eps-1"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart40eps-1"\),
      SequenceForm[ "Running ", "MBpart40eps-1"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart41eps-1"\),
      SequenceForm[ "Running ", "MBpart41eps-1"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart42eps-1"\),
      SequenceForm[ "Running ", "MBpart42eps-1"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart43eps-1"\),
      SequenceForm[ "Running ", "MBpart43eps-1"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart44eps-1"\),
      SequenceForm[ "Running ", "MBpart44eps-1"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart45eps-2"\),
      SequenceForm[ "Running ", "MBpart45eps-2"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart46eps-2"\),
      SequenceForm[ "Running ", "MBpart46eps-2"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart47eps-2"\),
      SequenceForm[ "Running ", "MBpart47eps-2"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart48eps-2"\),
      SequenceForm[ "Running ", "MBpart48eps-2"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart49eps-2"\),
      SequenceForm[ "Running ", "MBpart49eps-2"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart50eps-2"\),
      SequenceForm[ "Running ", "MBpart50eps-2"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart51eps-2"\),
      SequenceForm[ "Running ", "MBpart51eps-2"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart52eps-2"\),
      SequenceForm[ "Running ", "MBpart52eps-2"],
      Editable->False]], "Print"],

Cell[BoxData[
    \({\(-3.9010397686264504`\) + 0.3203021167911645`\/eps\^2 - 
        0.2667238859136233`\/eps, {\(\(0.004427752734693823`\)\(\
\[InvisibleSpace]\)\) + 0.00004788351828577323`\/eps\^2 + 
          0.0000875080421094382`\/eps, 0}}\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(\( (*\ 
      sectors : \
\[IndentingNewLine]\[IndentingNewLine]{{0.3203772836085565`\/ep\^2 - 
              0.26661944018488715`\/ep, \(-\(0.00030840055074709047`\/ep\^2\)\
\) - 0.0030136395958849436`\/ep, \(-\(0.001357853530901787`\/ep\^2\)\) + 
              0.07127306807466487`\/ep}}\[IndentingNewLine]*) \)\(\
\[IndentingNewLine]\)\)\)], "Input"],

Cell[BoxData[
    \( (*\ result\ from\ [12], \ expanded\ *) \)], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(B7l4m1Nms = \((\((ms^2*\((24*ep*\((\(-z3\) + 6*ep*z4)\) + 
                  29*ep^2*Log[ms]^4 + 
                  4*ep*Log[ms]^3*\((13 - 12*ep*Log[x])\) - 
                  12*Log[ms]^2*\((\(-3\) + 34*ep^2*z2 + 4*ep*Log[x])\) + 
                  16*ep*Log[
                      ms]*\((\(-3\)*z2 + 24*ep*z2*Log[x] + ep*Log[x]^3 - 
                        3*ep*\((6*z2 + Log[x]^2)\)*Log[1 + x])\) + 
                  24*ep^2*
                    Log[ms]*\((7*z3 - 4*Log[x]*PolyLog[2, \(-x\)] + 
                        4*PolyLog[3, \(-x\)])\))\))\)/\((24*
              ep^2)\))\)\)], "Input"],

Cell[BoxData[
    \(\(1\/\(24\ ep\^2\)\) \((ms\^2\ \((24\ ep\ \((\(-z3\) + 6\ ep\ z4)\) + 
              29\ ep\^2\ Log[ms]\^4 + 
              4\ ep\ Log[ms]\^3\ \((13 - 12\ ep\ Log[x])\) - 
              12\ Log[ms]\^2\ \((\(-3\) + 34\ ep\^2\ z2 + 4\ ep\ Log[x])\) + 
              16\ ep\ Log[
                  ms]\ \((\(-3\)\ z2 + 24\ ep\ z2\ Log[x] + ep\ Log[x]\^3 - 
                    3\ ep\ \((6\ z2 + Log[x]\^2)\)\ Log[1 + x])\) + 
              24\ ep\^2\ Log[
                  ms]\ \((7\ z3 - 4\ Log[x]\ PolyLog[2, \(-x\)] + 
                    4\ PolyLog[3, \(-x\)])\))\))\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(N[\(\(\(\(\(\(% /. z2 \[Rule] Zeta[2]\) /. z3 \[Rule] Zeta[3]\) /. 
                  z4 \[Rule] Zeta[4]\) /. ms \[Rule] \(-1\)/s\) /. 
              x \[Rule] t/s\) /. s \[Rule] \(-100\)\) /. t \[Rule] \(-3\)] // 
      Expand\)], "Input"],

Cell[BoxData[
    \(\(-0.020088587942813804`\) + 0.0031811388662870395`\/ep\^2 - 
      0.004892692633587863`\/ep\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(MBintegrate[
      ser, {s \[Rule] \(-100\), t \[Rule] \(-3\), m \[Rule] 1}]\)], "Input"],

Cell[BoxData[
    \("Shifting contours..."\)], "Print"],

Cell["Performing 0 1-dimensional integrations", "Print"],

Cell[BoxData[
    \("Higher-dimensional integrals"\)], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart1ep0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]4\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart1ep0", " (dim ", 4, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart2ep0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]4\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart2ep0", " (dim ", 4, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart3ep0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart3ep0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart4ep0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart4ep0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart5ep0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart5ep0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart6ep0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart6ep0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart7ep0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart7ep0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart8ep0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart8ep0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart9ep0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart9ep0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart10ep0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart10ep0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart11ep0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart11ep0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart12ep0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart12ep0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart13ep0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart13ep0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart14ep0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart14ep0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart15ep0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart15ep0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart16ep0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart16ep0", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart17ep0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart17ep0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart18ep0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart18ep0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart19ep0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart19ep0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart20ep0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart20ep0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart21ep0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart21ep0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart22ep0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart22ep0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart23ep0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart23ep0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart24ep0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart24ep0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart25ep0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart25ep0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart26ep0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart26ep0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart27ep0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart27ep0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart28ep0"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart28ep0", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart29ep-1"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart29ep-1", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart30ep-1"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart30ep-1", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart31ep-1"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart31ep-1", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart32ep-1"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart32ep-1", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart33ep-1"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart33ep-1", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart34ep-1"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart34ep-1", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart35ep-1"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart35ep-1", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart36ep-1"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]3\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart36ep-1", " (dim ", 3, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart37ep-1"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart37ep-1", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart38ep-1"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart38ep-1", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart39ep-1"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart39ep-1", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart40ep-1"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart40ep-1", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart41ep-1"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart41ep-1", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart42ep-1"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart42ep-1", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart43ep-1"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart43ep-1", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart44ep-1"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart44ep-1", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart45ep-2"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart45ep-2", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart46ep-2"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart46ep-2", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart47ep-2"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart47ep-2", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart48ep-2"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart48ep-2", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart49ep-2"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart49ep-2", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart50ep-2"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart50ep-2", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart51ep-2"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart51ep-2", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Preparing "\[InvisibleSpace]"MBpart52ep-2"\
\[InvisibleSpace]" (dim "\[InvisibleSpace]2\[InvisibleSpace]")"\),
      SequenceForm[ "Preparing ", "MBpart52ep-2", " (dim ", 2, ")"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart1ep0"\),
      SequenceForm[ "Running ", "MBpart1ep0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart2ep0"\),
      SequenceForm[ "Running ", "MBpart2ep0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart3ep0"\),
      SequenceForm[ "Running ", "MBpart3ep0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart4ep0"\),
      SequenceForm[ "Running ", "MBpart4ep0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart5ep0"\),
      SequenceForm[ "Running ", "MBpart5ep0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart6ep0"\),
      SequenceForm[ "Running ", "MBpart6ep0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart7ep0"\),
      SequenceForm[ "Running ", "MBpart7ep0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart8ep0"\),
      SequenceForm[ "Running ", "MBpart8ep0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart9ep0"\),
      SequenceForm[ "Running ", "MBpart9ep0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart10ep0"\),
      SequenceForm[ "Running ", "MBpart10ep0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart11ep0"\),
      SequenceForm[ "Running ", "MBpart11ep0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart12ep0"\),
      SequenceForm[ "Running ", "MBpart12ep0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart13ep0"\),
      SequenceForm[ "Running ", "MBpart13ep0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart14ep0"\),
      SequenceForm[ "Running ", "MBpart14ep0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart15ep0"\),
      SequenceForm[ "Running ", "MBpart15ep0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart16ep0"\),
      SequenceForm[ "Running ", "MBpart16ep0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart17ep0"\),
      SequenceForm[ "Running ", "MBpart17ep0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart18ep0"\),
      SequenceForm[ "Running ", "MBpart18ep0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart19ep0"\),
      SequenceForm[ "Running ", "MBpart19ep0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart20ep0"\),
      SequenceForm[ "Running ", "MBpart20ep0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart21ep0"\),
      SequenceForm[ "Running ", "MBpart21ep0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart22ep0"\),
      SequenceForm[ "Running ", "MBpart22ep0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart23ep0"\),
      SequenceForm[ "Running ", "MBpart23ep0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart24ep0"\),
      SequenceForm[ "Running ", "MBpart24ep0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart25ep0"\),
      SequenceForm[ "Running ", "MBpart25ep0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart26ep0"\),
      SequenceForm[ "Running ", "MBpart26ep0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart27ep0"\),
      SequenceForm[ "Running ", "MBpart27ep0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart28ep0"\),
      SequenceForm[ "Running ", "MBpart28ep0"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart29ep-1"\),
      SequenceForm[ "Running ", "MBpart29ep-1"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart30ep-1"\),
      SequenceForm[ "Running ", "MBpart30ep-1"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart31ep-1"\),
      SequenceForm[ "Running ", "MBpart31ep-1"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart32ep-1"\),
      SequenceForm[ "Running ", "MBpart32ep-1"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart33ep-1"\),
      SequenceForm[ "Running ", "MBpart33ep-1"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart34ep-1"\),
      SequenceForm[ "Running ", "MBpart34ep-1"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart35ep-1"\),
      SequenceForm[ "Running ", "MBpart35ep-1"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart36ep-1"\),
      SequenceForm[ "Running ", "MBpart36ep-1"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart37ep-1"\),
      SequenceForm[ "Running ", "MBpart37ep-1"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart38ep-1"\),
      SequenceForm[ "Running ", "MBpart38ep-1"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart39ep-1"\),
      SequenceForm[ "Running ", "MBpart39ep-1"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart40ep-1"\),
      SequenceForm[ "Running ", "MBpart40ep-1"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart41ep-1"\),
      SequenceForm[ "Running ", "MBpart41ep-1"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart42ep-1"\),
      SequenceForm[ "Running ", "MBpart42ep-1"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart43ep-1"\),
      SequenceForm[ "Running ", "MBpart43ep-1"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart44ep-1"\),
      SequenceForm[ "Running ", "MBpart44ep-1"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart45ep-2"\),
      SequenceForm[ "Running ", "MBpart45ep-2"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart46ep-2"\),
      SequenceForm[ "Running ", "MBpart46ep-2"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart47ep-2"\),
      SequenceForm[ "Running ", "MBpart47ep-2"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart48ep-2"\),
      SequenceForm[ "Running ", "MBpart48ep-2"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart49ep-2"\),
      SequenceForm[ "Running ", "MBpart49ep-2"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart50ep-2"\),
      SequenceForm[ "Running ", "MBpart50ep-2"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart51ep-2"\),
      SequenceForm[ "Running ", "MBpart51ep-2"],
      Editable->False]], "Print"],

Cell[BoxData[
    InterpretationBox[\("Running "\[InvisibleSpace]"MBpart52ep-2"\),
      SequenceForm[ "Running ", "MBpart52ep-2"],
      Editable->False]], "Print"],

Cell[BoxData[
    \({\(-0.024657977598000015`\) + 0.0030850238199999994`\/ep\^2 - 
        0.004805724930000002`\/ep, {\(\(0.000011991651159980758`\)\(\
\[InvisibleSpace]\)\) + 4.660425957592778`*^-7\/ep\^2 + 
          2.0602063528768976`*^-6\/ep, 0}}\)], "Output"]
}, Open  ]]
},
FrontEndVersion->"5.1 for X",
ScreenRectangle->{{0, 1024}, {0, 768}},
WindowSize->{1014, 710},
WindowMargins->{{0, Automatic}, {0, Automatic}},
PrintingPageRange->{Automatic, Automatic},
PrintingOptions->{"PaperSize"->{612, 792},
"PaperOrientation"->"Portrait",
"PostScriptOutputFile":>FrontEnd`FileName[{$RootDirectory, "home", "gluza", \
"papers", "MBRbuilder", "examples"}, "warsawIteration.ps", CharacterEncoding \
-> "iso8859-1"],
"Magnification"->1}
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 40, 1, 27, "Input"],
Cell[1819, 56, 41, 1, 23, "Print"],
Cell[1863, 59, 51, 1, 23, "Print"],
Cell[1917, 62, 62, 1, 23, "Print"],
Cell[1982, 65, 58, 1, 23, "Print"]
}, Open  ]],

Cell[CellGroupData[{
Cell[2077, 71, 43, 1, 27, "Input"],
Cell[2123, 74, 56, 1, 23, "Print"],
Cell[2182, 77, 60, 1, 23, "Print"]
}, Open  ]],
Cell[2257, 81, 180, 3, 59, "Input"],
Cell[2440, 86, 363, 6, 75, "Input"],
Cell[2806, 94, 377, 6, 43, "Input"],

Cell[CellGroupData[{
Cell[3208, 104, 43, 1, 27, "Input"],
Cell[3254, 107, 143, 3, 23, "Print"],
Cell[3400, 112, 491, 14, 23, "Print"],
Cell[3894, 128, 143, 3, 23, "Print"],
Cell[4040, 133, 174, 3, 20, "Message"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4251, 141, 50, 1, 27, "Input"],
Cell[4304, 144, 264, 4, 23, "Print"],
Cell[4571, 150, 257, 5, 23, "Print"],
Cell[4831, 157, 50, 1, 23, "Print"],
Cell[4884, 160, 58, 1, 23, "Print"],
Cell[4945, 163, 50, 1, 23, "Print"],
Cell[4998, 166, 184, 3, 25, "Print"],
Cell[5185, 171, 215, 4, 23, "Print"],
Cell[5403, 177, 1248, 25, 106, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6688, 207, 43, 1, 27, "Input"],
Cell[6734, 210, 143, 3, 23, "Print"],
Cell[6880, 215, 739, 21, 23, "Print"],
Cell[7622, 238, 143, 3, 23, "Print"],
Cell[7768, 243, 174, 3, 20, "Message"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7979, 251, 62, 1, 27, "Input"],
Cell[8044, 254, 264, 4, 23, "Print"],
Cell[8311, 260, 257, 5, 23, "Print"],
Cell[8571, 267, 50, 1, 23, "Print"],
Cell[8624, 270, 58, 1, 23, "Print"],
Cell[8685, 273, 50, 1, 23, "Print"],
Cell[8738, 276, 89, 1, 25, "Print"],
Cell[8830, 279, 56, 1, 23, "Print"],
Cell[8889, 282, 1663, 26, 146, "Print"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10589, 313, 60, 1, 27, "Input"],
Cell[10652, 316, 273, 5, 23, "Print"],
Cell[10928, 323, 226, 4, 23, "Print"],
Cell[11157, 329, 141, 6, 83, "Print"],
Cell[11301, 337, 80, 1, 23, "Print"],
Cell[11384, 340, 466, 12, 23, "Print"],
Cell[11853, 354, 197, 4, 23, "Print"],
Cell[12053, 360, 1462, 23, 150, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[13552, 388, 206, 4, 27, "Input"],
Cell[13761, 394, 942, 13, 118, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[14740, 412, 80, 1, 27, "Input"],
Cell[14823, 415, 128, 2, 20, "Message"],
Cell[14954, 419, 36, 1, 27, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[15027, 425, 212, 4, 27, "Input"],
Cell[15242, 431, 1014, 14, 118, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[16293, 450, 131, 3, 27, "Input"],
Cell[16427, 455, 128, 2, 20, "Message"],
Cell[16558, 459, 307, 5, 44, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[16902, 469, 87, 1, 27, "Input"],
Cell[16992, 472, 42, 1, 23, "Print"],
Cell[17037, 475, 444, 11, 23, "Print"],
Cell[17484, 488, 135, 3, 23, "Print"],
Cell[17622, 493, 145, 3, 23, "Print"],
Cell[17770, 498, 159, 3, 23, "Print"]
}, Open  ]],

Cell[CellGroupData[{
Cell[17966, 506, 76, 1, 27, "Input"],
Cell[18045, 509, 358, 5, 20, "Message"],
Cell[18406, 516, 2450, 36, 255, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[20893, 557, 156, 3, 27, "Input"],
Cell[21052, 562, 365, 5, 20, "Message"],
Cell[21420, 569, 42, 1, 23, "Print"],
Cell[21465, 572, 439, 11, 23, "Print"],
Cell[21907, 585, 355, 8, 23, "Print"],
Cell[22265, 595, 419, 10, 23, "Print"],
Cell[22687, 607, 422, 10, 23, "Print"],
Cell[23112, 619, 422, 10, 23, "Print"],
Cell[23537, 631, 135, 3, 23, "Print"],
Cell[23675, 636, 145, 3, 23, "Print"],
Cell[23823, 641, 393, 9, 23, "Print"],
Cell[24219, 652, 358, 8, 23, "Print"],
Cell[24580, 662, 422, 10, 23, "Print"],
Cell[25005, 674, 145, 3, 23, "Print"],
Cell[25153, 679, 344, 8, 23, "Print"],
Cell[25500, 689, 145, 3, 23, "Print"],
Cell[25648, 694, 344, 8, 23, "Print"],
Cell[25995, 704, 145, 3, 23, "Print"],
Cell[26143, 709, 145, 3, 23, "Print"],
Cell[26291, 714, 135, 3, 23, "Print"],
Cell[26429, 719, 151, 3, 23, "Print"],
Cell[26583, 724, 321, 7, 23, "Print"],
Cell[26907, 733, 151, 3, 23, "Print"],
Cell[27061, 738, 294, 6, 23, "Print"],
Cell[27358, 746, 151, 3, 23, "Print"],
Cell[27512, 751, 393, 9, 23, "Print"],
Cell[27908, 762, 151, 3, 23, "Print"],
Cell[28062, 767, 294, 6, 23, "Print"],
Cell[28359, 775, 151, 3, 23, "Print"],
Cell[28513, 780, 294, 6, 23, "Print"],
Cell[28810, 788, 135, 3, 23, "Print"],
Cell[28948, 793, 157, 3, 23, "Print"],
Cell[29108, 798, 157, 3, 23, "Print"],
Cell[29268, 803, 157, 3, 23, "Print"],
Cell[29428, 808, 157, 3, 23, "Print"],
Cell[29588, 813, 157, 3, 23, "Print"],
Cell[29748, 818, 161, 3, 23, "Print"],
Cell[29912, 823, 42, 1, 23, "Print"],
Cell[29957, 826, 355, 8, 23, "Print"],
Cell[30315, 836, 419, 10, 23, "Print"],
Cell[30737, 848, 53, 1, 23, "Print"],
Cell[30793, 851, 373, 8, 23, "Print"],
Cell[31169, 861, 422, 10, 23, "Print"],
Cell[31594, 873, 135, 3, 23, "Print"],
Cell[31732, 878, 145, 3, 23, "Print"],
Cell[31880, 883, 344, 8, 23, "Print"],
Cell[32227, 893, 373, 8, 23, "Print"],
Cell[32603, 903, 355, 8, 23, "Print"],
Cell[32961, 913, 145, 3, 23, "Print"],
Cell[33109, 918, 344, 8, 23, "Print"],
Cell[33456, 928, 419, 10, 23, "Print"],
Cell[33878, 940, 145, 3, 23, "Print"],
Cell[34026, 945, 439, 11, 23, "Print"],
Cell[34468, 958, 419, 10, 23, "Print"],
Cell[34890, 970, 422, 10, 23, "Print"],
Cell[35315, 982, 422, 10, 23, "Print"],
Cell[35740, 994, 135, 3, 23, "Print"],
Cell[35878, 999, 151, 3, 23, "Print"],
Cell[36032, 1004, 294, 6, 23, "Print"],
Cell[36329, 1012, 355, 8, 23, "Print"],
Cell[36687, 1022, 151, 3, 23, "Print"],
Cell[36841, 1027, 344, 8, 23, "Print"],
Cell[37188, 1037, 151, 3, 23, "Print"],
Cell[37342, 1042, 151, 3, 23, "Print"],
Cell[37496, 1047, 294, 6, 23, "Print"],
Cell[37793, 1055, 151, 3, 23, "Print"],
Cell[37947, 1060, 344, 8, 23, "Print"],
Cell[38294, 1070, 151, 3, 23, "Print"],
Cell[38448, 1075, 393, 9, 23, "Print"],
Cell[38844, 1086, 358, 8, 23, "Print"],
Cell[39205, 1096, 422, 10, 23, "Print"],
Cell[39630, 1108, 151, 3, 23, "Print"],
Cell[39784, 1113, 344, 8, 23, "Print"],
Cell[40131, 1123, 151, 3, 23, "Print"],
Cell[40285, 1128, 151, 3, 23, "Print"],
Cell[40439, 1133, 135, 3, 23, "Print"],
Cell[40577, 1138, 157, 3, 23, "Print"],
Cell[40737, 1143, 355, 8, 23, "Print"],
Cell[41095, 1153, 157, 3, 23, "Print"],
Cell[41255, 1158, 157, 3, 23, "Print"],
Cell[41415, 1163, 294, 6, 23, "Print"],
Cell[41712, 1171, 157, 3, 23, "Print"],
Cell[41872, 1176, 157, 3, 23, "Print"],
Cell[42032, 1181, 294, 6, 23, "Print"],
Cell[42329, 1189, 157, 3, 23, "Print"],
Cell[42489, 1194, 321, 7, 23, "Print"],
Cell[42813, 1203, 157, 3, 23, "Print"],
Cell[42973, 1208, 294, 6, 23, "Print"],
Cell[43270, 1216, 157, 3, 23, "Print"],
Cell[43430, 1221, 393, 9, 23, "Print"],
Cell[43826, 1232, 157, 3, 23, "Print"],
Cell[43986, 1237, 294, 6, 23, "Print"],
Cell[44283, 1245, 135, 3, 23, "Print"],
Cell[44421, 1250, 163, 3, 23, "Print"],
Cell[44587, 1255, 163, 3, 23, "Print"],
Cell[44753, 1260, 163, 3, 23, "Print"],
Cell[44919, 1265, 163, 3, 23, "Print"],
Cell[45085, 1270, 163, 3, 23, "Print"],
Cell[45251, 1275, 163, 3, 23, "Print"],
Cell[45417, 1280, 163, 3, 23, "Print"],
Cell[45583, 1285, 161, 3, 23, "Print"],
Cell[45747, 1290, 54917, 856, 5612, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[100701, 2151, 103, 2, 27, "Input"],
Cell[100807, 2155, 110148, 1570, 16985, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[210992, 3730, 109, 2, 27, "Input"],
Cell[211104, 3734, 55, 1, 23, "Print"],
Cell[211162, 3737, 56, 0, 23, "Print"],
Cell[211221, 3739, 63, 1, 23, "Print"],
Cell[211287, 3742, 250, 4, 23, "Print"],
Cell[211540, 3748, 250, 4, 23, "Print"],
Cell[211793, 3754, 250, 4, 23, "Print"],
Cell[212046, 3760, 250, 4, 23, "Print"],
Cell[212299, 3766, 250, 4, 23, "Print"],
Cell[212552, 3772, 250, 4, 23, "Print"],
Cell[212805, 3778, 250, 4, 23, "Print"],
Cell[213058, 3784, 250, 4, 23, "Print"],
Cell[213311, 3790, 250, 4, 23, "Print"],
Cell[213564, 3796, 252, 4, 23, "Print"],
Cell[213819, 3802, 252, 4, 23, "Print"],
Cell[214074, 3808, 252, 4, 23, "Print"],
Cell[214329, 3814, 252, 4, 23, "Print"],
Cell[214584, 3820, 252, 4, 23, "Print"],
Cell[214839, 3826, 252, 4, 23, "Print"],
Cell[215094, 3832, 252, 4, 23, "Print"],
Cell[215349, 3838, 252, 4, 23, "Print"],
Cell[215604, 3844, 252, 4, 23, "Print"],
Cell[215859, 3850, 252, 4, 23, "Print"],
Cell[216114, 3856, 252, 4, 23, "Print"],
Cell[216369, 3862, 252, 4, 23, "Print"],
Cell[216624, 3868, 252, 4, 23, "Print"],
Cell[216879, 3874, 252, 4, 23, "Print"],
Cell[217134, 3880, 252, 4, 23, "Print"],
Cell[217389, 3886, 252, 4, 23, "Print"],
Cell[217644, 3892, 252, 4, 23, "Print"],
Cell[217899, 3898, 252, 4, 23, "Print"],
Cell[218154, 3904, 252, 4, 23, "Print"],
Cell[218409, 3910, 254, 4, 23, "Print"],
Cell[218666, 3916, 254, 4, 23, "Print"],
Cell[218923, 3922, 254, 4, 23, "Print"],
Cell[219180, 3928, 254, 4, 23, "Print"],
Cell[219437, 3934, 254, 4, 23, "Print"],
Cell[219694, 3940, 254, 4, 23, "Print"],
Cell[219951, 3946, 254, 4, 23, "Print"],
Cell[220208, 3952, 254, 4, 23, "Print"],
Cell[220465, 3958, 254, 4, 23, "Print"],
Cell[220722, 3964, 254, 4, 23, "Print"],
Cell[220979, 3970, 254, 4, 23, "Print"],
Cell[221236, 3976, 254, 4, 23, "Print"],
Cell[221493, 3982, 254, 4, 23, "Print"],
Cell[221750, 3988, 254, 4, 23, "Print"],
Cell[222007, 3994, 254, 4, 23, "Print"],
Cell[222264, 4000, 254, 4, 23, "Print"],
Cell[222521, 4006, 254, 4, 23, "Print"],
Cell[222778, 4012, 254, 4, 23, "Print"],
Cell[223035, 4018, 254, 4, 23, "Print"],
Cell[223292, 4024, 254, 4, 23, "Print"],
Cell[223549, 4030, 254, 4, 23, "Print"],
Cell[223806, 4036, 254, 4, 23, "Print"],
Cell[224063, 4042, 254, 4, 23, "Print"],
Cell[224320, 4048, 254, 4, 23, "Print"],
Cell[224577, 4054, 163, 3, 23, "Print"],
Cell[224743, 4059, 163, 3, 23, "Print"],
Cell[224909, 4064, 163, 3, 23, "Print"],
Cell[225075, 4069, 163, 3, 23, "Print"],
Cell[225241, 4074, 163, 3, 23, "Print"],
Cell[225407, 4079, 163, 3, 23, "Print"],
Cell[225573, 4084, 163, 3, 23, "Print"],
Cell[225739, 4089, 163, 3, 23, "Print"],
Cell[225905, 4094, 163, 3, 23, "Print"],
Cell[226071, 4099, 165, 3, 23, "Print"],
Cell[226239, 4104, 165, 3, 23, "Print"],
Cell[226407, 4109, 165, 3, 23, "Print"],
Cell[226575, 4114, 165, 3, 23, "Print"],
Cell[226743, 4119, 165, 3, 23, "Print"],
Cell[226911, 4124, 165, 3, 23, "Print"],
Cell[227079, 4129, 165, 3, 23, "Print"],
Cell[227247, 4134, 165, 3, 23, "Print"],
Cell[227415, 4139, 165, 3, 23, "Print"],
Cell[227583, 4144, 165, 3, 23, "Print"],
Cell[227751, 4149, 165, 3, 23, "Print"],
Cell[227919, 4154, 165, 3, 23, "Print"],
Cell[228087, 4159, 165, 3, 23, "Print"],
Cell[228255, 4164, 165, 3, 23, "Print"],
Cell[228423, 4169, 165, 3, 23, "Print"],
Cell[228591, 4174, 165, 3, 23, "Print"],
Cell[228759, 4179, 165, 3, 23, "Print"],
Cell[228927, 4184, 165, 3, 23, "Print"],
Cell[229095, 4189, 165, 3, 23, "Print"],
Cell[229263, 4194, 167, 3, 23, "Print"],
Cell[229433, 4199, 167, 3, 23, "Print"],
Cell[229603, 4204, 167, 3, 23, "Print"],
Cell[229773, 4209, 167, 3, 23, "Print"],
Cell[229943, 4214, 167, 3, 23, "Print"],
Cell[230113, 4219, 167, 3, 23, "Print"],
Cell[230283, 4224, 167, 3, 23, "Print"],
Cell[230453, 4229, 167, 3, 23, "Print"],
Cell[230623, 4234, 167, 3, 23, "Print"],
Cell[230793, 4239, 167, 3, 23, "Print"],
Cell[230963, 4244, 167, 3, 23, "Print"],
Cell[231133, 4249, 167, 3, 23, "Print"],
Cell[231303, 4254, 167, 3, 23, "Print"],
Cell[231473, 4259, 167, 3, 23, "Print"],
Cell[231643, 4264, 167, 3, 23, "Print"],
Cell[231813, 4269, 167, 3, 23, "Print"],
Cell[231983, 4274, 167, 3, 23, "Print"],
Cell[232153, 4279, 167, 3, 23, "Print"],
Cell[232323, 4284, 167, 3, 23, "Print"],
Cell[232493, 4289, 167, 3, 23, "Print"],
Cell[232663, 4294, 167, 3, 23, "Print"],
Cell[232833, 4299, 167, 3, 23, "Print"],
Cell[233003, 4304, 167, 3, 23, "Print"],
Cell[233173, 4309, 167, 3, 23, "Print"],
Cell[233343, 4314, 260, 4, 47, "Output"]
}, Open  ]],
Cell[233618, 4321, 375, 7, 99, "Input"],
Cell[233996, 4330, 73, 1, 27, "Input"],

Cell[CellGroupData[{
Cell[234094, 4335, 617, 11, 91, "Input"],
Cell[234714, 4348, 604, 10, 85, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[235355, 4363, 260, 4, 27, "Input"],
Cell[235618, 4369, 127, 2, 47, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[235782, 4376, 109, 2, 27, "Input"],
Cell[235894, 4380, 55, 1, 23, "Print"],
Cell[235952, 4383, 56, 0, 23, "Print"],
Cell[236011, 4385, 63, 1, 23, "Print"],
Cell[236077, 4388, 248, 4, 23, "Print"],
Cell[236328, 4394, 248, 4, 23, "Print"],
Cell[236579, 4400, 248, 4, 23, "Print"],
Cell[236830, 4406, 248, 4, 23, "Print"],
Cell[237081, 4412, 248, 4, 23, "Print"],
Cell[237332, 4418, 248, 4, 23, "Print"],
Cell[237583, 4424, 248, 4, 23, "Print"],
Cell[237834, 4430, 248, 4, 23, "Print"],
Cell[238085, 4436, 248, 4, 23, "Print"],
Cell[238336, 4442, 250, 4, 23, "Print"],
Cell[238589, 4448, 250, 4, 23, "Print"],
Cell[238842, 4454, 250, 4, 23, "Print"],
Cell[239095, 4460, 250, 4, 23, "Print"],
Cell[239348, 4466, 250, 4, 23, "Print"],
Cell[239601, 4472, 250, 4, 23, "Print"],
Cell[239854, 4478, 250, 4, 23, "Print"],
Cell[240107, 4484, 250, 4, 23, "Print"],
Cell[240360, 4490, 250, 4, 23, "Print"],
Cell[240613, 4496, 250, 4, 23, "Print"],
Cell[240866, 4502, 250, 4, 23, "Print"],
Cell[241119, 4508, 250, 4, 23, "Print"],
Cell[241372, 4514, 250, 4, 23, "Print"],
Cell[241625, 4520, 250, 4, 23, "Print"],
Cell[241878, 4526, 250, 4, 23, "Print"],
Cell[242131, 4532, 250, 4, 23, "Print"],
Cell[242384, 4538, 250, 4, 23, "Print"],
Cell[242637, 4544, 250, 4, 23, "Print"],
Cell[242890, 4550, 250, 4, 23, "Print"],
Cell[243143, 4556, 252, 4, 23, "Print"],
Cell[243398, 4562, 252, 4, 23, "Print"],
Cell[243653, 4568, 252, 4, 23, "Print"],
Cell[243908, 4574, 252, 4, 23, "Print"],
Cell[244163, 4580, 252, 4, 23, "Print"],
Cell[244418, 4586, 252, 4, 23, "Print"],
Cell[244673, 4592, 252, 4, 23, "Print"],
Cell[244928, 4598, 252, 4, 23, "Print"],
Cell[245183, 4604, 252, 4, 23, "Print"],
Cell[245438, 4610, 252, 4, 23, "Print"],
Cell[245693, 4616, 252, 4, 23, "Print"],
Cell[245948, 4622, 252, 4, 23, "Print"],
Cell[246203, 4628, 252, 4, 23, "Print"],
Cell[246458, 4634, 252, 4, 23, "Print"],
Cell[246713, 4640, 252, 4, 23, "Print"],
Cell[246968, 4646, 252, 4, 23, "Print"],
Cell[247223, 4652, 252, 4, 23, "Print"],
Cell[247478, 4658, 252, 4, 23, "Print"],
Cell[247733, 4664, 252, 4, 23, "Print"],
Cell[247988, 4670, 252, 4, 23, "Print"],
Cell[248243, 4676, 252, 4, 23, "Print"],
Cell[248498, 4682, 252, 4, 23, "Print"],
Cell[248753, 4688, 252, 4, 23, "Print"],
Cell[249008, 4694, 252, 4, 23, "Print"],
Cell[249263, 4700, 161, 3, 23, "Print"],
Cell[249427, 4705, 161, 3, 23, "Print"],
Cell[249591, 4710, 161, 3, 23, "Print"],
Cell[249755, 4715, 161, 3, 23, "Print"],
Cell[249919, 4720, 161, 3, 23, "Print"],
Cell[250083, 4725, 161, 3, 23, "Print"],
Cell[250247, 4730, 161, 3, 23, "Print"],
Cell[250411, 4735, 161, 3, 23, "Print"],
Cell[250575, 4740, 161, 3, 23, "Print"],
Cell[250739, 4745, 163, 3, 23, "Print"],
Cell[250905, 4750, 163, 3, 23, "Print"],
Cell[251071, 4755, 163, 3, 23, "Print"],
Cell[251237, 4760, 163, 3, 23, "Print"],
Cell[251403, 4765, 163, 3, 23, "Print"],
Cell[251569, 4770, 163, 3, 23, "Print"],
Cell[251735, 4775, 163, 3, 23, "Print"],
Cell[251901, 4780, 163, 3, 23, "Print"],
Cell[252067, 4785, 163, 3, 23, "Print"],
Cell[252233, 4790, 163, 3, 23, "Print"],
Cell[252399, 4795, 163, 3, 23, "Print"],
Cell[252565, 4800, 163, 3, 23, "Print"],
Cell[252731, 4805, 163, 3, 23, "Print"],
Cell[252897, 4810, 163, 3, 23, "Print"],
Cell[253063, 4815, 163, 3, 23, "Print"],
Cell[253229, 4820, 163, 3, 23, "Print"],
Cell[253395, 4825, 163, 3, 23, "Print"],
Cell[253561, 4830, 163, 3, 23, "Print"],
Cell[253727, 4835, 163, 3, 23, "Print"],
Cell[253893, 4840, 165, 3, 23, "Print"],
Cell[254061, 4845, 165, 3, 23, "Print"],
Cell[254229, 4850, 165, 3, 23, "Print"],
Cell[254397, 4855, 165, 3, 23, "Print"],
Cell[254565, 4860, 165, 3, 23, "Print"],
Cell[254733, 4865, 165, 3, 23, "Print"],
Cell[254901, 4870, 165, 3, 23, "Print"],
Cell[255069, 4875, 165, 3, 23, "Print"],
Cell[255237, 4880, 165, 3, 23, "Print"],
Cell[255405, 4885, 165, 3, 23, "Print"],
Cell[255573, 4890, 165, 3, 23, "Print"],
Cell[255741, 4895, 165, 3, 23, "Print"],
Cell[255909, 4900, 165, 3, 23, "Print"],
Cell[256077, 4905, 165, 3, 23, "Print"],
Cell[256245, 4910, 165, 3, 23, "Print"],
Cell[256413, 4915, 165, 3, 23, "Print"],
Cell[256581, 4920, 165, 3, 23, "Print"],
Cell[256749, 4925, 165, 3, 23, "Print"],
Cell[256917, 4930, 165, 3, 23, "Print"],
Cell[257085, 4935, 165, 3, 23, "Print"],
Cell[257253, 4940, 165, 3, 23, "Print"],
Cell[257421, 4945, 165, 3, 23, "Print"],
Cell[257589, 4950, 165, 3, 23, "Print"],
Cell[257757, 4955, 165, 3, 23, "Print"],
Cell[257925, 4960, 266, 4, 49, "Output"]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

